

#ifndef __CMD_PROCESS_H__
#define __CMD_PROCESS_H__


#ifdef __cplusplus
 extern "C" {
#endif


#include "cs32f10x.h"
#include <stdbool.h>
#include "hal_comm_usart.h"
#include "usart_interface.h"
#include "ram_interface.h"
#include "flash_interface.h"
#include "ob_interface.h"
#include "can_interface.h"


/* Bootloader parameter */
#define CHIP_ID                             "F103"
#define BOOT_VERSION                        "0001"

/* Memory address limit */
#define RAM_ADDR_START                      (uint32_t)0x20000200
#define RAM_ADDR_END                        (uint32_t)0x20004FFF
#define OB_ADDR_MASK                        (uint32_t)0xFFFFFFF0
#define FLASH_ADDR_MASK                     (uint32_t)0xFFFE0000

/* Communication interface command */
#define CSBOOT_COMMAND_GET_ID                (uint8_t)0x20
#define CSBOOT_COMMAND_GET_VERSION           (uint8_t)0x21
#define CSBOOT_COMMAND_WRITE_MEMORY          (uint8_t)0x23
#define CSBOOT_COMMAND_READ_MEMORY           (uint8_t)0x24
#define CSBOOT_COMMAND_ERASE_PAGE            (uint8_t)0x25
#define CSBOOT_COMMAND_ERASE_CHIP            (uint8_t)0x26
#define CSBOOT_COMMAND_GO                    (uint8_t)0x27
#define CSBOOT_COMMAND_WRITE_PROTECT         (uint8_t)0x28
#define CSBOOT_COMMAND_WRITE_UNPROTECT       (uint8_t)0x29
#define CSBOOT_COMMAND_READOUT_PROTECT       (uint8_t)0x2A
#define CSBOOT_COMMAND_READOUT_UNPROTECT     (uint8_t)0x2B

/* Command execute state */
#define CMD_EXECUTE_SUCCESS                 (uint8_t)0x00       /*!< Command execute success. */
#define CMD_EXECUTE_NOTSUPPORT              (uint8_t)0x01       /*!< Not support command. */
#define CMD_EXECUTE_PARAMETER_ERROR         (uint8_t)0x02       /*!< Command parameter error. */
#define CMD_EXECUTE_NOPERMISSION            (uint8_t)0x03       /*!< No permission command. */
#define CMD_EXECUTE_FAIL                    (uint8_t)0x04       /*!< Command execute fail. */


void csboot_command_proess(uint8_t packet_status);

void comm_timeout_check(void);

#ifdef __cplusplus
}
#endif


#endif
