
#include "cs32f10x_can.h"
#include "can_interface.h"
#include "common.h"
#include "config.h"
#include "string.h"


static error_status_t can_send_msg(uint8_t *msg, uint8_t len);


void USB_LP_CAN1_RX0_IRQHandler(void)
{
    can_rx_msg_t rx_msg;

    /* index start from comm_rx_buffer[1] */
    if(comm_rx_index == 0)
    {
        comm_rx_index = 1;
    }
    
    can_message_receive(CAN1, CAN_RECEIVE_FIFO_0,&rx_msg);
    if((rx_msg.id_select == CAN_ID_STANDARD) && (rx_msg.remote_txreq == CAN_RMOTE_TX_REQ_DATA))
    {
        if(comm_rx_index < COMM_BUFFER_SIZE)
        {
            memcpy((void *)&comm_rx_buffer[comm_rx_index], rx_msg.data, rx_msg.data_length);
            comm_rx_index += rx_msg.data_length;
        }
    }
}


/**@brief       Gets the data received by CAN.
 * 
 * @param[in]   None.
 * 
 * @return      The status of receive process.
 */
uint8_t can_data_receive(void)
{
    uint8_t index = 0;
    comm_status = COMM_PACKET_NONE;
    
    if(comm_data_flag)              /* Can frame data process */
    {
        if((systick_counter - comm_start_time) > COMM_TIMEOUT)
        {
            comm_rx_index = 0;
            comm_data_flag = 0;
            comm_status = COMM_PACKET_NONE;
        }
        
        if(comm_rx_index >= 2)
        {
            /* Receive one frame data. 
               (comm_rx_index - 1) means a actual can frame data lenth because comm_rx_index start from index 1.
                CAN checksum used to check frame end */
            if((comm_rx_index - 1) == (comm_rx_buffer[2] + 4))
            {
                comm_rx_index -= 1;

                for(index = 1; index < comm_rx_buffer[2] + 3; index++)
                {
                    comm_frame_check += comm_rx_buffer[index];
                }

                if(comm_frame_check == *(uint16_t *)&comm_rx_buffer[comm_rx_index - 1])
                {
                    comm_status = COMM_PACKET_ACK;
                }
                else
                {
                    comm_status = COMM_PACKET_ERROR_CHECK;
                }
            }

            /* Lenth check */
            if(comm_rx_buffer[2] == 0)
            {
                comm_status = COMM_PACKET_ERROR_LENGTH_ZERO;
            }
            if(comm_rx_buffer[2] > COMM_DATA_LENGTH_MAX)
            {
                comm_status = COMM_PACKET_ERROR_LENGTH_MAX;
            }
        }
    }
    else                            /* Received can frame */
    {
        if(comm_rx_index)
        {
            if(comm_rx_buffer[1] != COMM_FRAME_HEAD)
            {
                comm_status = COMM_PACKET_ERROR_HEAD;
            }
            comm_frame_check = 0;
            comm_start_time = systick_counter;
            
            /* Set data flag */
            comm_data_flag = 0xFF;
        }
    }
    
    if(comm_status != COMM_PACKET_NONE)
    {
        comm_rx_index = 0;
        comm_data_flag = 0;
    }
    
    return comm_status;
}


/**@brief       Sends the data by CAN.
 * 
 * @param[in]   None.
 * 
 * @return      None.
 */
void can_data_send(void)
{
    uint8_t tx_len_temp = 0;
    uint8_t integer = 0, remain = 0;
    
    comm_tx_index = 0;
    comm_frame_check = 0;
    comm_tx_len = comm_tx_buffer[2] + 3;
    
    tx_len_temp = comm_tx_len + 2;
    
    integer = tx_len_temp / 8;
    remain = tx_len_temp % 8;
    
    for(comm_tx_index = 0; comm_tx_index < comm_tx_len; comm_tx_index++)
    {
        comm_frame_check += comm_tx_buffer[comm_tx_index];
    }
    
    *(volatile uint16_t *)&comm_tx_buffer[comm_tx_len] = comm_frame_check;

    comm_tx_index = 0;
    comm_start_time = systick_counter;

    while((systick_counter - comm_start_time) < COMM_TIMEOUT)
    {
        while(integer)
        {
            can_send_msg((uint8_t *)&comm_tx_buffer[comm_tx_index], CAN_FRAME_LENTH_MAX);
            
            comm_tx_index += CAN_FRAME_LENTH_MAX;       // set tx index
            integer --;
            if(integer == 0)
            {
                break;
            }
        }
        
        if(remain == 0)
        {
            break;
        }
        else
        {
            can_send_msg((uint8_t *)&comm_tx_buffer[comm_tx_index], remain);
        }
        
        break;
    }
}


/**@brief       CAN send message.
 *
 * @param[in]   msg: The pointer to the data array to be send. The array has 8 elements at most.
 *
 * @param[in]   len: The byte length to be send
 *
 * @return      Status of function.
 */
static error_status_t can_send_msg(uint8_t *msg, uint8_t len)
{
    uint8_t mailbox;
    uint16_t i = 0;
    can_tx_msg_t tx_msg;
    
    tx_msg.std_id = CAN_TX_FRAME_ID;
    tx_msg.ext_id = 0;
    tx_msg.id_select = CAN_ID_STANDARD;
    tx_msg.remote_txreq = CAN_RMOTE_TX_REQ_DATA;
    tx_msg.data_length = len;
    
    for(i = 0; i< len; i++)
    {
        tx_msg.data[i] = msg[i];
    }
    
    mailbox = can_message_send(CAN1, &tx_msg);
    
    i = 0;  //i used for timeout
    while(can_tx_status_get(CAN1,mailbox) != 0x00)
    {
        i++;
        if(i == 0XFFF)
        {
            return ERROR;
        }
    }

    return SUCCESS;
}
