

#include "ob_interface.h"


/**@brief       Reads data at the specified option bytes address into the buffer.
 * 
 * @param[in]   addr: Specifies the start address of option bytes.
 * 
 * @param[in]   length: Specifies the length of data by byte to be read.
 * 
 * @param[out]  pdata: Points to the buffer storing option bytes data.
 * 
 * @return      The status of read process.
 */
uint8_t ob_read(uint32_t addr, uint8_t length, uint8_t *pdata)
{
    uint8_t index;
    
    for(index = 0; index < length; index++)
    {
        *(pdata + index) = *((uint8_t *)addr + index);
    }
    
    return CMD_EXECUTE_SUCCESS;
}


/**@brief       Writes data from the buffer to the specified option bytes address.
 * 
 * @param[in]   addr: Specifies the start address of option bytes.
 * 
 * @param[in]   length: Specifies the length of data by half-word to be write.
 * 
 * @param[in]   pdata: Points to the buffer storing source data in half-word.
 * 
 * @return      The status of write process.
 */
uint8_t ob_write(uint32_t addr, uint8_t length, uint16_t *pdata)
{
    uint8_t index;
    uint8_t status;
    uint32_t offset;
    uint16_t ob_buffer[8] __attribute((aligned (4)));
    
    *(uint32_t *)ob_buffer = *(uint32_t *)OB0_ADDRESS;
    *((uint32_t *)ob_buffer + 1) = *(uint32_t *)OB1_ADDRESS;
    *((uint32_t *)ob_buffer + 2) = *(uint32_t *)OB2_ADDRESS;
    *((uint32_t *)ob_buffer + 3) = *(uint32_t *)OB3_ADDRESS;
    
    offset = (addr - OB0_ADDRESS) >> 1;
    
    for(index = 0; index < length; index++)
    {
        ob_buffer[offset + index] = *(pdata + index);
    }
    
    status = flash_operation_wait(ERASE_TIMEOUT);
    
    if(status == CMD_EXECUTE_SUCCESS)
    {
        FLASH->CCODE = KEY1;
        FLASH->CCODE = KEY2;
        FLASH->OBCCODE = KEY1;
        FLASH->OBCCODE = KEY2;
        FLASH->CTR |= FMC_CTR_OBERS;
        FLASH->CTR |= FMC_CTR_START;
        
        status = flash_operation_wait(ERASE_TIMEOUT);
        FLASH->CTR &= ~(uint32_t)FMC_CTR_OBERS;
        
        if(status == CMD_EXECUTE_SUCCESS)
        {
            FLASH->CTR |= FMC_CTR_OBPG;
            for(index = 0; index < 8; index++)
            {
                *((uint16_t *)OB0_ADDRESS + index) = ob_buffer[index];
                
                status = flash_operation_wait(ERASE_TIMEOUT);
                if(status != CMD_EXECUTE_SUCCESS)
                {
                    break;
                }
            }
            FLASH->CTR &= ~(uint32_t)FMC_CTR_OBPG;
        }
        
        FLASH->CTR &= ~(uint32_t)FMC_CTR_OBWEN;
        FLASH->CTR |= FMC_CTR_LOCK;
    }
    
    return status;
}

