

#include "ram_interface.h"


/**@brief       Reads data at the specified SRAM address into the buffer.
 * 
 * @param[in]   addr: Specifies the start address of SRAM.
 * 
 * @param[in]   length: Specifies the length of data by byte to be read.
 * 
 * @param[out]  pdata: Points to the buffer storing flash data.
 * 
 * @return      The status of read process.
 */
uint8_t ram_read(uint32_t addr, uint8_t length, uint8_t *pdata)
{
    uint8_t index;
    
    for(index = 0; index < length; index++)
    {
        *(pdata + index) = *((uint8_t *)addr + index);
    }
    
    return CMD_EXECUTE_SUCCESS;
}


/**@brief       Writes data from the buffer to the specified SRAM address.
 * 
 * @param[in]   addr: Specifies the start address of SRAM.
 * 
 * @param[in]   length: Specifies the length of data by half-word to be write.
 * 
 * @param[in]   pdata: Points to the buffer storing source data in half-word.
 * 
 * @return      The status of write process.
 */
uint8_t ram_write(uint32_t addr, uint8_t length, uint16_t *pdata)
{
    uint8_t index;
    
    for(index = 0; index < length; index++)
    {
        *((uint16_t *)addr + index) = *(pdata + index);
    }
    
    return CMD_EXECUTE_SUCCESS;
}
