

#include "usart_interface.h"
#include "common.h"


void USART1_IRQHandler(void)
{
    if(USART1->STS & USART_STS_RXNE)
    {
        if(++comm_rx_index < COMM_BUFFER_SIZE)
        {
            comm_rx_buffer[comm_rx_index] = (uint8_t)USART1->DATA;
        }
        else
        {
            (void)USART1->DATA;
        }
    }
}


/**@brief       Gets the data received by USART.
 * 
 * @param[in]   None.
 * 
 * @return      The status of receive process.
 */
uint8_t usart_data_receive(void)
{
    uint8_t index = 0;
    comm_status = COMM_PACKET_NONE;
    
    if(comm_data_flag)
    {
        if((systick_counter - comm_start_time) > COMM_TIMEOUT)
        {
            comm_rx_index = 0;
            comm_data_flag = 0;
            comm_status = COMM_PACKET_NONE;
        }
        if(comm_rx_index >= 2)
        {
            if(comm_rx_index == (comm_rx_buffer[2] + 4))
            {
                for(index = 1; index < comm_rx_buffer[2] + 3; index++)
                {
                    comm_frame_check += comm_rx_buffer[index];
                }

                if(comm_frame_check == *(uint16_t *)&comm_rx_buffer[comm_rx_index - 1])
                {
                    comm_status = COMM_PACKET_ACK;
                }
                else
                {
                    comm_status = COMM_PACKET_ERROR_CHECK;
                }
            }
            if(!comm_rx_buffer[2])
            {
                comm_status = COMM_PACKET_ERROR_LENGTH_ZERO;
            }
            if(comm_rx_buffer[2] > COMM_DATA_LENGTH_MAX)
            {
                comm_status = COMM_PACKET_ERROR_LENGTH_MAX;
            }
        }
    }
    else
    {
        if(comm_rx_index)
        {
            if(comm_rx_buffer[1] != COMM_FRAME_HEAD)
            {
                comm_status = COMM_PACKET_ERROR_HEAD;
            }
            comm_frame_check = 0;
            comm_start_time = systick_counter;
            comm_data_flag = 0xFF;
        }
    }
    
    if(comm_status != COMM_PACKET_NONE)
    {
        comm_rx_index = 0;
        comm_data_flag = 0;
    }
    
    return comm_status;
}


/**@brief       Sends the data by USART.
 * 
 * @param[in]   None.
 * 
 * @return      None.
 */
void usart_data_send(void)
{
    comm_tx_index = 0;
    comm_frame_check = 0;
    comm_tx_len = comm_tx_buffer[2] + 3;
    
    comm_start_time = systick_counter;
    
    while((systick_counter - comm_start_time) < COMM_TIMEOUT)
    {
        if(USART1->STS & USART_STS_TXE)
        {
            USART1->DATA = comm_tx_buffer[comm_tx_index];
            
            if(comm_tx_index < comm_tx_len)
            {
                comm_frame_check += comm_tx_buffer[comm_tx_index];
            }
            
            *(volatile uint16_t *)&comm_tx_buffer[comm_tx_len] = comm_frame_check;
            
            if(comm_tx_index > comm_tx_len)
            {
                while(!(USART1->STS & USART_STS_TCF)){}
                break;
            }
            comm_tx_index++;
        }
    }
    return;
}
