

#include "hal_comm_can.h"
#include "cs32f10x_rcu.h"
#include "cs32f10x_gpio.h"
#include "cs32f10x_can.h"
#include "cs32f10x_misc.h"
#include "config.h"


static void sysclk_deinit(void);

/**@brief       CAN mode init and config.
 *
 * @param[in]   tc: time_change, Specifies the maximum number of time unit the CAN module
 *              could be changed lengther or shorter.
 *
 * @param[in]   ts1: Time segment 1.
 *
 * @param[in]   ts2: Time segment 2.
 *
 * @param[in]   brp: CAN baud rate prescaler division. 
 *              CAN_BAUD = pclk/((ts1+1 + ts2+1 +1)*brp)   36M/((8+9+1)*4)=500Kbps
 *
 * @param[in]   mode: CAN work mode.CAN_MODE_NORMAL or CAN_MODE_LOOPBACK in this case.
 *
 * @return      None.
 */
static void can_mode_config(uint32_t tc, uint32_t ts1, uint32_t ts2, uint32_t brp,uint32_t mode);


void hal_comm_can_deinit(void)
{
    NVIC_DisableIRQ(IRQn_USB_LP_CAN1_RX0);               //رUSART1ںж
    NVIC_ClearPendingIRQ(IRQn_USB_LP_CAN1_RX0);          //USART1ںжϹλ
    
    RCU->APB2RST = (uint32_t)0x00005E7D;       //λUSART1IOPAAFIO
    RCU->APB2RST = 0;
    RCU->APB1RST = (uint32_t)0x02000000;        // can reset
    RCU->APB1RST = 0;
    RCU->APB2EN &= ~(uint32_t)0x00004005;      //رUSART1IOPAAFIOʱ
    RCU->APB1EN &= ~(uint32_t)0x02000000;       // can clock close
    
    SysTick->CTRL = 0;                      //رSysTickʹܡжʹ
    SysTick->VAL = 0;                       //ֵ
    
    sysclk_deinit();
}


void hal_comm_can_init(void)
{
    FWDT->CCODE = 0x00005555;
    FWDT->PDIV = 0x00000007;
    FWDT->CCODE = 0x0000AAAA;
    
//    hal_comm_can_deinit();
    nvic_priority_group_config(NVIC_PriorityGroup_4);
    
    /* CAN init */
    can_mode_config(CAN_TIME_CHANGE_UNIT_1, CAN_TS1_UNIT_9, CAN_TS2_UNIT_8, 4, CAN_MODE_NORMAL);

    SysTick->LOAD = 72000 - 1;                  //1msʱ
    SysTick->VAL = 0;
    SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;
}


static void sysclk_deinit(void)
{
    RCU->CTR |= (uint32_t)0x00000001;        //HSI
    RCU->CFG &= (uint32_t)0xF8FF0000;      //λMCOADCPREPPRE1PPRE2HPRESWϵͳʱлHSI
    RCU->CTR &= (uint32_t)0xFEF6FFFF;        //رPLLHSEʱ
//    RCU->CTR &= (uint32_t)0xFFFBFFFF;        //λHSEBYP
    RCU->CFG &= (uint32_t)0xFF80FFFF;      //λCFGRĴʣbit
    RCU->INTR = (uint32_t)0x009F0000;        //жϱ־λ
}


/**@brief       CAN mode init and config.
 *
 * @param[in]   tc: time_change, Specifies the maximum number of time unit the CAN module
 *              could be changed lengther or shorter.
 *
 * @param[in]   ts1: Time segment 1.
 *
 * @param[in]   ts2: Time segment 2.
 *
 * @param[in]   brp: CAN baud rate prescaler division. 
 *              CAN_BAUD = pclk/((ts1+1 + ts2+1 +1)*brp)   36M/((8+9+1)*4)=500Kbps
 *
 * @param[in]   mode: CAN work mode.CAN_MODE_NORMAL or CAN_MODE_LOOPBACK in this case.
 *
 * @return      None.
 */
static void can_mode_config(uint32_t tc, uint32_t ts1, uint32_t ts2, uint32_t brp,uint32_t mode)
{
    can_config_t can_config = {0};
    can_filter_config_t can_filter = {0};
    nvic_init_t nvic_struct ;
    
    __RCU_APB1_CLK_ENABLE(RCU_APB1_PERI_CAN1);
    __RCU_APB2_CLK_ENABLE(RCU_APB2_PERI_GPIOA);
    
    /* PA11 RX */
    gpio_mode_config(GPIOA, GPIO_PIN_11, GPIO_MODE_IN_PU);
    /* PA12 TX */
    gpio_mode_config(GPIOA, GPIO_PIN_12, GPIO_MODE_OUT_AFPP(GPIO_SPEED_HIGH));
    
    /* CAN config */
    can_config.time_trigger = DISABLE;
    can_config.auto_busoff = DISABLE;
    can_config.auto_active = DISABLE;
    can_config.no_retransmit = ENABLE;
    can_config.rx_overwrite = DISABLE;
    can_config.tx_priority = DISABLE;
    can_config.can_mode = mode;
    can_config.time_change = tc;
    can_config.time_seg1 = ts1;
    can_config.time_seg2 = ts2;
    can_config.baudrate_pdiv = brp;
    
    can_init(CAN1, &can_config);
    
    /* Filter config */
    can_filter.filter_fifo_connect = CAN_FILTER_CONNECT_FIFO0;  
    can_filter.filter_group = 0;    //filter 0
    can_filter.filter_mode = CAN_FILTER_MODE_IDENTIFY_MASK;
    can_filter.filter_identifier = (uint32_t)CAN_RX_FRAME_ID << (16 + 5) | CAN_ID_STANDARD;
    can_filter.filter_width = CAN_FILTER_WIDTH_32BIT;
    can_filter.filter_mask = (uint32_t)0x7FF << (16 + 5) | CAN_ID_STANDARD;
    can_filter.filter_active = ENABLE;
    
    can_filter_init(&can_filter);
    
    /* NVIC config */
    __CAN_INTR_ENABLE(CAN1, CAN_INTR_FF0_MSG_PENDING);
    nvic_struct.nvic_irqchannel = IRQn_USB_LP_CAN1_RX0;
    nvic_struct.nvic_irq_enable = ENABLE;
    nvic_struct.nvic_irq_pre_priority = 1;
    nvic_struct.nvic_irq_sub_priority = 0;
    nvic_init(&nvic_struct);
}
