/***************************************************************************//**
 * @file     cs32f10x.h
 * @version  V1.0.0
 * @author   Software Development
 * @brief    This file include all peripherals registers and bits definitions
 * 
 * @copyright Copyright (C) Software Development. All rights reserved.
 *****************************************************************************/


#ifndef __CS32F10X_H__
#define __CS32F10X_H__


/*****************************************************************************
 * @includes 
*****************************************************************************/
#include <stdint.h>
#include "cs32f10x_conf.h"
#include "RTE_Components.h"             /* Component selection */


#ifdef __cplusplus
 extern "C" {
#endif 




/*****************************************************************************
 * @definitions 
*****************************************************************************/
#if !defined (CS32F103xB)
 #error "Please select first the target CS32F10x device used in your application (in cs32f10x.h file)"
#endif

/**
  * @brief the value of External High Speed osillator (HXT) used in your applicaton
  */
#if !defined  HXT_VALUE
  #define HXT_VALUE    ((uint32_t)8000000) /*!< External oscillator in 8MHz */
#endif

/**
  * @brief  the External High Speed oscillator(HXT) Startup Timeout value
  */
#if !defined  (HXT_STARTUP_TIMEOUT)
    #define HXT_STARTUP_TIMEOUT     ((uint16_t)0x5000)  /*!< Time out for HXT start. */
#endif

#if !defined  (HRC_VALUE)
    #define HRC_VALUE               ((uint32_t)8000000) /*!< Value of the Internal high Speed oscillator in Hz. */
#endif

/**
  * @brief the Internal High Speed oscillator HRC) Startup Timeout value
  */
#if !defined  (HRC_STARTUP_TIMEOUT)
    #define HRC_STARTUP_TIMEOUT     ((uint16_t)05000)   /*!< Time out for HRC start. */
#endif

/**
 * @brief CS32F10x Standard Peripheral Library version number V2.0.1
   */
#define __CS32F10X_STDPERIPH_VERSION_MAIN   (0x01) /*!< [31:24] main version */                                  
#define __CS32F10X_STDPERIPH_VERSION_SUB1   (0x01) /*!< [23:16] sub1 version */
#define __CS32F10X_STDPERIPH_VERSION_SUB2   (0x00) /*!< [15:8]  sub2 version */
#define __CS32F10X_STDPERIPH_VERSION_RC     (0x00) /*!< [7:0]  release candidate */ 
#define __CS32F10X_STDPERIPH_VERSION       ( (__CS32F10X_STDPERIPH_VERSION_MAIN << 24)\
                                             |(__CS32F10X_STDPERIPH_VERSION_SUB1 << 16)\
                                             |(__CS32F10X_STDPERIPH_VERSION_SUB2 << 8)\
                                             |(__CS32F10X_STDPERIPH_VERSION_RC))



/** @addtogroup Configuration_section_for_CMSIS
  * @{
  */

/**
 * @brief Configuration of the Cortex-M3 Processor and Core Peripherals 
 */
#define __NVIC_PRIO_BITS          4 /*!< CS32F103 uses 4 Bits for the Priority Levels    */
#define __Vendor_SysTickConfig    0 /*!< Set to 1 if different SysTick Config is used */

/**
 * @brief CS32F10x Interrupt Number Definition, according to the selected device 
 *        in @ref Library_configuration_section 
 */
typedef enum IRQn
{
  NonMaskableInt_IRQn         = -14,    /*!< 2 Non Maskable Interrupt                             */
  MemoryManagement_IRQn       = -12,    /*!< 4 Cortex-M3 Memory Management Interrupt              */
  BusFault_IRQn               = -11,    /*!< 5 Cortex-M3 Bus Fault Interrupt                      */
  UsageFault_IRQn             = -10,    /*!< 6 Cortex-M3 Usage Fault Interrupt                    */
  SVCall_IRQn                 = -5,     /*!< 11 Cortex-M3 SV Call Interrupt                       */
  DebugMonitor_IRQn           = -4,     /*!< 12 Cortex-M3 Debug Monitor Interrupt                 */
  PendSV_IRQn                 = -2,     /*!< 14 Cortex-M3 Pend SV Interrupt                       */
  SysTick_IRQn                = -1,     /*!< 15 Cortex-M3 System Tick Interrupt                   */

  IRQn_WWDT                   = 0,      /*!< Window WatchDog Interrupt                            */
  IRQn_PVD                    = 1,      /*!< PVD through EXTI Line detection Interrupt            */
  IRQn_TAMPER                 = 2,      /*!< Tamper Interrupt                                     */
  IRQn_RTC                    = 3,      /*!< RTC global Interrupt                                 */
  IRQn_FLASH                  = 4,      /*!< FLASH global Interrupt                               */
  IRQn_RCU                    = 5,      /*!< RCU global Interrupt                                 */
  IRQn_EXTI0                  = 6,      /*!< EXTI Line0 Interrupt                                 */
  IRQn_EXTI1                  = 7,      /*!< EXTI Line1 Interrupt                                 */
  IRQn_EXTI2                  = 8,      /*!< EXTI Line2 Interrupt                                 */
  IRQn_EXTI3                  = 9,      /*!< EXTI Line3 Interrupt                                 */
  IRQn_EXTI4                  = 10,     /*!< EXTI Line4 Interrupt                                 */
  IRQn_DMA1_Channel1          = 11,     /*!< DMA1 Channel 1 global Interrupt                      */
  IRQn_DMA1_Channel2          = 12,     /*!< DMA1 Channel 2 global Interrupt                      */
  IRQn_DMA1_Channel3          = 13,     /*!< DMA1 Channel 3 global Interrupt                      */
  IRQn_DMA1_Channel4          = 14,     /*!< DMA1 Channel 4 global Interrupt                      */
  IRQn_DMA1_Channel5          = 15,     /*!< DMA1 Channel 5 global Interrupt                      */
  IRQn_DMA1_Channel6          = 16,     /*!< DMA1 Channel 6 global Interrupt                      */
  IRQn_DMA1_Channel7          = 17,     /*!< DMA1 Channel 7 global Interrupt                      */
  IRQn_ADC1_2                 = 18,     /*!< ADC1 and ADC2 global Interrupt                       */
  IRQn_USB_HP_CAN1_TX         = 19,     /*!< USB Device High Priority or CAN1 TX Interrupts       */
  IRQn_USB_LP_CAN1_RX0        = 20,     /*!< USB Device Low Priority or CAN1 RX0 Interrupts       */
  IRQn_CAN1_RX1               = 21,     /*!< CAN1 RX1 Interrupt                                   */
  IRQn_CAN1_SCE               = 22,     /*!< CAN1 SCE Interrupt                                   */
  IRQn_EXTI9_5                = 23,     /*!< External Line[9:5] Interrupts                        */
  IRQn_TIM1_BRK               = 24,     /*!< TIM1 Break Interrupt                                 */
  IRQn_TIM1_UP                = 25,     /*!< TIM1 Update Interrupt                                */
  IRQn_TIM1_TRG_COM           = 26,     /*!< TIM1 Trigger and Commutation Interrupt               */
  IRQn_TIM1_CC                = 27,     /*!< TIM1 Capture Compare Interrupt                       */
  IRQn_TIM2                   = 28,     /*!< TIM2 global Interrupt                                */
  IRQn_TIM3                   = 29,     /*!< TIM3 global Interrupt                                */
  IRQn_TIM4                   = 30,     /*!< TIM4 global Interrupt                                */
  IRQn_I2C1_EV                = 31,     /*!< I2C1 Event Interrupt                                 */
  IRQn_I2C1_ER                = 32,     /*!< I2C1 Error Interrupt                                 */
  IRQn_I2C2_EV                = 33,     /*!< I2C2 Event Interrupt                                 */
  IRQn_I2C2_ER                = 34,     /*!< I2C2 Error Interrupt                                 */
  IRQn_SPI1                   = 35,     /*!< SPI1 global Interrupt                                */
  IRQn_SPI2                   = 36,     /*!< SPI2 global Interrupt                                */
  IRQn_USART1                 = 37,     /*!< USART1 global Interrupt                              */
  IRQn_USART2                 = 38,     /*!< USART2 global Interrupt                              */
  IRQn_USART3                 = 39,     /*!< USART3 global Interrupt                              */
  IRQn_EXTI15_10              = 40,     /*!< External Line[15:10] Interrupts                      */
  IRQn_RTCAlarm               = 41,     /*!< RTC Alarm through EXTI Line Interrupt                */
  IRQn_USBWakeUp              = 42      /*!< USB Device WakeUp from suspend through EXTI Line Interrupt */  

} IRQn_Type;

/**
  * @}
  */

#include "core_cm3.h"
#include "system_cs32f10x.h"
#include <stdint.h>



/**
  * @enum error_status_t
  * @brief Error status definition
  */
typedef enum {
    ERROR   = 0,
    SUCCESS = !ERROR
} error_status_t;

/**
  * @enum bit_status_t
  * @brief bit flag set or status get definitoin
  */
typedef enum {
    RESET   = 0,
    SET     = !RESET
} bit_status_t;

/**
  * @enum enable_state_t
  * @brief enable or disable  definition
  */
typedef enum {
    DISABLE   = 0,
    ENABLE    = !DISABLE
} enable_state_t;

/**
  * @brief Check enable  definition
  */
#define FUNCTIONAL_STATE_CHECK(STATE)          (((STATE) == DISABLE) || ((STATE == ENABLE)))

/**
  * @brief Register modify macro
  */
#define MODIFY_REG(REG, CLEARMASK, SETMASK) ((REG) = ((REG) & (~(CLEARMASK))) | (SETMASK))


/** @addtogroup Peripheral_registers_structures
  * @{
  */   

/** 
  * @brief Analog to Digital Registers
  */

typedef struct
{
  __IO uint32_t STS;                /*!< ADC status register. */
  __IO uint32_t CTR1;               /*!< ADC control register 1. */
  __IO uint32_t CTR;                /*!< ADC general control register. */
  __IO uint32_t SMPR1;              /*!< ADC sample time register 1. */
  __IO uint32_t SMPR2;              /*!< ADC sample time register 2. */
  __IO uint32_t JOFT[4];            /*!< ADC injected channel data offset register. */
  __IO uint32_t WDTH;               /*!< Analog watchdog high threshold register. */
  __IO uint32_t WDTL;               /*!< Analog watchdog lower threshold register. */
  __IO uint32_t SQR1;               /*!< ADC regular sequence register 1. */
  __IO uint32_t SQR2;               /*!< ADC regular sequence register 2. */
  __IO uint32_t SQR3;               /*!< ADC regular sequence register 3. */
  __IO uint32_t JSQR;               /*!< ADC injected sequence register 1. */
  __IO uint32_t JOUTDAT[4];         /*!< ADC injected data register. */
  __IO uint32_t OUTDAT;             /*!< ADC conversion output data register. */
} adc_reg_t;

/** 
  * @brief Backup Registers  
  */

typedef struct
{
       uint32_t RESERVED0;          /*!< Reserved 0x00~0x03. */
  __IO uint32_t AOX[10];            /*!< RTC always on register 1 ~ 10. */
  __IO uint32_t CCR;                /*!< RTC clock calibration register. */
  __IO uint32_t CTR;                /*!< RTC backup control register. */
  __IO uint32_t CSR;                /*!< RTC backup control/status register. */
       uint32_t RESERVED1[2];       /*!< Reserved 0x38 ~ 0x3F. */
} bkp_reg_t;
  
/** 
  * @brief Controller Area Network TxMailBox 
  */

typedef struct
{
  __IO uint32_t TXMI;                   /*!< CAN TX mailbox identifier register */
  __IO uint32_t TXLT;                   /*!< CAN mailbox data length control and time stamp register */
  __IO uint32_t TXDL;                   /*!< CAN transmite data low register */
  __IO uint32_t TXDH;                   /*!< CAN transmite data high register */
} can_tx_mailbox_reg_t;

/** 
  * @brief Controller Area Network FIFOMailBox 
  */
  
typedef struct
{
  __IO uint32_t RFFID;                  /*!< CAN receive FIFO mailbox identifier register */
  __IO uint32_t RXLT;                   /*!< CAN receive FIFO mailbox data length control and time stamp register */
  __IO uint32_t RXDL;                   /*!< CAN receive FIFO data low register */
  __IO uint32_t RXDH;                   /*!< CAN receive FIFO data high register */
} can_rx_fifo_mailbox_reg_t;

/** 
  * @brief Controller Area Network FilterRegister 
  */
  
typedef struct
{
  __IO uint32_t FR1;
  __IO uint32_t FR2;
} can_filter_reg_t;

/** 
  * @brief Controller Area Network 
  */
  
typedef struct
{
  __IO uint32_t MCTR;                           /*!< CAN master control register */
  __IO uint32_t MSTS;                           /*!< CAN master status register */
  __IO uint32_t TXSTS;                          /*!< CAN transmit status register */
  __IO uint32_t RXFF0;                          /*!< CAN receive FIFO 0 register */
  __IO uint32_t RXFF1;                          /*!< CAN receive FIFO 1 register */
  __IO uint32_t INTR;                           /*!< CAN interrupt enable register */
  __IO uint32_t ERRSTS;                         /*!< CAN error status register */
  __IO uint32_t BTM;                            /*!< CAN bit timing register */
  uint32_t  RESERVED0[88];
  can_tx_mailbox_reg_t TX_MAILBOX[3];
  can_rx_fifo_mailbox_reg_t RX_FFMAILBOX[2];
  uint32_t  RESERVED1[12];
  __IO uint32_t FLTM;                           /*!< CAN filter master register */
  __IO uint32_t FMOD;                           /*!< CAN filter mode register */
  uint32_t  RESERVED2;
  __IO uint32_t FBW;                            /*!< CAN filter bit width */
  uint32_t  RESERVED3;
  __IO uint32_t FLTFA;                          /*!< CAN filter assign with  FIFO */
  uint32_t  RESERVED4;
  __IO uint32_t FAS;                            /*!< Filter activation settings */
  uint32_t  RESERVED5[8];
  can_filter_reg_t FILTER_REG[14];              /*!< Filter bank i register x */
} can_reg_t;

/** 
  * @brief CRC calculation unit 
  */

typedef struct
{
  __IO uint32_t DATA;
  __IO uint8_t  FREDATA;
  uint8_t   RESERVED0;
  uint16_t  RESERVED1;
  __IO uint32_t CTR;
} crc_reg_t;

/** 
  * @brief Debug MCU
  */

typedef struct
{
  __IO uint32_t ID;
  __IO uint32_t LPWCFG; 
}debug_reg_t;

/**
  * @brief DMA channel register structure defiition
  */
typedef struct
{
    __IO    uint32_t CHxCTR;        /*!< DMA channel x controller register  */
    __IO    uint32_t CHxNUM;        /*!< Transfer data number register of DMA cannel x  */
    __IO    uint32_t CHxPA;         /*!< Peripheral address register of DMA chanel x  */
    __IO    uint32_t CHxMA;         /*!< Memory address register of DMA channelx */
} dma_channel_reg_t;

/**
  * @brief DMA  register structure definition
  */
typedef struct {
    __IO    uint32_t STS;           /*!< DMA interrupt status register  */
    __IO    uint32_t INTFC;         /*!< DMA interrupt flag clear register  */
} dma_reg_t;

/** 
  * @brief Ethernet MAC
  */

typedef struct
{
  __IO uint32_t MACCR;
  __IO uint32_t MACFFR;
  __IO uint32_t MACHTHR;
  __IO uint32_t MACHTLR;
  __IO uint32_t MACMIIAR;
  __IO uint32_t MACMIIDR;
  __IO uint32_t MACFCR;
  __IO uint32_t MACVLANTR;             /*    8 */
       uint32_t RESERVED0[2];
  __IO uint32_t MACRWUFFR;             /*   11 */
  __IO uint32_t MACPMTCSR;
       uint32_t RESERVED1[2];
  __IO uint32_t MACSR;                 /*   15 */
  __IO uint32_t MACIMR;
  __IO uint32_t MACA0HR;
  __IO uint32_t MACA0LR;
  __IO uint32_t MACA1HR;
  __IO uint32_t MACA1LR;
  __IO uint32_t MACA2HR;
  __IO uint32_t MACA2LR;
  __IO uint32_t MACA3HR;
  __IO uint32_t MACA3LR;               /*   24 */
       uint32_t RESERVED2[40];
  __IO uint32_t MMCCR;                 /*   65 */
  __IO uint32_t MMCRIR;
  __IO uint32_t MMCTIR;
  __IO uint32_t MMCRIMR;
  __IO uint32_t MMCTIMR;               /*   69 */
       uint32_t RESERVED3[14];
  __IO uint32_t MMCTGFSCCR;            /*   84 */
  __IO uint32_t MMCTGFMSCCR;
       uint32_t RESERVED4[5];
  __IO uint32_t MMCTGFCR;
       uint32_t RESERVED5[10];
  __IO uint32_t MMCRFCECR;
  __IO uint32_t MMCRFAECR;
       uint32_t RESERVED6[10];
  __IO uint32_t MMCRGUFCR;
       uint32_t RESERVED7[334];
  __IO uint32_t PTPTSCR;
  __IO uint32_t PTPSSIR;
  __IO uint32_t PTPTSHR;
  __IO uint32_t PTPTSLR;
  __IO uint32_t PTPTSHUR;
  __IO uint32_t PTPTSLUR;
  __IO uint32_t PTPTSAR;
  __IO uint32_t PTPTTHR;
  __IO uint32_t PTPTTLR;
       uint32_t RESERVED8[567];
  __IO uint32_t DMABMR;
  __IO uint32_t DMATPDR;
  __IO uint32_t DMARPDR;
  __IO uint32_t DMARDLAR;
  __IO uint32_t DMATDLAR;
  __IO uint32_t DMASR;
  __IO uint32_t DMAOMR;
  __IO uint32_t DMAIER;
  __IO uint32_t DMAMFBOCR;
       uint32_t RESERVED9[9];
  __IO uint32_t DMACHTDR;
  __IO uint32_t DMACHRDR;
  __IO uint32_t DMACHTBAR;
  __IO uint32_t DMACHRBAR;
} ETH_TypeDef;

/** 
  * @brief External Interrupt/Event Controller
  */

typedef struct
{
    __IO    uint32_t INTEN;
    __IO    uint32_t EVTEN;
    __IO    uint32_t RTEN;
    __IO    uint32_t FTEN;
    __IO    uint32_t SWTIEN;
    __IO    uint32_t PDF;
} exti_reg_t;

/**
  * @struct fmc_reg_t
  * @brief FMC (FLASH Memory Control )Register structure definition
  */
typedef struct {
    __IO    uint32_t WCR;          //!< FMC Wait control register
    __IO    uint32_t CCODE;        //!< FMC Control code register
    __IO    uint32_t OBCCODE;      //!< FMC Option Byte Control code register
    __IO    uint32_t STS;          //!< FMC status register
    __IO    uint32_t CTR;          //!< FMC Control register
    __IO    uint32_t ERSADR;       //!< Flash erase address register,
    __IO    uint32_t RESERVED;     //!< Reserved
    __IO    uint32_t OBSTS;        //!< Option byte status register
    __IO    uint32_t WPSTS;        //!< Write protection status register
} fmc_reg_t;

/**
  * @struct ob_reg_t
  * @brief Option Bytes Registers structure deinition
  */
typedef struct
{
    __IO uint16_t RPROT;          /*!< FLASH option byte Read protection,               Address offset: 0x00 */
    __IO uint16_t USER;           /*!< FLASH option byte user options,                  Address offset: 0x02 */
    __IO uint16_t DATA0;          /*!< User data byte 0 (stored in FLASH_OBSTS[23:16]), Address offset: 0x04 */
    __IO uint16_t DATA1;          /*!< User data byte 1 (stored in FLASH_OBSTS[31:24]), Address offset: 0x06 */
    __IO uint16_t WPROTOB0;       /*!< FLASH option byte write protection 0,            Address offset: 0x08 */
    __IO uint16_t WPROTOB1;       /*!< FLASH option byte write protection 1,            Address offset: 0x0A */
    __IO uint16_t WPROTOB2;       /*!< FLASH option byte write protection 2,            Address offset: 0x0C */
    __IO uint16_t WPROTOB3;       /*!< FLASH option byte write protection 3,            Address offset: 0x0E */
} ob_reg_t;

/** 
  * @brief General Purpose I/O
  */

typedef struct
{
  __IO uint32_t CFGL;               /*!< GPIO configuration register low */
  __IO uint32_t CFGH;               /*!< GPIO configuration register high */
  __IO uint32_t DI;                 /*!< GPIO port data input register */
  __IO uint32_t DO;                 /*!< GPIO port data output register */
  __IO uint32_t SCR;                /*!< GPIO port set/clear register */
  __IO uint32_t CLRR;               /*!< GPIO port clear register */
  __IO uint32_t LOCK;               /*!< GPIO port lock register */
} gpio_reg_t;

/** 
  * @brief Alternate Function I/O
  */

typedef struct
{
  __IO uint32_t ECR;
  __IO uint32_t MP;
  __IO uint32_t EXTCFG[4];
} afio_reg_t;

/**
  * @struct fwdt_reg_t* @brief FWDT(Free Watchdog Timer) Registersstructure definition
  */
typedef struct {
    __IO    uint32_t CCODE;   //!< FWDT Control code register
    __IO    uint32_t PDIV;    //!< FWDT Pre-divider register
    __IO    uint32_t UVAL;    //!< FWDT Update register
    __IO    uint32_t STS;     //!< FWDT Status register
} fwdt_reg_t;

/** 
  * @brief Reset and Clock Control Registers sructure definition
  */

typedef struct
{
    __IO uint32_t CTR;
    __IO uint32_t CFG;
    __IO uint32_t INTR;
    __IO uint32_t APB2RST;
    __IO uint32_t APB1RST;
    __IO uint32_t AHBEN;
    __IO uint32_t APB2EN;
    __IO uint32_t APB1EN;
    __IO uint32_t VBDC;
    __IO uint32_t STS;
} rcu_reg_t;

/** 
  * @brief Real-Time Clock Registers
  */

typedef struct
{
  __IO uint32_t ICTR;               /*!< RTC interrupt control register. */
  __IO uint32_t STS;                /*!< RTC status register. */
  __IO uint32_t PDIVLH;             /*!< RTC pre-divider load register high. */
  __IO uint32_t PDIVLL;             /*!< RTC pre-divider load register low. */
  __IO uint32_t PDIVH;              /*!< RTC pre-divider value register high. */
  __IO uint32_t PDIVL;              /*!< RTC pre-divider value register low. */
  __IO uint32_t CNTRH;              /*!< RTC counter value register high. */
  __IO uint32_t CNTRL;              /*!< RTC counter value register low. */
  __IO uint32_t ALRH;               /*!< RTC alarm value register high. */
  __IO uint32_t ALRL;               /*!< RTC alarm value register low. */
} rtc_reg_t;


/** 
  * @brief PMU register
  */
typedef struct
{
  __IO uint32_t CTR;
  __IO uint32_t CS;
} pmu_reg_t;

/** 
  * @brief Serial Peripheral Interface
  */

typedef struct {
    __IO    uint16_t CTR1;          /*!< SPI Control register 1  */
            uint16_t RESERVED0;     /*!< Reserved  */
    __IO    uint16_t CTR2;          /*!< SPI Control register 2  */
            uint16_t RESERVED1;     /*!< Reserved */
    __IO    uint16_t STS;           /*!< SPI Status register  */
            uint16_t RESERVED2;     /*!< Reserved */
    __IO    uint16_t DATA;          /*!< SPI data register */
            uint16_t RESERVED3;     /*!< Reserved */
    __IO    uint16_t CRCPOLYR;      /*!< SPI CRC polynomial register */
            uint16_t RESERVED4;     /*!< Reserved */
    __IO    uint16_t RCRC;          /*!< SPI Rx CRC register */
            uint16_t RESERVED5;     /*!< Reserved  */
    __IO    uint16_t TCRC;          /*!< SPI Tx CRC register */
            uint16_t RESERVED6;     /*!< Reserved */
} spi_reg_t;

/**
  * @brief TIM register structure definition
  */
typedef struct
{
  __IO uint32_t CTR1;               /*!< TIMx Control Register 1. */
  __IO uint32_t CTR2;               /*!< TIMx Control Register 2. */
  __IO uint32_t SMCFG;              /*!< TIMx slave mode configuration register. */
  __IO uint32_t DIEN;               /*!< TIMx DMA and interrupt enable register. */
  __IO uint32_t STS;                /*!< TIMx status register. */
  __IO uint32_t SWEGR;              /*!< TIMx software event generation register. */
  __IO uint32_t CHXCFGR[2];         /*!< TIMx channel 1~4 configuration register. */
  __IO uint32_t CCCTR;              /*!< TIMx channel capture compare control register. */
  __IO uint32_t CNT;                /*!< TIMx counter. */
  __IO uint32_t PDIV;               /*!< TIMx pre-divider. */
  __IO uint32_t UVAL;               /*!< TIMx counter update register. */
  __IO uint32_t UVALREP;            /*!< TIMx counter update repetition register. */
  __IO uint32_t CHXCCVAL[4];        /*!< TIMx channel 1~4 capture compare value register. */
  __IO uint32_t CHOPR;              /*!< TIMx channel output protect register. */
  __IO uint32_t DMAACR;             /*!< TIMx DMA access configuration register. */
  __IO uint32_t DMAIR;              /*!< TIMx DMA access interface register. */
}tim_reg_t;

/** 
  * @brief Universal Synchronous Asynchronous Receiver Transmitter
  */
 
typedef struct
{
  __IO uint32_t STS;
  __IO uint32_t DATA;
  __IO uint32_t BRT;
  __IO uint32_t CTR1;
  __IO uint32_t CTR2;
  __IO uint32_t CTR3;
  __IO uint32_t GTPDIV;
}usart_reg_t;

/**
* @struct wwdt_reg_t
* @brief WWDT(Window Watchdog Timer) registes  registers structure definition
*/
typedef struct {
    __IO    uint32_t CTR1;   /*!< WWDT Control register 1 */
    __IO    uint32_t CTR2;   /*!< WWDT Control register 2 */
    __IO    uint32_t STS;    /*!< WWDT Status register */
} wwdt_reg_t;

/** 
 * @struct i2c_reg_t
 * @brief I2C registes structure definition
 */
typedef struct
{
  __IO uint16_t CTR1;
  uint16_t  RESERVED0;
  __IO uint16_t CTR2;
  uint16_t  RESERVED1;
  __IO uint16_t SADR1;
  uint16_t  RESERVED2;
  __IO uint16_t SADR2;
  uint16_t  RESERVED3;
  __IO uint16_t DTR;
  uint16_t  RESERVED4;
  __IO uint16_t STR1;
  uint16_t  RESERVED5;
  __IO uint16_t STR2;
  uint16_t  RESERVED6;
  __IO uint16_t CCTR;
  uint16_t  RESERVED7;
  __IO uint16_t MRTR;
  uint16_t  RESERVED8;
} i2c_reg_t;

/**
  * @}
  */
  
/** @addtogroup Peripheral_memory_map
  * @{
  */


#define FLASH_BASE            ((uint32_t)0x08000000) /*!< FLASH base address in the alias region */
#define OB_BASE               ((uint32_t)0x1FFFF800) /*!< FLASH Option Bytes base address */

#define SRAM_BASE             ((uint32_t)0x20000000) /*!< SRAM base address in the alias region */
#define PERIPH_BASE           ((uint32_t)0x40000000) /*!< Peripheral base address in the alias region */

#define SRAM_BB_BASE          ((uint32_t)0x22000000) /*!< SRAM base address in the bit-band region */
#define PERIPH_BB_BASE        ((uint32_t)0x42000000) /*!< Peripheral base address in the bit-band region */

/*!< Peripheral memory map */
#define APB1PERIPH_BASE       PERIPH_BASE
#define APB2PERIPH_BASE       (PERIPH_BASE + 0x10000)
#define AHBPERIPH_BASE        (PERIPH_BASE + 0x20000)

#define TIM2_BASE             (APB1PERIPH_BASE + 0x0000)
#define TIM3_BASE             (APB1PERIPH_BASE + 0x0400)
#define TIM4_BASE             (APB1PERIPH_BASE + 0x0800)
#define RTC_BASE              (APB1PERIPH_BASE + 0x2800)
#define WWDT_BASE             (APB1PERIPH_BASE + 0x2C00)
#define FWDT_BASE             (APB1PERIPH_BASE + 0x3000)
#define SPI2_BASE             (APB1PERIPH_BASE + 0x3800)
#define SPI3_BASE             (APB1PERIPH_BASE + 0x3C00)
#define USART2_BASE           (APB1PERIPH_BASE + 0x4400)
#define USART3_BASE           (APB1PERIPH_BASE + 0x4800)
#define UART4_BASE            (APB1PERIPH_BASE + 0x4C00)
#define UART5_BASE            (APB1PERIPH_BASE + 0x5000)
#define I2C1_BASE             (APB1PERIPH_BASE + 0x5400)
#define I2C2_BASE             (APB1PERIPH_BASE + 0x5800)
#define CAN1_BASE             (APB1PERIPH_BASE + 0x6400)
#define BKP_BASE              (APB1PERIPH_BASE + 0x6C00)
#define PMU_BASE              (APB1PERIPH_BASE + 0x7000)

#define AFIO_BASE             (APB2PERIPH_BASE + 0x0000)
#define EXTI_BASE             (APB2PERIPH_BASE + 0x0400)
#define GPIOA_BASE            (APB2PERIPH_BASE + 0x0800)
#define GPIOB_BASE            (APB2PERIPH_BASE + 0x0C00)
#define GPIOC_BASE            (APB2PERIPH_BASE + 0x1000)
#define GPIOD_BASE            (APB2PERIPH_BASE + 0x1400)
#define GPIOE_BASE            (APB2PERIPH_BASE + 0x1800)
#define GPIOF_BASE            (APB2PERIPH_BASE + 0x1C00)
#define GPIOG_BASE            (APB2PERIPH_BASE + 0x2000)
#define ADC1_BASE             (APB2PERIPH_BASE + 0x2400)
#define ADC2_BASE             (APB2PERIPH_BASE + 0x2800)
#define TIM1_BASE             (APB2PERIPH_BASE + 0x2C00)
#define SPI1_BASE             (APB2PERIPH_BASE + 0x3000)
#define USART1_BASE           (APB2PERIPH_BASE + 0x3800)

#define DMA1_BASE             (AHBPERIPH_BASE + 0x0000)
#define DMA1_CHANNEL1_BASE    (AHBPERIPH_BASE + 0x0008)
#define DMA1_CHANNEL2_BASE    (AHBPERIPH_BASE + 0x001C)
#define DMA1_CHANNEL3_BASE    (AHBPERIPH_BASE + 0x0030)
#define DMA1_CHANNEL4_BASE    (AHBPERIPH_BASE + 0x0044)
#define DMA1_CHANNEL5_BASE    (AHBPERIPH_BASE + 0x0058)
#define DMA1_CHANNEL6_BASE    (AHBPERIPH_BASE + 0x006C)
#define DMA1_CHANNEL7_BASE    (AHBPERIPH_BASE + 0x0080)
#define DMA2_BASE             (AHBPERIPH_BASE + 0x0400)
#define DMA2_CHANNEL1_BASE    (AHBPERIPH_BASE + 0x0408)
#define DMA2_CHANNEL2_BASE    (AHBPERIPH_BASE + 0x041C)
#define DMA2_CHANNEL3_BASE    (AHBPERIPH_BASE + 0x0430)
#define DMA2_CHANNEL4_BASE    (AHBPERIPH_BASE + 0x0444)
#define DMA2_CHANNEL5_BASE    (AHBPERIPH_BASE + 0x0458)
#define RCU_BASE              (AHBPERIPH_BASE + 0x1000)
#define CRC_BASE              (AHBPERIPH_BASE + 0x3000)

#define FLASH_REG_BASE        (AHBPERIPH_BASE + 0x2000) /*!< FLASH registrs base address */


#define DBGMCU_BASE          ((uint32_t)0xE0042000)

/**
  * @}
  */
  
/** @addtogroup Peripheral_declaration
  * @{
  */  

#define TIM2                ((tim_reg_t *) TIM2_BASE)
#define TIM3                ((tim_reg_t *) TIM3_BASE)
#define TIM4                ((tim_reg_t *) TIM4_BASE)
#define RTC                 ((rtc_reg_t *) RTC_BASE)
#define WWDT                ((wwdt_reg_t *) WWDT_BASE)
#define FWDT                ((fwdt_reg_t *) FWDT_BASE)
#define SPI2                ((spi_reg_t *) SPI2_BASE)
#define SPI3                ((spi_reg_t *) SPI3_BASE)
#define USART2              ((usart_reg_t *) USART2_BASE)
#define USART3              ((usart_reg_t *) USART3_BASE)
#define I2C1                ((i2c_reg_t *) I2C1_BASE)
#define I2C2                ((i2c_reg_t *) I2C2_BASE)
#define CAN1                ((can_reg_t *) CAN1_BASE)
#define BKP                 ((bkp_reg_t *) BKP_BASE)
#define PMU                 ((pmu_reg_t *) PMU_BASE)
#define DAC                 ((DAC_TypeDef *) DAC_BASE)
#define CEC                 ((CEC_TypeDef *) CEC_BASE)
#define AFIO                ((afio_reg_t *) AFIO_BASE)
#define EXTI                ((exti_reg_t *) EXTI_BASE)
#define GPIOA               ((gpio_reg_t *) GPIOA_BASE)
#define GPIOB               ((gpio_reg_t *) GPIOB_BASE)
#define GPIOC               ((gpio_reg_t *) GPIOC_BASE)
#define GPIOD               ((gpio_reg_t *) GPIOD_BASE)
#define GPIOE               ((gpio_reg_t *) GPIOE_BASE)
#define GPIOF               ((gpio_reg_t *) GPIOF_BASE)
#define GPIOG               ((gpio_reg_t *) GPIOG_BASE)
#define ADC1                ((adc_reg_t *) ADC1_BASE)
#define ADC2                ((adc_reg_t *) ADC2_BASE)
#define TIM1                ((tim_reg_t *) TIM1_BASE)
#define SPI1                ((spi_reg_t *) SPI1_BASE)
#define USART1              ((usart_reg_t *) USART1_BASE)
#define DMA1                ((dma_reg_t *) DMA1_BASE)
#define DMA2                ((dma_reg_t *) DMA2_BASE)
#define DMA1_CHANNEL1       ((dma_channel_reg_t *) DMA1_CHANNEL1_BASE)
#define DMA1_CHANNEL2       ((dma_channel_reg_t *) DMA1_CHANNEL2_BASE)
#define DMA1_CHANNEL3       ((dma_channel_reg_t *) DMA1_CHANNEL3_BASE)
#define DMA1_CHANNEL4       ((dma_channel_reg_t *) DMA1_CHANNEL4_BASE)
#define DMA1_CHANNEL5       ((dma_channel_reg_t *) DMA1_CHANNEL5_BASE)
#define DMA1_CHANNEL6       ((dma_channel_reg_t *) DMA1_CHANNEL6_BASE)
#define DMA1_CHANNEL7       ((dma_channel_reg_t *) DMA1_CHANNEL7_BASE)
#define DMA2_CHANNEL1       ((dma_channel_reg_t *) DMA2_CHANNEL1_BASE)
#define DMA2_CHANNEL2       ((dma_channel_reg_t *) DMA2_CHANNEL2_BASE)
#define DMA2_CHANNEL3       ((dma_channel_reg_t *) DMA2_CHANNEL3_BASE)
#define DMA2_CHANNEL4       ((dma_channel_reg_t *) DMA2_CHANNEL4_BASE)
#define DMA2_CHANNEL5       ((dma_channel_reg_t *) DMA2_CHANNEL5_BASE)
#define RCU                 ((rcu_reg_t *) RCU_BASE)
#define CRC                 ((crc_reg_t *) CRC_BASE)
#define FLASH               ((fmc_reg_t *)        FLASH_REG_BASE)
#define OB                  ((ob_reg_t *)         OB_BASE)
#define DBGMCU              ((debug_reg_t *)      DBGMCU_BASE)

/**
  * @}
  */

/** @addtogroup Exported_constants
  * @{
  */
  
  /** @addtogroup Peripheral_Registers_Bits_Definition
  * @{
  */
    
/******************************************************************************/
/*                         Peripheral Registers_Bits_Definition               */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/*                          CRC calculation unit                              */
/*                                                                            */
/******************************************************************************/
#define  CRC_DATA_DATA                       ((uint32_t)0xFFFFFFFF) /*!< Data register bits */
#define  CRC_FREDATA_DATA                    ((uint8_t)0xFF)        /*!< General-purpose 8-bit data register bits */
#define  CRC_CTR_RST                         ((uint8_t)0x01)        /*!< RESET bit */

/******************************************************************************/
/*                                                                            */
/*                             Power Control                                  */
/*                                                                            */
/******************************************************************************/

/**
  * @brief PMU_CTR register Bit definition
  */
#define  PMU_CTR_VBTWEN_EN                    ((uint16_t)0x0100)     /*!< VBAT domain write enable */
#define  PMU_CTR_LVD_VOLTAGE                  ((uint16_t)0x00E0)     /*!< LVD voltage */
#define  PMU_CTR_LVD_EN                       ((uint16_t)0x0010)     /*!< LVD enable */
#define  PMU_CTR_CLRPWDF                      ((uint16_t)0x0008)     /*!< Clear standby flag */
#define  PMU_CTR_CLRWUPF                      ((uint16_t)0x0004)     /*!< Clear wakeup flag */
#define  PMU_CTR_DSMODE                       ((uint16_t)0x0002)     /*!< Deepsleep mode */
#define  PMU_CTR_LDOLM                        ((uint16_t)0x0001)     /*!< LDO ow power Control */

/**
  * @brief PMU_CS register Bit definition
  */
#define  PMU_CS_WKUP_PIN_EN                   ((uint16_t)0x0100)     /*!< wakeup pin enable */
#define  PMU_CS_LVDO_FLAG                     ((uint16_t)0x0004)     /*!< LVD ouput flag */
#define  PMU_CS_PWDF_FLAG                     ((uint16_t)0x0002)     /*!< standby flag */
#define  PMU_CS_WUPF_FLAG                     ((uint16_t)0x0001)     /*!< wakeup flag */

/******************************************************************************/
/*                                                                            */
/*                            Backup registers                                */
/*                                                                            */
/******************************************************************************/

/**
  * @brief BKP_AOx register Bit definition
  */
#define  BKP_AOX_AO                         ((uint32_t)0x0000FFFF)      /*!< Always on register. */

/**
  * @brief BKP_CCR register Bit definition
  */
#define  BKP_CCR_ALMSOS                     ((uint32_t)0x00000200)      /*!< Alarm or second output selection. */
#define  BKP_CCR_ALMSOEN                    ((uint32_t)0x00000100)      /*!< Alarm or second output enable. */
#define  BKP_CCR_CALCO                      ((uint32_t)0x00000080)      /*!< Calibration clock output. */
#define  BKP_CCR_CALDATA                    ((uint32_t)0x0000007F)      /*!< Calibration data. */

/**
  * @brief BKP_CTR register Bit definition
  */
#define  BKP_CTR_TAMPPA                     ((uint32_t)0x00000002)      /*!< TAMPER pin active level. */
#define  BKP_CTR_TAMPPEN                    ((uint32_t)0x00000001)      /*!< TAMPER pin enable. */

/**
  * @brief BKP_CSR register Bit definition
  */
#define  BKP_CSR_TAMPIF                     ((uint32_t)0x00000200)      /*!< Tamper interrupt flag. */
#define  BKP_CSR_TAMPEF                     ((uint32_t)0x00000100)      /*!< Tamper event flag. */
#define  BKP_CSR_TAMPIE                     ((uint32_t)0x00000004)      /*!< TAMPER pin interrupt enable. */
#define  BKP_CSR_TAMPIFCLR                  ((uint32_t)0x00000002)      /*!< Clear tamper interrupt. */
#define  BKP_CSR_TAMPECLR                   ((uint32_t)0x00000001)      /*!< Clear tamper event. */

/******************************************************************************/
/*                                                                            */
/*                         Reset and clock unit                               */
/*                                                                            */
/******************************************************************************/

/**
  * @brief RCU_CTR register Bit definition
  */
#define  RCU_CTR_HRCEN                       ((uint32_t)0x00000001)        /*!< HRC Enable */
#define  RCU_CTR_HRCSTAB                     ((uint32_t)0x00000002)        /*!< HRC Stabilization flag */
#define  RCU_CTR_HRCTRIM                     ((uint32_t)0x000000F8)        /*!< HRC Triming  */
#define  RCU_CTR_HRCCALIB                    ((uint32_t)0x0000FF00)        /*!< HRC Internal 8MHz RC calibration */
#define  RCU_CTR_HXTEN                       ((uint32_t)0x00010000)        /*!< HXT Enable */
#define  RCU_CTR_HXTSTAB                     ((uint32_t)0x00020000)        /*!< HXT Stabilization flag */
#define  RCU_CTR_HXTBPS                      ((uint32_t)0x00040000)        /*!< HXT Bypass Enable  */
#define  RCU_CTR_HXTME                       ((uint32_t)0x00080000)        /*!< HXT Monitor Enable */
#define  RCU_CTR_PLLEN                       ((uint32_t)0x01000000)        /*!< PLL enable */
#define  RCU_CTR_PLLSTAB                     ((uint32_t)0x02000000)        /*!< PLL Stabilization flag */


/**
  * @brief RCU_CFG register Bit definition
  */
#define  RCU_CFG_SYSSW                       ((uint32_t)0x00000003)        /*!< System clock switch Control  */
#define  RCU_CFG_SYSSS                       ((uint32_t)0x0000000C)        /*!< System clock switch status */
#define  RCU_CFG_HCLKPDIV                    ((uint32_t)0x000000F0)        /*!< HCLK Pre-divider  */
#define  RCU_CFG_PCLKPDIV                    ((uint32_t)0x00000700)        /*!< PCLK Pre-divider */
#define  RCU_CFG_HPCLKPDIV                   ((uint32_t)0x00003800)        /*!< APB high-speed prescale */
#define  RCU_CFG_ADCPDIV                     ((uint32_t)0x0000C000)        /*!< ADC Clock Pre-divider */
#define  RCU_CFG_PLLSEL                      ((uint32_t)0x00010000)        /*!< PLL source Selction */
#define  RCU_CFG_PLLHXTPDIV                  ((uint32_t)0x00020000)        /*!< PLL source HXT pre-divdier */
#define  RCU_CFG_PLLMUF                      ((uint32_t)0x003C0000)        /*!< PLL clock multiplication factor */
#define  RCU_CFG_USBPDIV                     ((uint32_t)0x00400000)        /*!< USB Clock Pre-divider */
#define  RCU_CFG_CKOSEL                      ((uint32_t)0x07000000)        /*!< CLKOUT Source Selection */


/**
  * @brief RCU_INTR register Bit definition
  */
#define  RCU_INTR_LRCSTABIF                  ((uint32_t)0x00000001)        /*!< LRC stabilization Interrupt Flag */
#define  RCU_INTR_LXTSTABIF                  ((uint32_t)0x00000002)        /*!< LXT stabilization Interrupt Flag */
#define  RCU_INTR_HRCSTABIF                  ((uint32_t)0x00000004)        /*!< HRC stabilization Interrupt Flag */
#define  RCU_INTR_HXTSTABIF                  ((uint32_t)0x00000008)        /*!< HXT stabilization Interrupt Flag */
#define  RCU_INTR_PLLSTABIF                  ((uint32_t)0x00000010)        /*!< PLL stabilization Interrupt Flag */
#define  RCU_INTR_CKFAILIF                   ((uint32_t)0x00000080)        /*!< HXT Clock Fail Interrupt Flag */
#define  RCU_INTR_LRCSTABIE                  ((uint32_t)0x00000100)        /*!< LRC stabilization Interrupt Enable */
#define  RCU_INTR_LXTSTABIE                  ((uint32_t)0x00000200)        /*!< LXT stabilization Interrupt Enable */
#define  RCU_INTR_HRCSTABIE                  ((uint32_t)0x00000400)        /*!< HRC stabilization Interrupt Enable */
#define  RCU_INTR_HXTSTABIE                  ((uint32_t)0x00000800)        /*!< HXT stabilization Interrupt Enable */
#define  RCU_INTR_PLLSTABIE                  ((uint32_t)0x00001000)        /*!< PLL stabilization Interrupt Enable */
#define  RCU_INTR_LRCSTABIC                  ((uint32_t)0x00010000)        /*!< LRC stabilization Interrupt Clear */
#define  RCU_INTR_LXTSTABIC                  ((uint32_t)0x00020000)        /*!< LXT stabilization Interrupt Clear */
#define  RCU_INTR_HRCSTABIC                  ((uint32_t)0x00040000)        /*!< HRC stabilization Interrupt Clear */
#define  RCU_INTR_HXTSTABIC                  ((uint32_t)0x00080000)        /*!< HXT stabilization Interrupt Clear */
#define  RCU_INTR_PLLSTABIC                  ((uint32_t)0x00100000)        /*!< PLL stabilization Interrupt Clear */
#define  RCU_INTR_CKFAILIC                   ((uint32_t)0x00800000)        /*!< HXT Clock Fail Interrupt Clear */


/**
  * @brief RCU_APB2RST register Bit definition
  */
#define  RCU_APB2RST_AFIORST                ((uint32_t)0x00000001)         /*!< Alternate Function I/O reset */
#define  RCU_APB2RST_PARST                  ((uint32_t)0x00000004)         /*!< I/O port A reset */
#define  RCU_APB2RST_PBRST                  ((uint32_t)0x00000008)         /*!< I/O port B reset */
#define  RCU_APB2RST_PCRST                  ((uint32_t)0x00000010)         /*!< I/O port C reset */
#define  RCU_APB2RST_PDRST                  ((uint32_t)0x00000020)         /*!< I/O port D reset */
#define  RCU_APB2RST_PERST                  ((uint32_t)0x00000040)         /*!< I/O port E reset */
#define  RCC_APB2RST_ADC1RST                ((uint32_t)0x00000200)         /*!< ADC 1 interface reset */
#define  RCC_APB2RST_ADC2RST                ((uint32_t)0x00000400)         /*!< ADC 2 interface reset */
#define  RCC_APB2RST_TIM1RST                ((uint32_t)0x00000800)         /*!< TIM1 Timer reset */
#define  RCC_APB2RST_SPI1RST                ((uint32_t)0x00001000)         /*!< SPI1 reset */
#define  RCC_APB2RST_USART1RST              ((uint32_t)0x00004000)         /*!< USART1 reset */


/**
  * @brief RCU_APB1RST register Bit definition
  */
#define  RCU_APB1RST_TIM2RST                ((uint32_t)0x00000001)        /*!< Timer 2 reset */
#define  RCU_APB1RST_TIM3RST                ((uint32_t)0x00000002)        /*!< Timer 3 reset */
#define  RCU_APB1RST_TIM4RST                ((uint32_t)0x00000004)        /*!< Timer 4 reset */
#define  RCU_APB1RST_WWDTRST                ((uint32_t)0x00000800)        /*!< Window Watchdog reset */
#define  RCU_APB1RST_SPI2RST                ((uint32_t)0x00004000)        /*!< SPI 2 reset */
#define  RCU_APB1RST_USART2RST              ((uint32_t)0x00020000)        /*!< USART 2 reset */
#define  RCU_APB1RST_USART3RST              ((uint32_t)0x00040000)        /*!< USART 3 reset */
#define  RCU_APB1RST_I2C1RST                ((uint32_t)0x00200000)        /*!< I2C 1 reset */
#define  RCU_APB1RST_I2C2RST                ((uint32_t)0x00400000)        /*!< I2C 2 reset */
#define  RCU_APB1RST_USBRST                 ((uint32_t)0x00800000)        /*!< USB Device reset */
#define  RCU_APB1RST_CAN1RST                ((uint32_t)0x02000000)        /*!< CAN1 reset */
#define  RCU_APB1RST_BKPRST                 ((uint32_t)0x08000000)        /*!< Backup interface reset */
#define  RCU_APB1RST_PMURST                 ((uint32_t)0x10000000)        /*!< Power interface reset */


/**
  * @brief RCU_AHBEN register Bit definition
  */
#define  RCU_AHBEN_DMA1EN                   ((uint16_t)0x0001)            /*!< enable bit for DMA1 clock */
#define  RCU_AHBEN_RMCEN                    ((uint16_t)0x0004)            /*!< enable bit for SRAM clock */
#define  RCU_AHBEN_FMCEN                    ((uint16_t)0x0010)            /*!< enable bit for FMC clock */
#define  RCU_AHBEN_CRCEN                    ((uint16_t)0x0040)            /*!< enable bit for CRC clock */


/**
  * @brief RCU_APB2EN register Bit definition
  */
#define  RCU_APB2EN_AFIOEN                  ((uint32_t)0x00000001)         /*!< enable bit for Alternate Function clock */
#define  RCU_APB2EN_PAEN                    ((uint32_t)0x00000004)         /*!< enable bit for GPIOA clock */
#define  RCU_APB2EN_PBEN                    ((uint32_t)0x00000008)         /*!< enable bit for GPIOB clock */
#define  RCU_APB2EN_PCEN                    ((uint32_t)0x00000010)         /*!< enable bit for GPIOC clock */
#define  RCU_APB2EN_PDEN                    ((uint32_t)0x00000020)         /*!< enable bit for GPIOD clock */
#define  RCU_APB2EN_PEEN                    ((uint32_t)0x00000040)         /*!< enable bit for GPIOE clock */

#define  RCU_APB2EN_ADC1EN                  ((uint32_t)0x00000200)         /*!< enable bit for ADC1 clock */
#define  RCU_APB2EN_ADC2EN                  ((uint32_t)0x00000400)         /*!< enable bit for ADC2 clock */

#define  RCU_APB2EN_TIM1EN                  ((uint32_t)0x00000800)         /*!< enable bit for TIM1 clock */
#define  RCU_APB2EN_SPI1EN                  ((uint32_t)0x00001000)         /*!< enable bit for SPI1 clock */
#define  RCU_APB2EN_USART1EN                ((uint32_t)0x00004000)         /*!< enable bit for USART1 clock */


/**
  * @brief RCU_APB1EN register Bit definition
  */
#define  RCU_APB1EN_TIM2EN                  ((uint32_t)0x00000001)        /*!< enable bit for TIM2 clock*/
#define  RCU_APB1EN_TIM3EN                  ((uint32_t)0x00000002)        /*!< enable bit for TIM3 clock */
#define  RCU_APB1EN_TIM4EN                  ((uint32_t)0x00000004)        /*!< enable bit for TIM4 clock */
#define  RCU_APB1EN_WWDTEN                  ((uint32_t)0x00000800)        /*!< enable bit for Window Watchdog clock */
#define  RCU_APB1EN_SPI2EN                  ((uint32_t)0x00004000)        /*!< enable bit for SPI2 clock */
#define  RCU_APB1EN_USART2EN                ((uint32_t)0x00020000)        /*!< enable bit for USART2 clock */
#define  RCU_APB1EN_USART3EN                ((uint32_t)0x00040000)        /*!< enable bit for USART3 clock */
#define  RCU_APB1EN_I2C1EN                  ((uint32_t)0x00200000)        /*!< enable bit for I2C1 clock */
#define  RCU_APB1EN_I2C2EN                  ((uint32_t)0x00400000)        /*!< enable bit for I2C2 clock */
#define  RCU_APB1EN_USBEN                   ((uint32_t)0x00800000)        /*!< enable bit for USB clock */
#define  RCU_APB1EN_CAN1EN                  ((uint32_t)0x02000000)        /*!< enable bit for CAN1 clock */
#define  RCC_APB1EN_BKPEN                   ((uint32_t)0x08000000)        /*!< enable bit for BAKEUP INTERFACE */
#define  RCC_APB1EN_PMUEN                   ((uint32_t)0x10000000)        /*!< enable bit for POWER clock */


/**
  * @brief RCU_VBDC register Bit definition
  */
#define  RCU_VBDC_LXTEN                      ((uint32_t)0x00000001)        /*!< LXT enable */
#define  RCU_VBDC_LXTSTAB                    ((uint32_t)0x00000002)        /*!< LXT stable flag */
#define  RCU_VBDC_LXTBYP                     ((uint32_t)0x00000004)        /*!< LXT bypass enable bit */
#define  RCU_VBDC_RTCSRC                     ((uint32_t)0x00000300)        /*!< RTC clock source selection */
#define  RCU_VBDC_RTCCLKEN                   ((uint32_t)0x00008000)        /*!< enable bit for RTC clock */
#define  RCU_VBDC_VBTRST                     ((uint32_t)0x00010000)        /*!< Backup domain software reset  */

/**
  * @brief RCU_STS register Bit definition
  */
#define  RCU_STS_LRCEN                       ((uint32_t)0x00000001)        /*!< LRC oscillator enable */
#define  RCU_STS_LRCSTAB                     ((uint32_t)0x00000002)        /*!< LRC stable flag */
#define  RCU_STS_CRSTF                       ((uint32_t)0x01000000)        /*!< Clear reset Flag */
#define  RCU_STS_NRSTF                       ((uint32_t)0x04000000)        /*!< reset flag of pin nRST */
#define  RCU_STS_PORRSTF                     ((uint32_t)0x08000000)        /*!< POR/PDR reset flag */
#define  RCU_STS_SWRSTF                      ((uint32_t)0x10000000)        /*!< Software Reset flag */
#define  RCU_STS_FWDTRSTF                    ((uint32_t)0x20000000)        /*!< FWDT reset flag */
#define  RCU_STS_WWDTRSTF                    ((uint32_t)0x40000000)        /*!< WWDT reset flag */
#define  RCU_STS_LPERSTF                     ((uint32_t)0x80000000)        /*!< Low-power mode enter-reset flag */




/******************************************************************************/
/*                                                                            */
/*           General Purpose and Alternate Function I/O unit                  */
/*                                                                            */
/******************************************************************************/


/**
  * @brief GPIO_CFGL register Bit definition
  */
#define  GPIO_CFGL_MODE                       ((uint32_t)0x33333333)        /*!< GPIO port mode bits */

#define  GPIO_CFGL_CNF                        ((uint32_t)0xCCCCCCCC)        /*!< GPIO port in/out bits */

/**
  * @brief GPIO_CFGH register Bit definition
  */
#define  GPIO_CFGH_MODE                       ((uint32_t)0x33333333)        /*!< GPIO port mode bits */

#define  GPIO_CFGH_CNF                        ((uint32_t)0xCCCCCCCC)        /*!< GPIO port in/out bits */

/**
  * @brief GPIO_DI register Bit definition
  */
#define GPIO_DI_DI0                        ((uint16_t)0x0001)            /*!< Port data input, bit 0 */
#define GPIO_DI_DI1                        ((uint16_t)0x0002)            /*!< Port data input, bit 1 */
#define GPIO_DI_DI2                        ((uint16_t)0x0004)            /*!< Port data input, bit 2 */
#define GPIO_DI_DI3                        ((uint16_t)0x0008)            /*!< Port data input, bit 3 */
#define GPIO_DI_DI4                        ((uint16_t)0x0010)            /*!< Port data input, bit 4 */
#define GPIO_DI_DI5                        ((uint16_t)0x0020)            /*!< Port data input, bit 5 */
#define GPIO_DI_DI6                        ((uint16_t)0x0040)            /*!< Port data input, bit 6 */
#define GPIO_DI_DI7                        ((uint16_t)0x0080)            /*!< Port data input, bit 7 */
#define GPIO_DI_DI8                        ((uint16_t)0x0100)            /*!< Port data input, bit 8 */
#define GPIO_DI_DI9                        ((uint16_t)0x0200)            /*!< Port data input, bit 9 */
#define GPIO_DI_DI10                       ((uint16_t)0x0400)            /*!< Port data input, bit 10 */
#define GPIO_DI_DI11                       ((uint16_t)0x0800)            /*!< Port data input, bit 11 */
#define GPIO_DI_DI12                       ((uint16_t)0x1000)            /*!< Port data input, bit 12 */
#define GPIO_DI_DI13                       ((uint16_t)0x2000)            /*!< Port data input, bit 13 */
#define GPIO_DI_DI14                       ((uint16_t)0x4000)            /*!< Port data input, bit 14 */
#define GPIO_DI_DI15                       ((uint16_t)0x8000)            /*!< Port data input, bit 15 */

/**
  * @brief GPIO_DO register Bit definition
  */
#define GPIO_DO_DO0                        ((uint16_t)0x0001)            /*!< Port data output, bit 0 */
#define GPIO_DO_DO1                        ((uint16_t)0x0002)            /*!< Port data output, bit 1 */
#define GPIO_DO_DO2                        ((uint16_t)0x0004)            /*!< Port data output, bit 2 */
#define GPIO_DO_DO3                        ((uint16_t)0x0008)            /*!< Port data output, bit 3 */
#define GPIO_DO_DO4                        ((uint16_t)0x0010)            /*!< Port data output, bit 4 */
#define GPIO_DO_DO5                        ((uint16_t)0x0020)            /*!< Port data output, bit 5 */
#define GPIO_DO_DO6                        ((uint16_t)0x0040)            /*!< Port data output, bit 6 */
#define GPIO_DO_DO7                        ((uint16_t)0x0080)            /*!< Port data output, bit 7 */
#define GPIO_DO_DO8                        ((uint16_t)0x0100)            /*!< Port data output, bit 8 */
#define GPIO_DO_DO9                        ((uint16_t)0x0200)            /*!< Port data output, bit 9 */
#define GPIO_DO_DO10                       ((uint16_t)0x0400)            /*!< Port data output, bit 10 */
#define GPIO_DO_DO11                       ((uint16_t)0x0800)            /*!< Port data output, bit 11 */
#define GPIO_DO_DO12                       ((uint16_t)0x1000)            /*!< Port data output, bit 12 */
#define GPIO_DO_DO13                       ((uint16_t)0x2000)            /*!< Port data output, bit 13 */
#define GPIO_DO_DO14                       ((uint16_t)0x4000)            /*!< Port data output, bit 14 */
#define GPIO_DO_DO15                       ((uint16_t)0x8000)            /*!< Port data output, bit 15 */

/**
  * @brief GPIO_SCR register Bit definition
  */
#define GPIO_SCR_BS0                        ((uint32_t)0x00000001)        /*!< GPIO Port Set bit 0 */
#define GPIO_SCR_BS1                        ((uint32_t)0x00000002)        /*!< GPIO Port Set bit 1 */
#define GPIO_SCR_BS2                        ((uint32_t)0x00000004)        /*!< GPIO Port Set bit 2 */
#define GPIO_SCR_BS3                        ((uint32_t)0x00000008)        /*!< GPIO Port Set bit 3 */
#define GPIO_SCR_BS4                        ((uint32_t)0x00000010)        /*!< GPIO Port Set bit 4 */
#define GPIO_SCR_BS5                        ((uint32_t)0x00000020)        /*!< GPIO Port Set bit 5 */
#define GPIO_SCR_BS6                        ((uint32_t)0x00000040)        /*!< GPIO Port Set bit 6 */
#define GPIO_SCR_BS7                        ((uint32_t)0x00000080)        /*!< GPIO Port Set bit 7 */
#define GPIO_SCR_BS8                        ((uint32_t)0x00000100)        /*!< GPIO Port Set bit 8 */
#define GPIO_SCR_BS9                        ((uint32_t)0x00000200)        /*!< GPIO Port Set bit 9 */
#define GPIO_SCR_BS10                       ((uint32_t)0x00000400)        /*!< GPIO Port Set bit 10 */
#define GPIO_SCR_BS11                       ((uint32_t)0x00000800)        /*!< GPIO Port Set bit 11 */
#define GPIO_SCR_BS12                       ((uint32_t)0x00001000)        /*!< GPIO Port Set bit 12 */
#define GPIO_SCR_BS13                       ((uint32_t)0x00002000)        /*!< GPIO Port Set bit 13 */
#define GPIO_SCR_BS14                       ((uint32_t)0x00004000)        /*!< GPIO Port Set bit 14 */
#define GPIO_SCR_BS15                       ((uint32_t)0x00008000)        /*!< GPIO Port Set bit 15 */

#define GPIO_SCR_BC0                        ((uint32_t)0x00010000)        /*!< GPIO Port Clear bit 0 */
#define GPIO_SCR_BC1                        ((uint32_t)0x00020000)        /*!< GPIO Port Clear bit 1 */
#define GPIO_SCR_BC2                        ((uint32_t)0x00040000)        /*!< GPIO Port Clear bit 2 */
#define GPIO_SCR_BC3                        ((uint32_t)0x00080000)        /*!< GPIO Port Clear bit 3 */
#define GPIO_SCR_BC4                        ((uint32_t)0x00100000)        /*!< GPIO Port Clear bit 4 */
#define GPIO_SCR_BC5                        ((uint32_t)0x00200000)        /*!< GPIO Port Clear bit 5 */
#define GPIO_SCR_BC6                        ((uint32_t)0x00400000)        /*!< GPIO Port Clear bit 6 */
#define GPIO_SCR_BC7                        ((uint32_t)0x00800000)        /*!< GPIO Port Clear bit 7 */
#define GPIO_SCR_BC8                        ((uint32_t)0x01000000)        /*!< GPIO Port Clear bit 8 */
#define GPIO_SCR_BC9                        ((uint32_t)0x02000000)        /*!< GPIO Port Clear bit 9 */
#define GPIO_SCR_BC10                       ((uint32_t)0x04000000)        /*!< GPIO Port Clear bit 10 */
#define GPIO_SCR_BC11                       ((uint32_t)0x08000000)        /*!< GPIO Port Clear bit 11 */
#define GPIO_SCR_BC12                       ((uint32_t)0x10000000)        /*!< GPIO Port Clear bit 12 */
#define GPIO_SCR_BC13                       ((uint32_t)0x20000000)        /*!< GPIO Port Clear bit 13 */
#define GPIO_SCR_BC14                       ((uint32_t)0x40000000)        /*!< GPIO Port Clear bit 14 */
#define GPIO_SCR_BC15                       ((uint32_t)0x80000000)        /*!< GPIO Port Clear bit 15 */

/**
  * @brief GPIO_CLRR register Bit definition
  */
#define GPIO_CLRR_BCLR0                      ((uint16_t)0x0001)            /*!< GPIO Port Reset bit 0 */
#define GPIO_CLRR_BCLR1                      ((uint16_t)0x0002)            /*!< GPIO Port Reset bit 1 */
#define GPIO_CLRR_BCLR2                      ((uint16_t)0x0004)            /*!< GPIO Port Reset bit 2 */
#define GPIO_CLRR_BCLR3                      ((uint16_t)0x0008)            /*!< GPIO Port Reset bit 3 */
#define GPIO_CLRR_BCLR4                      ((uint16_t)0x0010)            /*!< GPIO Port Reset bit 4 */
#define GPIO_CLRR_BCLR5                      ((uint16_t)0x0020)            /*!< GPIO Port Reset bit 5 */
#define GPIO_CLRR_BCLR6                      ((uint16_t)0x0040)            /*!< GPIO Port Reset bit 6 */
#define GPIO_CLRR_BCLR7                      ((uint16_t)0x0080)            /*!< GPIO Port Reset bit 7 */
#define GPIO_CLRR_BCLR8                      ((uint16_t)0x0100)            /*!< GPIO Port Reset bit 8 */
#define GPIO_CLRR_BCLR9                      ((uint16_t)0x0200)            /*!< GPIO Port Reset bit 9 */
#define GPIO_CLRR_BCLR10                     ((uint16_t)0x0400)            /*!< GPIO Port Reset bit 10 */
#define GPIO_CLRR_BCLR11                     ((uint16_t)0x0800)            /*!< GPIO Port Reset bit 11 */
#define GPIO_CLRR_BCLR12                     ((uint16_t)0x1000)            /*!< GPIO Port Reset bit 12 */
#define GPIO_CLRR_BCLR13                     ((uint16_t)0x2000)            /*!< GPIO Port Reset bit 13 */
#define GPIO_CLRR_BCLR14                     ((uint16_t)0x4000)            /*!< GPIO Port Reset bit 14 */
#define GPIO_CLRR_BCLR15                     ((uint16_t)0x8000)            /*!< GPIO Port Reset bit 15 */

/**
  * @brief GPIO_LOCK register Bit definition
  */
#define GPIO_LOCK_LOCK0                      ((uint32_t)0x00000001)        /*!< GPIO Port Lock bit 0 */
#define GPIO_LOCK_LOCK1                      ((uint32_t)0x00000002)        /*!< GPIO Port Lock bit 1 */
#define GPIO_LOCK_LOCK2                      ((uint32_t)0x00000004)        /*!< GPIO Port Lock bit 2 */
#define GPIO_LOCK_LOCK3                      ((uint32_t)0x00000008)        /*!< GPIO Port Lock bit 3 */
#define GPIO_LOCK_LOCK4                      ((uint32_t)0x00000010)        /*!< GPIO Port Lock bit 4 */
#define GPIO_LOCK_LOCK5                      ((uint32_t)0x00000020)        /*!< GPIO Port Lock bit 5 */
#define GPIO_LOCK_LOCK6                      ((uint32_t)0x00000040)        /*!< GPIO Port Lock bit 6 */
#define GPIO_LOCK_LOCK7                      ((uint32_t)0x00000080)        /*!< GPIO Port Lock bit 7 */
#define GPIO_LOCK_LOCK8                      ((uint32_t)0x00000100)        /*!< GPIO Port Lock bit 8 */
#define GPIO_LOCK_LOCK9                      ((uint32_t)0x00000200)        /*!< GPIO Port Lock bit 9 */
#define GPIO_LOCK_LOCK10                     ((uint32_t)0x00000400)        /*!< GPIO Port Lock bit 10 */
#define GPIO_LOCK_LOCK11                     ((uint32_t)0x00000800)        /*!< GPIO Port Lock bit 11 */
#define GPIO_LOCK_LOCK12                     ((uint32_t)0x00001000)        /*!< GPIO Port Lock bit 12 */
#define GPIO_LOCK_LOCK13                     ((uint32_t)0x00002000)        /*!< GPIO Port Lock bit 13 */
#define GPIO_LOCK_LOCK14                     ((uint32_t)0x00004000)        /*!< GPIO Port Lock bit 14 */
#define GPIO_LOCK_LOCK15                     ((uint32_t)0x00008000)        /*!< GPIO Port Lock bit 15 */
#define GPIO_LOCK_LOCKK                      ((uint32_t)0x00010000)        /*!< Lock key */

/******************************************************************************/
/*                                                                            */
/*                        Alternate Function I/O unit                         */
/*                                                                            */
/******************************************************************************/

/**
  * @brief AFIO_ECR register Bit definition
  */
#define AFIO_ECR_EOPIN                       ((uint8_t)0x0F)               /*!< Event output pin selection bits */
#define AFIO_ECR_EOPS                        ((uint8_t)0x70)               /*!< Event output port selection bits */
#define AFIO_ECR_EOEN                        ((uint8_t)0x80)               /*!< Event Output Enable */

/**
  * @brief AFIO_MP register Bit definition
  */
#define AFIO_MP_SPI1_RMP                     ((uint32_t)0x00000001)        /*!< SPI1 remapping */
#define AFIO_MP_I2C1_RMP                     ((uint32_t)0x00000002)        /*!< I2C1 remapping */
#define AFIO_MP_USART1_RMP                   ((uint32_t)0x00000004)        /*!< USART1 remapping */
#define AFIO_MP_USART2_RMP                   ((uint32_t)0x00000008)        /*!< USART2 remapping */
#define AFIO_MP_USART3_RMP                   ((uint32_t)0x00000030)        /*!< USART3_REMAP[1:0] bits (USART3 remapping) */
#define AFIO_MP_TIM1_RMP                     ((uint32_t)0x000000C0)        /*!< TIM1_REMAP[1:0] bits (TIM1 remapping) */
#define AFIO_MP_TIM2_RMP                     ((uint32_t)0x00000300)        /*!< TIM2_REMAP[1:0] bits (TIM2 remapping) */
#define AFIO_MP_TIM3_RMP                     ((uint32_t)0x00000C00)        /*!< TIM3_REMAP[1:0] bits (TIM3 remapping) */
#define AFIO_MP_TIM4_RMP                     ((uint32_t)0x00001000)        /*!< TIM4_REMAP bit (TIM4 remapping) */
#define AFIO_MP_CAN_RMP                      ((uint32_t)0x00006000)        /*!< CAN_REMAP[1:0] bits (CAN Alternate function remapping) */
#define AFIO_MP_PD01_RMP                     ((uint32_t)0x00008000)        /*!< Port D0/Port D1 mapping on OSC_IN/OSC_OUT */
/*!< SWJ_CFG configuration */
#define AFIO_MP_SWJ_CFG                    ((uint32_t)0x07000000)        /*!< SWJ_CFG[2:0] bits (Serial Wire JTAG configuration) */

/**
  * @brief AFIO_EXTCFG1 register Bit definition
  */
#define AFIO_EXTCFG1_EXTI0                   ((uint16_t)0x000F)            /*!< EXTI 0 */
#define AFIO_EXTCFG1_EXTI1                   ((uint16_t)0x00F0)            /*!< EXTI 1 */
#define AFIO_EXTCFG1_EXTI2                   ((uint16_t)0x0F00)            /*!< EXTI 2 */
#define AFIO_EXTCFG1_EXTI3                   ((uint16_t)0xF000)            /*!< EXTI 3 */

/*!< EXTI0 configuration */
#define AFIO_EXTCFG1_EXTI0_PA                ((uint16_t)0x0000)            /*!< PA[0] pin */
#define AFIO_EXTCFG1_EXTI0_PB                ((uint16_t)0x0001)            /*!< PB[0] pin */
#define AFIO_EXTCFG1_EXTI0_PC                ((uint16_t)0x0002)            /*!< PC[0] pin */
#define AFIO_EXTCFG1_EXTI0_PD                ((uint16_t)0x0003)            /*!< PD[0] pin */
#define AFIO_EXTCFG1_EXTI0_PE                ((uint16_t)0x0004)            /*!< PE[0] pin */


/*!< EXTI1 configuration */
#define AFIO_EXTCFG1_EXTI1_PA                ((uint16_t)0x0000)            /*!< PA[1] pin */
#define AFIO_EXTCFG1_EXTI1_PB                ((uint16_t)0x0010)            /*!< PB[1] pin */
#define AFIO_EXTCFG1_EXTI1_PC                ((uint16_t)0x0020)            /*!< PC[1] pin */
#define AFIO_EXTCFG1_EXTI1_PD                ((uint16_t)0x0030)            /*!< PD[1] pin */
#define AFIO_EXTCFG1_EXTI1_PE                ((uint16_t)0x0040)            /*!< PE[1] pin */

/*!< EXTI2 configuration */  
#define AFIO_EXTCFG1_EXTI2_PA                ((uint16_t)0x0000)            /*!< PA[2] pin */
#define AFIO_EXTCFG1_EXTI2_PB                ((uint16_t)0x0100)            /*!< PB[2] pin */
#define AFIO_EXTCFG1_EXTI2_PC                ((uint16_t)0x0200)            /*!< PC[2] pin */
#define AFIO_EXTCFG1_EXTI2_PD                ((uint16_t)0x0300)            /*!< PD[2] pin */
#define AFIO_EXTCFG1_EXTI2_PE                ((uint16_t)0x0400)            /*!< PE[2] pin */

/*!< EXTI3 configuration */
#define AFIO_EXTCFG1_EXTI3_PA                ((uint16_t)0x0000)            /*!< PA[3] pin */
#define AFIO_EXTCFG1_EXTI3_PB                ((uint16_t)0x1000)            /*!< PB[3] pin */
#define AFIO_EXTCFG1_EXTI3_PC                ((uint16_t)0x2000)            /*!< PC[3] pin */
#define AFIO_EXTCFG1_EXTI3_PD                ((uint16_t)0x3000)            /*!< PD[3] pin */
#define AFIO_EXTCFG1_EXTI3_PE                ((uint16_t)0x4000)            /*!< PE[3] pin */

/**
  * @brief AFIO_EXTCFG2 register Bit definition
  */
#define AFIO_EXTCFG2_EXTI4                   ((uint16_t)0x000F)            /*!< EXTI 4 */
#define AFIO_EXTCFG2_EXTI5                   ((uint16_t)0x00F0)            /*!< EXTI 5 */
#define AFIO_EXTCFG2_EXTI6                   ((uint16_t)0x0F00)            /*!< EXTI 6 */
#define AFIO_EXTCFG2_EXTI7                   ((uint16_t)0xF000)            /*!< EXTI 7 */

/*!< EXTI4 configuration */
#define AFIO_EXTCFG2_EXTI4_PA                ((uint16_t)0x0000)            /*!< PA[4] pin */
#define AFIO_EXTCFG2_EXTI4_PB                ((uint16_t)0x0001)            /*!< PB[4] pin */
#define AFIO_EXTCFG2_EXTI4_PC                ((uint16_t)0x0002)            /*!< PC[4] pin */
#define AFIO_EXTCFG2_EXTI4_PD                ((uint16_t)0x0003)            /*!< PD[4] pin */
#define AFIO_EXTCFG2_EXTI4_PE                ((uint16_t)0x0004)            /*!< PE[4] pin */

/* EXTI5 configuration */
#define AFIO_EXTCFG2_EXTI5_PA                ((uint16_t)0x0000)            /*!< PA[5] pin */
#define AFIO_EXTCFG2_EXTI5_PB                ((uint16_t)0x0010)            /*!< PB[5] pin */
#define AFIO_EXTCFG2_EXTI5_PC                ((uint16_t)0x0020)            /*!< PC[5] pin */
#define AFIO_EXTCFG2_EXTI5_PD                ((uint16_t)0x0030)            /*!< PD[5] pin */
#define AFIO_EXTCFG2_EXTI5_PE                ((uint16_t)0x0040)            /*!< PE[5] pin */

/*!< EXTI6 configuration */  
#define AFIO_EXTCFG2_EXTI6_PA                ((uint16_t)0x0000)            /*!< PA[6] pin */
#define AFIO_EXTCFG2_EXTI6_PB                ((uint16_t)0x0100)            /*!< PB[6] pin */
#define AFIO_EXTCFG2_EXTI6_PC                ((uint16_t)0x0200)            /*!< PC[6] pin */
#define AFIO_EXTCFG2_EXTI6_PD                ((uint16_t)0x0300)            /*!< PD[6] pin */
#define AFIO_EXTCFG2_EXTI6_PE                ((uint16_t)0x0400)            /*!< PE[6] pin */

/*!< EXTI7 configuration */
#define AFIO_EXTCFG2_EXTI7_PA                ((uint16_t)0x0000)            /*!< PA[7] pin */
#define AFIO_EXTCFG2_EXTI7_PB                ((uint16_t)0x1000)            /*!< PB[7] pin */
#define AFIO_EXTCFG2_EXTI7_PC                ((uint16_t)0x2000)            /*!< PC[7] pin */
#define AFIO_EXTCFG2_EXTI7_PD                ((uint16_t)0x3000)            /*!< PD[7] pin */
#define AFIO_EXTCFG2_EXTI7_PE                ((uint16_t)0x4000)            /*!< PE[7] pin */

/**
  * @brief AFIO_EXTCFG3 register Bit definition
  */
#define AFIO_EXTCFG3_EXTI8                   ((uint16_t)0x000F)            /*!< EXTI 8 */
#define AFIO_EXTCFG3_EXTI9                   ((uint16_t)0x00F0)            /*!< EXTI 9 */
#define AFIO_EXTCFG3_EXTI10                  ((uint16_t)0x0F00)            /*!< EXTI 10 */
#define AFIO_EXTCFG3_EXTI11                  ((uint16_t)0xF000)            /*!< EXTI 11 */

/*!< EXTI8 configuration */
#define AFIO_EXTCFG3_EXTI8_PA                ((uint16_t)0x0000)            /*!< PA[8] pin */
#define AFIO_EXTCFG3_EXTI8_PB                ((uint16_t)0x0001)            /*!< PB[8] pin */
#define AFIO_EXTCFG3_EXTI8_PC                ((uint16_t)0x0002)            /*!< PC[8] pin */
#define AFIO_EXTCFG3_EXTI8_PD                ((uint16_t)0x0003)            /*!< PD[8] pin */
#define AFIO_EXTCFG3_EXTI8_PE                ((uint16_t)0x0004)            /*!< PE[8] pin */

/*!< EXTI9 configuration */
#define AFIO_EXTCFG3_EXTI9_PA                ((uint16_t)0x0000)            /*!< PA[9] pin */
#define AFIO_EXTCFG3_EXTI9_PB                ((uint16_t)0x0010)            /*!< PB[9] pin */
#define AFIO_EXTCFG3_EXTI9_PC                ((uint16_t)0x0020)            /*!< PC[9] pin */
#define AFIO_EXTCFG3_EXTI9_PD                ((uint16_t)0x0030)            /*!< PD[9] pin */
#define AFIO_EXTCFG3_EXTI9_PE                ((uint16_t)0x0040)            /*!< PE[9] pin */

/*!< EXTI10 configuration */  
#define AFIO_EXTCFG3_EXTI10_PA               ((uint16_t)0x0000)            /*!< PA[10] pin */
#define AFIO_EXTCFG3_EXTI10_PB               ((uint16_t)0x0100)            /*!< PB[10] pin */
#define AFIO_EXTCFG3_EXTI10_PC               ((uint16_t)0x0200)            /*!< PC[10] pin */
#define AFIO_EXTCFG3_EXTI10_PD               ((uint16_t)0x0300)            /*!< PD[10] pin */
#define AFIO_EXTCFG3_EXTI10_PE               ((uint16_t)0x0400)            /*!< PE[10] pin */

/*!< EXTI11 configuration */
#define AFIO_EXTCFG3_EXTI11_PA               ((uint16_t)0x0000)            /*!< PA[11] pin */
#define AFIO_EXTCFG3_EXTI11_PB               ((uint16_t)0x1000)            /*!< PB[11] pin */
#define AFIO_EXTCFG3_EXTI11_PC               ((uint16_t)0x2000)            /*!< PC[11] pin */
#define AFIO_EXTCFG3_EXTI11_PD               ((uint16_t)0x3000)            /*!< PD[11] pin */
#define AFIO_EXTCFG3_EXTI11_PE               ((uint16_t)0x4000)            /*!< PE[11] pin */

/**
  * @brief AFIO_EXTCFG4 register Bit definition
  */
#define AFIO_EXTCFG4_EXTI12                  ((uint16_t)0x000F)            /*!< EXTI 12 */
#define AFIO_EXTCFG4_EXTI13                  ((uint16_t)0x00F0)            /*!< EXTI 13 */
#define AFIO_EXTCFG4_EXTI14                  ((uint16_t)0x0F00)            /*!< EXTI 14 */
#define AFIO_EXTCFG4_EXTI15                  ((uint16_t)0xF000)            /*!< EXTI 15 */

/* EXTI12 configuration */
#define AFIO_EXTCFG4_EXTI12_PA               ((uint16_t)0x0000)            /*!< PA[12] pin */
#define AFIO_EXTCFG4_EXTI12_PB               ((uint16_t)0x0001)            /*!< PB[12] pin */
#define AFIO_EXTCFG4_EXTI12_PC               ((uint16_t)0x0002)            /*!< PC[12] pin */
#define AFIO_EXTCFG4_EXTI12_PD               ((uint16_t)0x0003)            /*!< PD[12] pin */
#define AFIO_EXTCFG4_EXTI12_PE               ((uint16_t)0x0004)            /*!< PE[12] pin */

/* EXTI13 configuration */
#define AFIO_EXTCFG4_EXTI13_PA               ((uint16_t)0x0000)            /*!< PA[13] pin */
#define AFIO_EXTCFG4_EXTI13_PB               ((uint16_t)0x0010)            /*!< PB[13] pin */
#define AFIO_EXTCFG4_EXTI13_PC               ((uint16_t)0x0020)            /*!< PC[13] pin */
#define AFIO_EXTCFG4_EXTI13_PD               ((uint16_t)0x0030)            /*!< PD[13] pin */
#define AFIO_EXTCFG4_EXTI13_PE               ((uint16_t)0x0040)            /*!< PE[13] pin */

/*!< EXTI14 configuration */  
#define AFIO_EXTCFG4_EXTI14_PA               ((uint16_t)0x0000)            /*!< PA[14] pin */
#define AFIO_EXTCFG4_EXTI14_PB               ((uint16_t)0x0100)            /*!< PB[14] pin */
#define AFIO_EXTCFG4_EXTI14_PC               ((uint16_t)0x0200)            /*!< PC[14] pin */
#define AFIO_EXTCFG4_EXTI14_PD               ((uint16_t)0x0300)            /*!< PD[14] pin */
#define AFIO_EXTCFG4_EXTI14_PE               ((uint16_t)0x0400)            /*!< PE[14] pin */

/*!< EXTI15 configuration */
#define AFIO_EXTCFG4_EXTI15_PA               ((uint16_t)0x0000)            /*!< PA[15] pin */
#define AFIO_EXTCFG4_EXTI15_PB               ((uint16_t)0x1000)            /*!< PB[15] pin */
#define AFIO_EXTCFG4_EXTI15_PC               ((uint16_t)0x2000)            /*!< PC[15] pin */
#define AFIO_EXTCFG4_EXTI15_PD               ((uint16_t)0x3000)            /*!< PD[15] pin */
#define AFIO_EXTCFG4_EXTI15_PE               ((uint16_t)0x4000)            /*!< PE[15] pin */


/******************************************************************************/
/*                                                                            */
/*                               SystemTick                                   */
/*                                                                            */
/******************************************************************************/

/**
  * @brief SysTick_CTRL register Bit definition
  */
#define  SysTick_CTRL_ENABLE                 ((uint32_t)0x00000001)        /*!< Counter enable */
#define  SysTick_CTRL_TICKINT                ((uint32_t)0x00000002)        /*!< Counting down to 0 pends the SysTick handler */
#define  SysTick_CTRL_CLKSOURCE              ((uint32_t)0x00000004)        /*!< Clock source */
#define  SysTick_CTRL_COUNTFLAG              ((uint32_t)0x00010000)        /*!< Count Flag */

/**
  * @brief SysTick_LOAD register Bit definition
  */
#define  SysTick_LOAD_RELOAD                 ((uint32_t)0x00FFFFFF)        /*!< Value to load into the SysTick Current Value Register when the counter reaches 0 */

/**
  * @brief SysTick_VAL register Bit definition
  */
#define  SysTick_VAL_CURRENT                 ((uint32_t)0x00FFFFFF)        /*!< Current value at the time the register is accessed */

/**
  * @brief SysTick_CALIB register Bit definition
  */
#define  SysTick_CALIB_TENMS                 ((uint32_t)0x00FFFFFF)        /*!< Reload value to use for 10ms timing */
#define  SysTick_CALIB_SKEW                  ((uint32_t)0x40000000)        /*!< Calibration value is not exactly 10 ms */
#define  SysTick_CALIB_NOREF                 ((uint32_t)0x80000000)        /*!< The reference clock is not provided */

/******************************************************************************/
/*                                                                            */
/*                  Nested Vectored Interrupt Controller                      */
/*                                                                            */
/******************************************************************************/

/**
  * @brief NVIC_ISER register Bit definition
  */
#define  NVIC_ISER_SETENA                    ((uint32_t)0xFFFFFFFF)        /*!< Interrupt set enable bits */
#define  NVIC_ISER_SETENA_0                  ((uint32_t)0x00000001)        /*!< bit 0 */
#define  NVIC_ISER_SETENA_1                  ((uint32_t)0x00000002)        /*!< bit 1 */
#define  NVIC_ISER_SETENA_2                  ((uint32_t)0x00000004)        /*!< bit 2 */
#define  NVIC_ISER_SETENA_3                  ((uint32_t)0x00000008)        /*!< bit 3 */
#define  NVIC_ISER_SETENA_4                  ((uint32_t)0x00000010)        /*!< bit 4 */
#define  NVIC_ISER_SETENA_5                  ((uint32_t)0x00000020)        /*!< bit 5 */
#define  NVIC_ISER_SETENA_6                  ((uint32_t)0x00000040)        /*!< bit 6 */
#define  NVIC_ISER_SETENA_7                  ((uint32_t)0x00000080)        /*!< bit 7 */
#define  NVIC_ISER_SETENA_8                  ((uint32_t)0x00000100)        /*!< bit 8 */
#define  NVIC_ISER_SETENA_9                  ((uint32_t)0x00000200)        /*!< bit 9 */
#define  NVIC_ISER_SETENA_10                 ((uint32_t)0x00000400)        /*!< bit 10 */
#define  NVIC_ISER_SETENA_11                 ((uint32_t)0x00000800)        /*!< bit 11 */
#define  NVIC_ISER_SETENA_12                 ((uint32_t)0x00001000)        /*!< bit 12 */
#define  NVIC_ISER_SETENA_13                 ((uint32_t)0x00002000)        /*!< bit 13 */
#define  NVIC_ISER_SETENA_14                 ((uint32_t)0x00004000)        /*!< bit 14 */
#define  NVIC_ISER_SETENA_15                 ((uint32_t)0x00008000)        /*!< bit 15 */
#define  NVIC_ISER_SETENA_16                 ((uint32_t)0x00010000)        /*!< bit 16 */
#define  NVIC_ISER_SETENA_17                 ((uint32_t)0x00020000)        /*!< bit 17 */
#define  NVIC_ISER_SETENA_18                 ((uint32_t)0x00040000)        /*!< bit 18 */
#define  NVIC_ISER_SETENA_19                 ((uint32_t)0x00080000)        /*!< bit 19 */
#define  NVIC_ISER_SETENA_20                 ((uint32_t)0x00100000)        /*!< bit 20 */
#define  NVIC_ISER_SETENA_21                 ((uint32_t)0x00200000)        /*!< bit 21 */
#define  NVIC_ISER_SETENA_22                 ((uint32_t)0x00400000)        /*!< bit 22 */
#define  NVIC_ISER_SETENA_23                 ((uint32_t)0x00800000)        /*!< bit 23 */
#define  NVIC_ISER_SETENA_24                 ((uint32_t)0x01000000)        /*!< bit 24 */
#define  NVIC_ISER_SETENA_25                 ((uint32_t)0x02000000)        /*!< bit 25 */
#define  NVIC_ISER_SETENA_26                 ((uint32_t)0x04000000)        /*!< bit 26 */
#define  NVIC_ISER_SETENA_27                 ((uint32_t)0x08000000)        /*!< bit 27 */
#define  NVIC_ISER_SETENA_28                 ((uint32_t)0x10000000)        /*!< bit 28 */
#define  NVIC_ISER_SETENA_29                 ((uint32_t)0x20000000)        /*!< bit 29 */
#define  NVIC_ISER_SETENA_30                 ((uint32_t)0x40000000)        /*!< bit 30 */
#define  NVIC_ISER_SETENA_31                 ((uint32_t)0x80000000)        /*!< bit 31 */

/**
  * @brief NVIC_ICER register Bit definition
  */
#define  NVIC_ICER_CLRENA                   ((uint32_t)0xFFFFFFFF)        /*!< Interrupt clear-enable bits */
#define  NVIC_ICER_CLRENA_0                  ((uint32_t)0x00000001)        /*!< bit 0 */
#define  NVIC_ICER_CLRENA_1                  ((uint32_t)0x00000002)        /*!< bit 1 */
#define  NVIC_ICER_CLRENA_2                  ((uint32_t)0x00000004)        /*!< bit 2 */
#define  NVIC_ICER_CLRENA_3                  ((uint32_t)0x00000008)        /*!< bit 3 */
#define  NVIC_ICER_CLRENA_4                  ((uint32_t)0x00000010)        /*!< bit 4 */
#define  NVIC_ICER_CLRENA_5                  ((uint32_t)0x00000020)        /*!< bit 5 */
#define  NVIC_ICER_CLRENA_6                  ((uint32_t)0x00000040)        /*!< bit 6 */
#define  NVIC_ICER_CLRENA_7                  ((uint32_t)0x00000080)        /*!< bit 7 */
#define  NVIC_ICER_CLRENA_8                  ((uint32_t)0x00000100)        /*!< bit 8 */
#define  NVIC_ICER_CLRENA_9                  ((uint32_t)0x00000200)        /*!< bit 9 */
#define  NVIC_ICER_CLRENA_10                 ((uint32_t)0x00000400)        /*!< bit 10 */
#define  NVIC_ICER_CLRENA_11                 ((uint32_t)0x00000800)        /*!< bit 11 */
#define  NVIC_ICER_CLRENA_12                 ((uint32_t)0x00001000)        /*!< bit 12 */
#define  NVIC_ICER_CLRENA_13                 ((uint32_t)0x00002000)        /*!< bit 13 */
#define  NVIC_ICER_CLRENA_14                 ((uint32_t)0x00004000)        /*!< bit 14 */
#define  NVIC_ICER_CLRENA_15                 ((uint32_t)0x00008000)        /*!< bit 15 */
#define  NVIC_ICER_CLRENA_16                 ((uint32_t)0x00010000)        /*!< bit 16 */
#define  NVIC_ICER_CLRENA_17                 ((uint32_t)0x00020000)        /*!< bit 17 */
#define  NVIC_ICER_CLRENA_18                 ((uint32_t)0x00040000)        /*!< bit 18 */
#define  NVIC_ICER_CLRENA_19                 ((uint32_t)0x00080000)        /*!< bit 19 */
#define  NVIC_ICER_CLRENA_20                 ((uint32_t)0x00100000)        /*!< bit 20 */
#define  NVIC_ICER_CLRENA_21                 ((uint32_t)0x00200000)        /*!< bit 21 */
#define  NVIC_ICER_CLRENA_22                 ((uint32_t)0x00400000)        /*!< bit 22 */
#define  NVIC_ICER_CLRENA_23                 ((uint32_t)0x00800000)        /*!< bit 23 */
#define  NVIC_ICER_CLRENA_24                 ((uint32_t)0x01000000)        /*!< bit 24 */
#define  NVIC_ICER_CLRENA_25                 ((uint32_t)0x02000000)        /*!< bit 25 */
#define  NVIC_ICER_CLRENA_26                 ((uint32_t)0x04000000)        /*!< bit 26 */
#define  NVIC_ICER_CLRENA_27                 ((uint32_t)0x08000000)        /*!< bit 27 */
#define  NVIC_ICER_CLRENA_28                 ((uint32_t)0x10000000)        /*!< bit 28 */
#define  NVIC_ICER_CLRENA_29                 ((uint32_t)0x20000000)        /*!< bit 29 */
#define  NVIC_ICER_CLRENA_30                 ((uint32_t)0x40000000)        /*!< bit 30 */
#define  NVIC_ICER_CLRENA_31                 ((uint32_t)0x80000000)        /*!< bit 31 */

/**
  * @brief NVIC_ISPR register Bit definition
  */
#define  NVIC_ISPR_SETPEND                   ((uint32_t)0xFFFFFFFF)        /*!< Interrupt set-pending bits */
#define  NVIC_ISPR_SETPEND_0                 ((uint32_t)0x00000001)        /*!< bit 0 */
#define  NVIC_ISPR_SETPEND_1                 ((uint32_t)0x00000002)        /*!< bit 1 */
#define  NVIC_ISPR_SETPEND_2                 ((uint32_t)0x00000004)        /*!< bit 2 */
#define  NVIC_ISPR_SETPEND_3                 ((uint32_t)0x00000008)        /*!< bit 3 */
#define  NVIC_ISPR_SETPEND_4                 ((uint32_t)0x00000010)        /*!< bit 4 */
#define  NVIC_ISPR_SETPEND_5                 ((uint32_t)0x00000020)        /*!< bit 5 */
#define  NVIC_ISPR_SETPEND_6                 ((uint32_t)0x00000040)        /*!< bit 6 */
#define  NVIC_ISPR_SETPEND_7                 ((uint32_t)0x00000080)        /*!< bit 7 */
#define  NVIC_ISPR_SETPEND_8                 ((uint32_t)0x00000100)        /*!< bit 8 */
#define  NVIC_ISPR_SETPEND_9                 ((uint32_t)0x00000200)        /*!< bit 9 */
#define  NVIC_ISPR_SETPEND_10                ((uint32_t)0x00000400)        /*!< bit 10 */
#define  NVIC_ISPR_SETPEND_11                ((uint32_t)0x00000800)        /*!< bit 11 */
#define  NVIC_ISPR_SETPEND_12                ((uint32_t)0x00001000)        /*!< bit 12 */
#define  NVIC_ISPR_SETPEND_13                ((uint32_t)0x00002000)        /*!< bit 13 */
#define  NVIC_ISPR_SETPEND_14                ((uint32_t)0x00004000)        /*!< bit 14 */
#define  NVIC_ISPR_SETPEND_15                ((uint32_t)0x00008000)        /*!< bit 15 */
#define  NVIC_ISPR_SETPEND_16                ((uint32_t)0x00010000)        /*!< bit 16 */
#define  NVIC_ISPR_SETPEND_17                ((uint32_t)0x00020000)        /*!< bit 17 */
#define  NVIC_ISPR_SETPEND_18                ((uint32_t)0x00040000)        /*!< bit 18 */
#define  NVIC_ISPR_SETPEND_19                ((uint32_t)0x00080000)        /*!< bit 19 */
#define  NVIC_ISPR_SETPEND_20                ((uint32_t)0x00100000)        /*!< bit 20 */
#define  NVIC_ISPR_SETPEND_21                ((uint32_t)0x00200000)        /*!< bit 21 */
#define  NVIC_ISPR_SETPEND_22                ((uint32_t)0x00400000)        /*!< bit 22 */
#define  NVIC_ISPR_SETPEND_23                ((uint32_t)0x00800000)        /*!< bit 23 */
#define  NVIC_ISPR_SETPEND_24                ((uint32_t)0x01000000)        /*!< bit 24 */
#define  NVIC_ISPR_SETPEND_25                ((uint32_t)0x02000000)        /*!< bit 25 */
#define  NVIC_ISPR_SETPEND_26                ((uint32_t)0x04000000)        /*!< bit 26 */
#define  NVIC_ISPR_SETPEND_27                ((uint32_t)0x08000000)        /*!< bit 27 */
#define  NVIC_ISPR_SETPEND_28                ((uint32_t)0x10000000)        /*!< bit 28 */
#define  NVIC_ISPR_SETPEND_29                ((uint32_t)0x20000000)        /*!< bit 29 */
#define  NVIC_ISPR_SETPEND_30                ((uint32_t)0x40000000)        /*!< bit 30 */
#define  NVIC_ISPR_SETPEND_31                ((uint32_t)0x80000000)        /*!< bit 31 */

/**
  * @brief NVIC_ICPR register Bit definition
  */
#define  NVIC_ICPR_CLRPEND                   ((uint32_t)0xFFFFFFFF)        /*!< Interrupt clear-pending bits */
#define  NVIC_ICPR_CLRPEND_0                 ((uint32_t)0x00000001)        /*!< bit 0 */
#define  NVIC_ICPR_CLRPEND_1                 ((uint32_t)0x00000002)        /*!< bit 1 */
#define  NVIC_ICPR_CLRPEND_2                 ((uint32_t)0x00000004)        /*!< bit 2 */
#define  NVIC_ICPR_CLRPEND_3                 ((uint32_t)0x00000008)        /*!< bit 3 */
#define  NVIC_ICPR_CLRPEND_4                 ((uint32_t)0x00000010)        /*!< bit 4 */
#define  NVIC_ICPR_CLRPEND_5                 ((uint32_t)0x00000020)        /*!< bit 5 */
#define  NVIC_ICPR_CLRPEND_6                 ((uint32_t)0x00000040)        /*!< bit 6 */
#define  NVIC_ICPR_CLRPEND_7                 ((uint32_t)0x00000080)        /*!< bit 7 */
#define  NVIC_ICPR_CLRPEND_8                 ((uint32_t)0x00000100)        /*!< bit 8 */
#define  NVIC_ICPR_CLRPEND_9                 ((uint32_t)0x00000200)        /*!< bit 9 */
#define  NVIC_ICPR_CLRPEND_10                ((uint32_t)0x00000400)        /*!< bit 10 */
#define  NVIC_ICPR_CLRPEND_11                ((uint32_t)0x00000800)        /*!< bit 11 */
#define  NVIC_ICPR_CLRPEND_12                ((uint32_t)0x00001000)        /*!< bit 12 */
#define  NVIC_ICPR_CLRPEND_13                ((uint32_t)0x00002000)        /*!< bit 13 */
#define  NVIC_ICPR_CLRPEND_14                ((uint32_t)0x00004000)        /*!< bit 14 */
#define  NVIC_ICPR_CLRPEND_15                ((uint32_t)0x00008000)        /*!< bit 15 */
#define  NVIC_ICPR_CLRPEND_16                ((uint32_t)0x00010000)        /*!< bit 16 */
#define  NVIC_ICPR_CLRPEND_17                ((uint32_t)0x00020000)        /*!< bit 17 */
#define  NVIC_ICPR_CLRPEND_18                ((uint32_t)0x00040000)        /*!< bit 18 */
#define  NVIC_ICPR_CLRPEND_19                ((uint32_t)0x00080000)        /*!< bit 19 */
#define  NVIC_ICPR_CLRPEND_20                ((uint32_t)0x00100000)        /*!< bit 20 */
#define  NVIC_ICPR_CLRPEND_21                ((uint32_t)0x00200000)        /*!< bit 21 */
#define  NVIC_ICPR_CLRPEND_22                ((uint32_t)0x00400000)        /*!< bit 22 */
#define  NVIC_ICPR_CLRPEND_23                ((uint32_t)0x00800000)        /*!< bit 23 */
#define  NVIC_ICPR_CLRPEND_24                ((uint32_t)0x01000000)        /*!< bit 24 */
#define  NVIC_ICPR_CLRPEND_25                ((uint32_t)0x02000000)        /*!< bit 25 */
#define  NVIC_ICPR_CLRPEND_26                ((uint32_t)0x04000000)        /*!< bit 26 */
#define  NVIC_ICPR_CLRPEND_27                ((uint32_t)0x08000000)        /*!< bit 27 */
#define  NVIC_ICPR_CLRPEND_28                ((uint32_t)0x10000000)        /*!< bit 28 */
#define  NVIC_ICPR_CLRPEND_29                ((uint32_t)0x20000000)        /*!< bit 29 */
#define  NVIC_ICPR_CLRPEND_30                ((uint32_t)0x40000000)        /*!< bit 30 */
#define  NVIC_ICPR_CLRPEND_31                ((uint32_t)0x80000000)        /*!< bit 31 */

/**
  * @brief NVIC_IABR register Bit definition
  */
#define  NVIC_IABR_ACTIVE                    ((uint32_t)0xFFFFFFFF)        /*!< Interrupt active flags */
#define  NVIC_IABR_ACTIVE_0                  ((uint32_t)0x00000001)        /*!< bit 0 */
#define  NVIC_IABR_ACTIVE_1                  ((uint32_t)0x00000002)        /*!< bit 1 */
#define  NVIC_IABR_ACTIVE_2                  ((uint32_t)0x00000004)        /*!< bit 2 */
#define  NVIC_IABR_ACTIVE_3                  ((uint32_t)0x00000008)        /*!< bit 3 */
#define  NVIC_IABR_ACTIVE_4                  ((uint32_t)0x00000010)        /*!< bit 4 */
#define  NVIC_IABR_ACTIVE_5                  ((uint32_t)0x00000020)        /*!< bit 5 */
#define  NVIC_IABR_ACTIVE_6                  ((uint32_t)0x00000040)        /*!< bit 6 */
#define  NVIC_IABR_ACTIVE_7                  ((uint32_t)0x00000080)        /*!< bit 7 */
#define  NVIC_IABR_ACTIVE_8                  ((uint32_t)0x00000100)        /*!< bit 8 */
#define  NVIC_IABR_ACTIVE_9                  ((uint32_t)0x00000200)        /*!< bit 9 */
#define  NVIC_IABR_ACTIVE_10                 ((uint32_t)0x00000400)        /*!< bit 10 */
#define  NVIC_IABR_ACTIVE_11                 ((uint32_t)0x00000800)        /*!< bit 11 */
#define  NVIC_IABR_ACTIVE_12                 ((uint32_t)0x00001000)        /*!< bit 12 */
#define  NVIC_IABR_ACTIVE_13                 ((uint32_t)0x00002000)        /*!< bit 13 */
#define  NVIC_IABR_ACTIVE_14                 ((uint32_t)0x00004000)        /*!< bit 14 */
#define  NVIC_IABR_ACTIVE_15                 ((uint32_t)0x00008000)        /*!< bit 15 */
#define  NVIC_IABR_ACTIVE_16                 ((uint32_t)0x00010000)        /*!< bit 16 */
#define  NVIC_IABR_ACTIVE_17                 ((uint32_t)0x00020000)        /*!< bit 17 */
#define  NVIC_IABR_ACTIVE_18                 ((uint32_t)0x00040000)        /*!< bit 18 */
#define  NVIC_IABR_ACTIVE_19                 ((uint32_t)0x00080000)        /*!< bit 19 */
#define  NVIC_IABR_ACTIVE_20                 ((uint32_t)0x00100000)        /*!< bit 20 */
#define  NVIC_IABR_ACTIVE_21                 ((uint32_t)0x00200000)        /*!< bit 21 */
#define  NVIC_IABR_ACTIVE_22                 ((uint32_t)0x00400000)        /*!< bit 22 */
#define  NVIC_IABR_ACTIVE_23                 ((uint32_t)0x00800000)        /*!< bit 23 */
#define  NVIC_IABR_ACTIVE_24                 ((uint32_t)0x01000000)        /*!< bit 24 */
#define  NVIC_IABR_ACTIVE_25                 ((uint32_t)0x02000000)        /*!< bit 25 */
#define  NVIC_IABR_ACTIVE_26                 ((uint32_t)0x04000000)        /*!< bit 26 */
#define  NVIC_IABR_ACTIVE_27                 ((uint32_t)0x08000000)        /*!< bit 27 */
#define  NVIC_IABR_ACTIVE_28                 ((uint32_t)0x10000000)        /*!< bit 28 */
#define  NVIC_IABR_ACTIVE_29                 ((uint32_t)0x20000000)        /*!< bit 29 */
#define  NVIC_IABR_ACTIVE_30                 ((uint32_t)0x40000000)        /*!< bit 30 */
#define  NVIC_IABR_ACTIVE_31                 ((uint32_t)0x80000000)        /*!< bit 31 */

/**
  * @brief NVIC_PRI0 register Bit definition
  */
#define  NVIC_IPR0_PRI_0                     ((uint32_t)0x000000FF)        /*!< Priority of interrupt 0 */
#define  NVIC_IPR0_PRI_1                     ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 1 */
#define  NVIC_IPR0_PRI_2                     ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 2 */
#define  NVIC_IPR0_PRI_3                     ((uint32_t)0xFF000000)        /*!< Priority of interrupt 3 */

/**
  * @brief NVIC_PRI1 register Bit definition
  */
#define  NVIC_IPR1_PRI_4                     ((uint32_t)0x000000FF)        /*!< Priority of interrupt 4 */
#define  NVIC_IPR1_PRI_5                     ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 5 */
#define  NVIC_IPR1_PRI_6                     ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 6 */
#define  NVIC_IPR1_PRI_7                     ((uint32_t)0xFF000000)        /*!< Priority of interrupt 7 */

/**
  * @brief NVIC_PRI2 register Bit definition
  */
#define  NVIC_IPR2_PRI_8                     ((uint32_t)0x000000FF)        /*!< Priority of interrupt 8 */
#define  NVIC_IPR2_PRI_9                     ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 9 */
#define  NVIC_IPR2_PRI_10                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 10 */
#define  NVIC_IPR2_PRI_11                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 11 */

/**
  * @brief NVIC_PRI3 register Bit definition
  */
#define  NVIC_IPR3_PRI_12                    ((uint32_t)0x000000FF)        /*!< Priority of interrupt 12 */
#define  NVIC_IPR3_PRI_13                    ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 13 */
#define  NVIC_IPR3_PRI_14                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 14 */
#define  NVIC_IPR3_PRI_15                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 15 */

/**
  * @brief NVIC_PRI4 register Bit definition
  */
#define  NVIC_IPR4_PRI_16                    ((uint32_t)0x000000FF)        /*!< Priority of interrupt 16 */
#define  NVIC_IPR4_PRI_17                    ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 17 */
#define  NVIC_IPR4_PRI_18                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 18 */
#define  NVIC_IPR4_PRI_19                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 19 */

/**
  * @brief NVIC_PRI5 register Bit definition
  */
#define  NVIC_IPR5_PRI_20                    ((uint32_t)0x000000FF)        /*!< Priority of interrupt 20 */
#define  NVIC_IPR5_PRI_21                    ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 21 */
#define  NVIC_IPR5_PRI_22                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 22 */
#define  NVIC_IPR5_PRI_23                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 23 */

/**
  * @brief NVIC_PRI6 register Bit definition
  */
#define  NVIC_IPR6_PRI_24                    ((uint32_t)0x000000FF)        /*!< Priority of interrupt 24 */
#define  NVIC_IPR6_PRI_25                    ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 25 */
#define  NVIC_IPR6_PRI_26                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 26 */
#define  NVIC_IPR6_PRI_27                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 27 */

/**
  * @brief NVIC_PRI7 register Bit definition
  */
#define  NVIC_IPR7_PRI_28                    ((uint32_t)0x000000FF)        /*!< Priority of interrupt 28 */
#define  NVIC_IPR7_PRI_29                    ((uint32_t)0x0000FF00)        /*!< Priority of interrupt 29 */
#define  NVIC_IPR7_PRI_30                    ((uint32_t)0x00FF0000)        /*!< Priority of interrupt 30 */
#define  NVIC_IPR7_PRI_31                    ((uint32_t)0xFF000000)        /*!< Priority of interrupt 31 */

/**
  * @brief SCB_CPUID register Bit definition
  */
#define  SCB_CPUID_REVISION                  ((uint32_t)0x0000000F)        /*!< Implementation defined revision number */
#define  SCB_CPUID_PARTNO                    ((uint32_t)0x0000FFF0)        /*!< Number of processor within family */
#define  SCB_CPUID_Constant                  ((uint32_t)0x000F0000)        /*!< Reads as 0x0F */
#define  SCB_CPUID_VARIANT                   ((uint32_t)0x00F00000)        /*!< Implementation defined variant number */
#define  SCB_CPUID_IMPLEMENTER               ((uint32_t)0xFF000000)        /*!< Implementer code. ARM is 0x41 */

/**
  * @brief SCB_ICSR register Bit definition
  */
#define  SCB_ICSR_VECTACTIVE                 ((uint32_t)0x000001FF)        /*!< Active ISR number field */
#define  SCB_ICSR_RETTOBASE                  ((uint32_t)0x00000800)        /*!< All active exceptions minus the IPSR_current_exception yields the empty set */
#define  SCB_ICSR_VECTPENDING                ((uint32_t)0x003FF000)        /*!< Pending ISR number field */
#define  SCB_ICSR_ISRPENDING                 ((uint32_t)0x00400000)        /*!< Interrupt pending flag */
#define  SCB_ICSR_ISRPREEMPT                 ((uint32_t)0x00800000)        /*!< It indicates that a pending interrupt becomes active in the next running cycle */
#define  SCB_ICSR_PENDSTCLR                  ((uint32_t)0x02000000)        /*!< Clear pending SysTick bit */
#define  SCB_ICSR_PENDSTSET                  ((uint32_t)0x04000000)        /*!< Set pending SysTick bit */
#define  SCB_ICSR_PENDSVCLR                  ((uint32_t)0x08000000)        /*!< Clear pending pendSV bit */
#define  SCB_ICSR_PENDSVSET                  ((uint32_t)0x10000000)        /*!< Set pending pendSV bit */
#define  SCB_ICSR_NMIPENDSET                 ((uint32_t)0x80000000)        /*!< Set pending NMI bit */

/**
  * @brief SCB_VTOR register Bit definition
  */
#define  SCB_VTOR_TBLOFF                     ((uint32_t)0x1FFFFF80)        /*!< Vector table base offset field */
#define  SCB_VTOR_TBLBASE                    ((uint32_t)0x20000000)        /*!< Table base in code(0) or RAM(1) */

/**
  * @brief SCB_AIRCR register Bit definition
  */
#define  SCB_AIRCR_VECTRESET                 ((uint32_t)0x00000001)        /*!< System Reset bit */
#define  SCB_AIRCR_VECTCLRACTIVE             ((uint32_t)0x00000002)        /*!< Clear active vector bit */
#define  SCB_AIRCR_SYSRESETREQ               ((uint32_t)0x00000004)        /*!< Requests chip control logic to generate a reset */

#define  SCB_AIRCR_PRIGROUP                  ((uint32_t)0x00000700)        /*!< PRIGROUP[2:0] bits (Priority group) */
#define  SCB_AIRCR_PRIGROUP_0                ((uint32_t)0x00000100)        /*!< Bit 0 */
#define  SCB_AIRCR_PRIGROUP_1                ((uint32_t)0x00000200)        /*!< Bit 1 */
#define  SCB_AIRCR_PRIGROUP_2                ((uint32_t)0x00000400)        /*!< Bit 2  */

/* prority group definition */
#define  SCB_AIRCR_PRIGROUP0                 ((uint32_t)0x00000000)        /*!< Priority group=0 (7 bits of pre-emption priority, 1 bit of subpriority) */
#define  SCB_AIRCR_PRIGROUP1                 ((uint32_t)0x00000100)        /*!< Priority group=1 (6 bits of pre-emption priority, 2 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP2                 ((uint32_t)0x00000200)        /*!< Priority group=2 (5 bits of pre-emption priority, 3 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP3                 ((uint32_t)0x00000300)        /*!< Priority group=3 (4 bits of pre-emption priority, 4 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP4                 ((uint32_t)0x00000400)        /*!< Priority group=4 (3 bits of pre-emption priority, 5 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP5                 ((uint32_t)0x00000500)        /*!< Priority group=5 (2 bits of pre-emption priority, 6 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP6                 ((uint32_t)0x00000600)        /*!< Priority group=6 (1 bit of pre-emption priority, 7 bits of subpriority) */
#define  SCB_AIRCR_PRIGROUP7                 ((uint32_t)0x00000700)        /*!< Priority group=7 (no pre-emption priority, 8 bits of subpriority) */

#define  SCB_AIRCR_ENDIANESS                 ((uint32_t)0x00008000)        /*!< Data endianness bit */
#define  SCB_AIRCR_VECTKEY                   ((uint32_t)0xFFFF0000)        /*!< Register key (VECTKEY) - Reads as 0xFA05 (VECTKEYSTAT) */

/**
  * @brief SCB_SCR register Bit definition
  */
#define  SCB_SCR_SLEEPONEXIT                 ((uint8_t)0x02)               /*!< Sleep on exit bit */
#define  SCB_SCR_SLEEPDEEP                   ((uint8_t)0x04)               /*!< Sleep deep bit */
#define  SCB_SCR_SEVONPEND                   ((uint8_t)0x10)               /*!< Wake up from WFE */

/**
  * @brief SCB_CCR register Bit definition
  */
#define  SCB_CCR_NONBASETHRDENA              ((uint16_t)0x0001)            /*!< Thread mode can be entered from any level in Handler mode by controlled return value */
#define  SCB_CCR_USERSETMPEND                ((uint16_t)0x0002)            /*!< Enables user code to write the Software Trigger Interrupt register to trigger (pend) a Main exception */
#define  SCB_CCR_UNALIGN_TRP                 ((uint16_t)0x0008)            /*!< Trap for unaligned access */
#define  SCB_CCR_DIV_0_TRP                   ((uint16_t)0x0010)            /*!< Trap on Divide by 0 */
#define  SCB_CCR_BFHFNMIGN                   ((uint16_t)0x0100)            /*!< Handlers running at priority -1 and -2 */
#define  SCB_CCR_STKALIGN                    ((uint16_t)0x0200)            /*!< On exception entry, the SP used prior to the exception is adjusted to be 8-byte aligned */

/**
  * @brief SCB_SHPR register Bit definition
  */
#define  SCB_SHPR_PRI_N                      ((uint32_t)0x000000FF)        /*!< Priority of system handler 4,8, and 12. Mem Manage, reserved and Debug Monitor */
#define  SCB_SHPR_PRI_N1                     ((uint32_t)0x0000FF00)        /*!< Priority of system handler 5,9, and 13. Bus Fault, reserved and reserved */
#define  SCB_SHPR_PRI_N2                     ((uint32_t)0x00FF0000)        /*!< Priority of system handler 6,10, and 14. Usage Fault, reserved and PendSV */
#define  SCB_SHPR_PRI_N3                     ((uint32_t)0xFF000000)        /*!< Priority of system handler 7,11, and 15. Reserved, SVCall and SysTick */

/**
  * @brief SCB_SHCSR register Bit definition
  */
#define  SCB_SHCSR_MEMFAULTACT               ((uint32_t)0x00000001)        /*!< MemManage is active */
#define  SCB_SHCSR_BUSFAULTACT               ((uint32_t)0x00000002)        /*!< BusFault is active */
#define  SCB_SHCSR_USGFAULTACT               ((uint32_t)0x00000008)        /*!< UsageFault is active */
#define  SCB_SHCSR_SVCALLACT                 ((uint32_t)0x00000080)        /*!< SVCall is active */
#define  SCB_SHCSR_MONITORACT                ((uint32_t)0x00000100)        /*!< Monitor is active */
#define  SCB_SHCSR_PENDSVACT                 ((uint32_t)0x00000400)        /*!< PendSV is active */
#define  SCB_SHCSR_SYSTICKACT                ((uint32_t)0x00000800)        /*!< SysTick is active */
#define  SCB_SHCSR_USGFAULTPENDED            ((uint32_t)0x00001000)        /*!< Usage Fault is pended */
#define  SCB_SHCSR_MEMFAULTPENDED            ((uint32_t)0x00002000)        /*!< MemManage is pended */
#define  SCB_SHCSR_BUSFAULTPENDED            ((uint32_t)0x00004000)        /*!< Bus Fault is pended */
#define  SCB_SHCSR_SVCALLPENDED              ((uint32_t)0x00008000)        /*!< SVCall is pended */
#define  SCB_SHCSR_MEMFAULTENA               ((uint32_t)0x00010000)        /*!< MemManage enable */
#define  SCB_SHCSR_BUSFAULTENA               ((uint32_t)0x00020000)        /*!< Bus Fault enable */
#define  SCB_SHCSR_USGFAULTENA               ((uint32_t)0x00040000)        /*!< UsageFault enable */

/**
  * @brief SCB_CFSR register Bit definition
  */
/*!< MFSR */
#define  SCB_CFSR_IACCVIOL                   ((uint32_t)0x00000001)        /*!< Instruction access violation */
#define  SCB_CFSR_DACCVIOL                   ((uint32_t)0x00000002)        /*!< Data access violation */
#define  SCB_CFSR_MUNSTKERR                  ((uint32_t)0x00000008)        /*!< Unstacking error */
#define  SCB_CFSR_MSTKERR                    ((uint32_t)0x00000010)        /*!< Stacking error */
#define  SCB_CFSR_MMARVALID                  ((uint32_t)0x00000080)        /*!< Memory Manage Address Register address valid flag */
/*!< BFSR */
#define  SCB_CFSR_IBUSERR                    ((uint32_t)0x00000100)        /*!< Instruction bus error flag */
#define  SCB_CFSR_PRECISERR                  ((uint32_t)0x00000200)        /*!< Precise data bus error */
#define  SCB_CFSR_IMPRECISERR                ((uint32_t)0x00000400)        /*!< Imprecise data bus error */
#define  SCB_CFSR_UNSTKERR                   ((uint32_t)0x00000800)        /*!< Unstacking error */
#define  SCB_CFSR_STKERR                     ((uint32_t)0x00001000)        /*!< Stacking error */
#define  SCB_CFSR_BFARVALID                  ((uint32_t)0x00008000)        /*!< Bus Fault Address Register address valid flag */
/*!< UFSR */
#define  SCB_CFSR_UNDEFINSTR                 ((uint32_t)0x00010000)        /*!< The processor attempt to execute an undefined instruction */
#define  SCB_CFSR_INVSTATE                   ((uint32_t)0x00020000)        /*!< Invalid combination of EPSR and instruction */
#define  SCB_CFSR_INVPC                      ((uint32_t)0x00040000)        /*!< Attempt to load EXC_RETURN into pc illegally */
#define  SCB_CFSR_NOCP                       ((uint32_t)0x00080000)        /*!< Attempt to use a coprocessor instruction */
#define  SCB_CFSR_UNALIGNED                  ((uint32_t)0x01000000)        /*!< Fault occurs when there is an attempt to make an unaligned memory access */
#define  SCB_CFSR_DIVBYZERO                  ((uint32_t)0x02000000)        /*!< Fault occurs when SDIV or DIV instruction is used with a divisor of 0 */

/**
  * @brief SCB_HFSR register Bit definition
  */
#define  SCB_HFSR_VECTTBL                    ((uint32_t)0x00000002)        /*!< Fault occurs because of vector table read on exception processing */
#define  SCB_HFSR_FORCED                     ((uint32_t)0x40000000)        /*!< Hard Fault activated when a configurable Fault was received and cannot activate */
#define  SCB_HFSR_DEBUGEVT                   ((uint32_t)0x80000000)        /*!< Fault related to debug */

/**
  * @brief SCB_DFSR register Bit definition
  */
#define  SCB_DFSR_HALTED                     ((uint8_t)0x01)               /*!< Halt request flag */
#define  SCB_DFSR_BKPT                       ((uint8_t)0x02)               /*!< BKPT flag */
#define  SCB_DFSR_DWTTRAP                    ((uint8_t)0x04)               /*!< Data Watchpoint and Trace (DWT) flag */
#define  SCB_DFSR_VCATCH                     ((uint8_t)0x08)               /*!< Vector catch flag */
#define  SCB_DFSR_EXTERNAL                   ((uint8_t)0x10)               /*!< External debug request flag */

/**
  * @brief SCB_MMFAR register Bit definition
  */
#define  SCB_MMFAR_ADDRESS                   ((uint32_t)0xFFFFFFFF)        /*!< Mem Manage fault address field */

/**
  * @brief SCB_BFAR register Bit definition
  */
#define  SCB_BFAR_ADDRESS                    ((uint32_t)0xFFFFFFFF)        /*!< Bus fault address field */

/**
  * @brief SCB_afsr register Bit definition
  */
#define  SCB_AFSR_IMPDEF                     ((uint32_t)0xFFFFFFFF)        /*!< Implementation defined */

/******************************************************************************/
/*                                                                            */
/*                    External Interrupt/Event Controller                     */
/*                                                                            */
/******************************************************************************/

/**
  * @brief EXTI_INTEN register Bit definition
  */
#define  EXTI_INTEN_INTEN0                    ((uint32_t)0x00000001)        /*!< Interrupt enable for line0 */
#define  EXTI_INTEN_INTEN1                    ((uint32_t)0x00000002)        /*!< Interrupt enable for line1 */
#define  EXTI_INTEN_INTEN2                    ((uint32_t)0x00000004)        /*!< Interrupt enable for line2 */
#define  EXTI_INTEN_INTEN3                    ((uint32_t)0x00000008)        /*!< Interrupt enable for line3 */
#define  EXTI_INTEN_INTEN4                    ((uint32_t)0x00000010)        /*!< Interrupt enable for line4 */
#define  EXTI_INTEN_INTEN5                    ((uint32_t)0x00000020)        /*!< Interrupt enable for line5 */
#define  EXTI_INTEN_INTEN6                    ((uint32_t)0x00000040)        /*!< Interrupt enable for line6 */
#define  EXTI_INTEN_INTEN7                    ((uint32_t)0x00000080)        /*!< Interrupt enable for line7 */
#define  EXTI_INTEN_INTEN8                    ((uint32_t)0x00000100)        /*!< Interrupt enable for line8 */
#define  EXTI_INTEN_INTEN9                    ((uint32_t)0x00000200)        /*!< Interrupt enable for line9 */
#define  EXTI_INTEN_INTEN10                   ((uint32_t)0x00000400)        /*!< Interrupt enable for line10 */
#define  EXTI_INTEN_INTEN11                   ((uint32_t)0x00000800)        /*!< Interrupt enable for line11 */
#define  EXTI_INTEN_INTEN12                   ((uint32_t)0x00001000)        /*!< Interrupt enable for line12 */
#define  EXTI_INTEN_INTEN13                   ((uint32_t)0x00002000)        /*!< Interrupt enable for line13 */
#define  EXTI_INTEN_INTEN14                   ((uint32_t)0x00004000)        /*!< Interrupt enable for line14 */
#define  EXTI_INTEN_INTEN15                   ((uint32_t)0x00008000)        /*!< Interrupt enable for line15 */
#define  EXTI_INTEN_INTEN16                   ((uint32_t)0x00010000)        /*!< Interrupt enable for line16 */
#define  EXTI_INTEN_INTEN17                   ((uint32_t)0x00020000)        /*!< Interrupt enable for line17 */
#define  EXTI_INTEN_INTEN18                   ((uint32_t)0x00040000)        /*!< Interrupt enable for line18 */

/**
  * @brief EXTI_EVTEN register Bit definition
  */
#define  EXTI_EVTEN_EVTEN0                    ((uint32_t)0x00000001)        /*!< Event enable for line0 */
#define  EXTI_EVTEN_EVTEN1                    ((uint32_t)0x00000002)        /*!< Event enable for line1 */
#define  EXTI_EVTEN_EVTEN2                    ((uint32_t)0x00000004)        /*!< Event enable for line2 */
#define  EXTI_EVTEN_EVTEN3                    ((uint32_t)0x00000008)        /*!< Event enable for line3 */
#define  EXTI_EVTEN_EVTEN4                    ((uint32_t)0x00000010)        /*!< Event enable for line4 */
#define  EXTI_EVTEN_EVTEN5                    ((uint32_t)0x00000020)        /*!< Event enable for line5 */
#define  EXTI_EVTEN_EVTEN6                    ((uint32_t)0x00000040)        /*!< Event enable for line6 */
#define  EXTI_EVTEN_EVTEN7                    ((uint32_t)0x00000080)        /*!< Event enable for line7 */
#define  EXTI_EVTEN_EVTEN8                    ((uint32_t)0x00000100)        /*!< Event enable for line8 */
#define  EXTI_EVTEN_EVTEN9                    ((uint32_t)0x00000200)        /*!< Event enable for line9 */
#define  EXTI_EVTEN_EVTEN10                   ((uint32_t)0x00000400)        /*!< Event enable for line10 */
#define  EXTI_EVTEN_EVTEN11                   ((uint32_t)0x00000800)        /*!< Event enable for line11 */
#define  EXTI_EVTEN_EVTEN12                   ((uint32_t)0x00001000)        /*!< Event enable for line12 */
#define  EXTI_EVTEN_EVTEN13                   ((uint32_t)0x00002000)        /*!< Event enable for line13 */
#define  EXTI_EVTEN_EVTEN14                   ((uint32_t)0x00004000)        /*!< Event enable for line14 */
#define  EXTI_EVTEN_EVTEN15                   ((uint32_t)0x00008000)        /*!< Event enable for line15 */
#define  EXTI_EVTEN_EVTEN16                   ((uint32_t)0x00010000)        /*!< Event enable for line16 */
#define  EXTI_EVTEN_EVTEN17                   ((uint32_t)0x00020000)        /*!< Event enable for line17 */
#define  EXTI_EVTEN_EVTEN18                   ((uint32_t)0x00040000)        /*!< Event enable for line18 */

/**
  * @brief EXTI_RTEN register Bit definition
  */
#define  EXTI_RTEN_RTEN0                      ((uint32_t)0x00000001)        /*!< Rising edge trigger enable for line0 */
#define  EXTI_RTEN_RTEN1                      ((uint32_t)0x00000002)        /*!< Rising edge trigger enable for line1 */
#define  EXTI_RTEN_RTEN2                      ((uint32_t)0x00000004)        /*!< Rising edge trigger enable for line2 */
#define  EXTI_RTEN_RTEN3                      ((uint32_t)0x00000008)        /*!< Rising edge trigger enable for line3 */
#define  EXTI_RTEN_RTEN4                      ((uint32_t)0x00000010)        /*!< Rising edge trigger enable for line4 */
#define  EXTI_RTEN_RTEN5                      ((uint32_t)0x00000020)        /*!< Rising edge trigger enable for line5 */
#define  EXTI_RTEN_RTEN6                      ((uint32_t)0x00000040)        /*!< Rising edge trigger enable for line6 */
#define  EXTI_RTEN_RTEN7                      ((uint32_t)0x00000080)        /*!< Rising edge trigger enable for line7 */
#define  EXTI_RTEN_RTEN8                      ((uint32_t)0x00000100)        /*!< Rising edge trigger enable for line8 */
#define  EXTI_RTEN_RTEN9                      ((uint32_t)0x00000200)        /*!< Rising edge trigger enable for line9 */
#define  EXTI_RTEN_RTEN10                     ((uint32_t)0x00000400)        /*!< Rising edge trigger enable for line10 */
#define  EXTI_RTEN_RTEN11                     ((uint32_t)0x00000800)        /*!< Rising edge trigger enable for line11 */
#define  EXTI_RTEN_RTEN12                     ((uint32_t)0x00001000)        /*!< Rising edge trigger enable for line12 */
#define  EXTI_RTEN_RTEN13                     ((uint32_t)0x00002000)        /*!< Rising edge trigger enable for line13 */
#define  EXTI_RTEN_RTEN14                     ((uint32_t)0x00004000)        /*!< Rising edge trigger enable for line14 */
#define  EXTI_RTEN_RTEN15                     ((uint32_t)0x00008000)        /*!< Rising edge trigger enable for line15 */
#define  EXTI_RTEN_RTEN16                     ((uint32_t)0x00010000)        /*!< Rising edge trigger enable for line16 */
#define  EXTI_RTEN_RTEN17                     ((uint32_t)0x00020000)        /*!< Rising edge trigger enable for line17 */
#define  EXTI_RTEN_RTEN18                     ((uint32_t)0x00040000)        /*!< Rising edge trigger enable for line18 */

/**
  * @brief EXTI_FTEN register Bit definition
  */
#define  EXTI_FTEN_FTEN0                      ((uint32_t)0x00000001)        /*!< Falling edge trigger enable for line0 */
#define  EXTI_FTEN_FTEN1                      ((uint32_t)0x00000002)        /*!< Falling edge trigger enable for line1 */
#define  EXTI_FTEN_FTEN2                      ((uint32_t)0x00000004)        /*!< Falling edge trigger enable for line2 */
#define  EXTI_FTEN_FTEN3                      ((uint32_t)0x00000008)        /*!< Falling edge trigger enable for line3 */
#define  EXTI_FTEN_FTEN4                      ((uint32_t)0x00000010)        /*!< Falling edge trigger enable for line4 */
#define  EXTI_FTEN_FTEN5                      ((uint32_t)0x00000020)        /*!< Falling edge trigger enable for line5 */
#define  EXTI_FTEN_FTEN6                      ((uint32_t)0x00000040)        /*!< Falling edge trigger enable for line6 */
#define  EXTI_FTEN_FTEN7                      ((uint32_t)0x00000080)        /*!< Falling edge trigger enable for line7 */
#define  EXTI_FTEN_FTEN8                      ((uint32_t)0x00000100)        /*!< Falling edge trigger enable for line8 */
#define  EXTI_FTEN_FTEN9                      ((uint32_t)0x00000200)        /*!< Falling edge trigger enable for line9 */
#define  EXTI_FTEN_FTEN10                     ((uint32_t)0x00000400)        /*!< Falling edge trigger enable for line10 */
#define  EXTI_FTEN_FTEN11                     ((uint32_t)0x00000800)        /*!< Falling edge trigger enable for line11 */
#define  EXTI_FTEN_FTEN12                     ((uint32_t)0x00001000)        /*!< Falling edge trigger enable for line12 */
#define  EXTI_FTEN_FTEN13                     ((uint32_t)0x00002000)        /*!< Falling edge trigger enable for line13 */
#define  EXTI_FTEN_FTEN14                     ((uint32_t)0x00004000)        /*!< Falling edge trigger enable for line14 */
#define  EXTI_FTEN_FTEN15                     ((uint32_t)0x00008000)        /*!< Falling edge trigger enable for line15 */
#define  EXTI_FTEN_FTEN16                     ((uint32_t)0x00010000)        /*!< Falling edge trigger enable for line16 */
#define  EXTI_FTEN_FTEN17                     ((uint32_t)0x00020000)        /*!< Falling edge trigger enable for line17 */
#define  EXTI_FTEN_FTEN18                     ((uint32_t)0x00040000)        /*!< Falling edge trigger enable for line18 */

/**
  * @brief EXTI_SWTIEN register Bit definition
  */
#define  EXTI_SWTIEN_SWTIEN0                  ((uint32_t)0x00000001)        /*!< Software trigger interrupt/event for line0 */
#define  EXTI_SWTIEN_SWTIEN1                  ((uint32_t)0x00000002)        /*!< Software trigger interrupt/event for line1 */
#define  EXTI_SWTIEN_SWTIEN2                  ((uint32_t)0x00000004)        /*!< Software trigger interrupt/event for line2 */
#define  EXTI_SWTIEN_SWTIEN3                  ((uint32_t)0x00000008)        /*!< Software trigger interrupt/event for line3 */
#define  EXTI_SWTIEN_SWTIEN4                  ((uint32_t)0x00000010)        /*!< Software trigger interrupt/event for line4 */
#define  EXTI_SWTIEN_SWTIEN5                  ((uint32_t)0x00000020)        /*!< Software trigger interrupt/event for line5 */
#define  EXTI_SWTIEN_SWTIEN6                  ((uint32_t)0x00000040)        /*!< Software trigger interrupt/event for line6 */
#define  EXTI_SWTIEN_SWTIEN7                  ((uint32_t)0x00000080)        /*!< Software trigger interrupt/event for line7 */
#define  EXTI_SWTIEN_SWTIEN8                  ((uint32_t)0x00000100)        /*!< Software trigger interrupt/event for line8 */
#define  EXTI_SWTIEN_SWTIEN9                  ((uint32_t)0x00000200)        /*!< Software trigger interrupt/event for line9 */
#define  EXTI_SWTIEN_SWTIEN10                 ((uint32_t)0x00000400)        /*!< Software trigger interrupt/event for line10 */
#define  EXTI_SWTIEN_SWTIEN11                 ((uint32_t)0x00000800)        /*!< Software trigger interrupt/event for line11 */
#define  EXTI_SWTIEN_SWTIEN12                 ((uint32_t)0x00001000)        /*!< Software trigger interrupt/event for line12 */
#define  EXTI_SWTIEN_SWTIEN13                 ((uint32_t)0x00002000)        /*!< Software trigger interrupt/event for line13 */
#define  EXTI_SWTIEN_SWTIEN14                 ((uint32_t)0x00004000)        /*!< Software trigger interrupt/event for line14 */
#define  EXTI_SWTIEN_SWTIEN15                 ((uint32_t)0x00008000)        /*!< Software trigger interrupt/event for line15 */
#define  EXTI_SWTIEN_SWTIEN16                 ((uint32_t)0x00010000)        /*!< Software trigger interrupt/event for line16 */
#define  EXTI_SWTIEN_SWTIEN17                 ((uint32_t)0x00020000)        /*!< Software trigger interrupt/event for line17 */
#define  EXTI_SWTIEN_SWTIEN18                 ((uint32_t)0x00040000)        /*!< Software trigger interrupt/event for line18 */

/**
  * @brief EXTI_PDF register Bit definition
  */
#define  EXTI_PDF_PDF0                        ((uint32_t)0x00000001)        /*!< Pending interrupt flag for line 0 */
#define  EXTI_PDF_PDF1                        ((uint32_t)0x00000002)        /*!< Pending interrupt flag for line 1 */
#define  EXTI_PDF_PDF2                        ((uint32_t)0x00000004)        /*!< Pending interrupt flag for line 2 */
#define  EXTI_PDF_PDF3                        ((uint32_t)0x00000008)        /*!< Pending interrupt flag for line 3 */
#define  EXTI_PDF_PDF4                        ((uint32_t)0x00000010)        /*!< Pending interrupt flag for line 4 */
#define  EXTI_PDF_PDF5                        ((uint32_t)0x00000020)        /*!< Pending interrupt flag for line 5 */
#define  EXTI_PDF_PDF6                        ((uint32_t)0x00000040)        /*!< Pending interrupt flag for line 6 */
#define  EXTI_PDF_PDF7                        ((uint32_t)0x00000080)        /*!< Pending interrupt flag for line 7 */
#define  EXTI_PDF_PDF8                        ((uint32_t)0x00000100)        /*!< Pending interrupt flag for line 8 */
#define  EXTI_PDF_PDF9                        ((uint32_t)0x00000200)        /*!< Pending interrupt flag for line 9 */
#define  EXTI_PDF_PDF10                       ((uint32_t)0x00000400)        /*!< Pending interrupt flag for line 10 */
#define  EXTI_PDF_PDF11                       ((uint32_t)0x00000800)        /*!< Pending interrupt flag for line 11 */
#define  EXTI_PDF_PDF12                       ((uint32_t)0x00001000)        /*!< Pending interrupt flag for line 12 */
#define  EXTI_PDF_PDF13                       ((uint32_t)0x00002000)        /*!< Pending interrupt flag for line 13 */
#define  EXTI_PDF_PDF14                       ((uint32_t)0x00004000)        /*!< Pending interrupt flag for line 14 */
#define  EXTI_PDF_PDF15                       ((uint32_t)0x00008000)        /*!< Pending interrupt flag for line 15 */
#define  EXTI_PDF_PDF16                       ((uint32_t)0x00010000)        /*!< Pending interrupt flag for line 16 */
#define  EXTI_PDF_PDF17                       ((uint32_t)0x00020000)        /*!< Pending interrupt flag for line 17 */
#define  EXTI_PDF_PDF18                       ((uint32_t)0x00040000)        /*!< Pending interrupt flag for line 18 */

/******************************************************************************/
/*                                                                            */
/*                             DMA Controller                                 */
/*                                                                            */
/******************************************************************************/

/**
  * @brief DMA_STS register Bit definition
  */
#define  DMA_STS_GIF1                        ((uint32_t)0x00000001)        /*!< lobal interrupt flag of Channel 1 */
#define  DMA_STS_CMPIF1                      ((uint32_t)0x00000002)        /*!< ransfer complete flag of Channel 1  */
#define  DMA_STS_HLFIF1                      ((uint32_t)0x00000004)        /*!< alf transfer flag of Channel 1  */
#define  DMA_STS_ERRIF1                      ((uint32_t)0x00000008)        /*!< ransfer error flag of Channel 1 */
                                                                          
#define  DMA_STS_GIF2                        ((uint32_t)0x00000010)        /*!< lobal interrupt flag of Channel 2 */
#define  DMA_STS_CMPIF2                      ((uint32_t)0x00000020)        /*!< ransfer complete flag of Channel 2 */
#define  DMA_STS_HLFIF2                      ((uint32_t)0x00000040)        /*!< alf transfer flag of Channel 2 */
#define  DMA_STS_ERRIF2                      ((uint32_t)0x00000080)        /*!< ransfer error flag of Channel 2 */
                                                                          
#define  DMA_STS_GIF3                        ((uint32_t)0x00000100)        /*!< lobal interrupt flag of Channel 3 */
#define  DMA_STS_CMPIF3                      ((uint32_t)0x00000200)        /*!< ransfer complete flag of Channel 3 */
#define  DMA_STS_HLFIF3                      ((uint32_t)0x00000400)        /*!< alf transfer flag of Channel 3 */
#define  DMA_STS_ERRIF3                      ((uint32_t)0x00000800)        /*!< ransfer error flag of Channel 3 */
                                                                          
#define  DMA_STS_GIF4                        ((uint32_t)0x00001000)        /*!< lobal interrupt flag of Channel 4 */
#define  DMA_STS_CMPIF4                      ((uint32_t)0x00002000)        /*!< ransfer complete flag of Channel 4 */
#define  DMA_STS_HLFIF4                      ((uint32_t)0x00004000)        /*!< alf transfer flag of Channel 4  */
#define  DMA_STS_ERRIF4                      ((uint32_t)0x00008000)        /*!< ransfer error flag of Channel 4 */
                                                                          
#define  DMA_STS_GIF5                        ((uint32_t)0x00010000)        /*!< lobal interrupt flag of Channel 5 */
#define  DMA_STS_CMPIF5                      ((uint32_t)0x00020000)        /*!< ransfer complete flag of Channel 5  */
#define  DMA_STS_HLFIF5                      ((uint32_t)0x00040000)        /*!< alf transfer flag of Channel 5  */
#define  DMA_STS_ERRIF5                      ((uint32_t)0x00080000)        /*!< ransfer error flag of Channel 5  */
                                                                          
                                                                          
#define  DMA_STS_GIF6                        ((uint32_t)0x00100000)        /*!< lobal interrupt flag of Channel 6 */
#define  DMA_STS_CMPIF6                      ((uint32_t)0x00200000)        /*!< ransfer complete flag of Channel 6 */
#define  DMA_STS_HLFIF6                      ((uint32_t)0x00400000)        /*!< alf transfer flag of Channel 6 */
#define  DMA_STS_ERRIF6                      ((uint32_t)0x00800000)        /*!< ransfer error flag of Channel 6 */
                                                                          
#define  DMA_STS_GIF7                        ((uint32_t)0x01000000)        /*!< lobal interrupt flag of Channel 7*/
#define  DMA_STS_CMPIF7                      ((uint32_t)0x02000000)        /*!< ransfer complete flag of Channel 7 */
#define  DMA_STS_HLFIF7                      ((uint32_t)0x04000000)        /*!< alf transfer flag of Channel 7 */
#define  DMA_STS_ERRIF7                      ((uint32_t)0x08000000)        /*!< ransfer error flag of Channel 7 */
                                                                          
/**
  * @brief DMA_INTFC register  Bit definition
  */
#define  DMA_INTFC_GIFC1                     ((uint32_t)0x00000001)        /*!<Global interrupt flag clear of Channel 1 */
#define  DMA_INTFC_CMPIFC1                   ((uint32_t)0x00000002)        /*!<Transfer complete flag clear of Channe 1 */
#define  DMA_INTFC_HLFIFC1                   ((uint32_t)0x00000004)        /*!<Half transfer flag clear of Channel 1 */
#define  DMA_INTFC_ERRIFC1                   ((uint32_t)0x00000008)        /*!<Transfer error flag clear of Channel 1 */

#define  DMA_INTFC_GIFC2                     ((uint32_t)0x00000010)        /*!<Global interrupt flag clear of Channel 2 */
#define  DMA_INTFC_CMPIFC2                   ((uint32_t)0x00000020)        /*!<Transfer complete flag clear of Channe 2 */
#define  DMA_INTFC_HLFIFC2                   ((uint32_t)0x00000040)        /*!<Half transfer flag clear of Channel 2 */
#define  DMA_INTFC_ERRIFC2                   ((uint32_t)0x00000080)        /*!<Transfer error flag clear of Channel 2 */

#define  DMA_INTFC_GIFC3                     ((uint32_t)0x00000100)        /*!<Global interrupt flag clear of Channel 3 */
#define  DMA_INTFC_CMPIFC3                   ((uint32_t)0x00000200)        /*!<Transfer complete flag clear of Channe 3 */
#define  DMA_INTFC_HLFIFC3                   ((uint32_t)0x00000400)        /*!<Half transfer flag clear of Channel 3 */
#define  DMA_INTFC_ERRIFC3                   ((uint32_t)0x00000800)        /*!<Transfer error flag clear of Channel 3 */

#define  DMA_INTFC_GIFC4                     ((uint32_t)0x00001000)        /*!<Global interrupt flag clear of Channel 4 */
#define  DMA_INTFC_CMPIFC4                   ((uint32_t)0x00002000)        /*!<Transfer complete flag clear of Channe 4 */
#define  DMA_INTFC_HLFIFC4                   ((uint32_t)0x00004000)        /*!<Half transfer flag clear of Channel 4 */
#define  DMA_INTFC_ERRIFC4                   ((uint32_t)0x00008000)        /*!<Transfer error flag clear of Channel 4 */

#define  DMA_INTFC_GIFC5                     ((uint32_t)0x00010000)        /*!<Global interrupt flag clear of Channel 5 */
#define  DMA_INTFC_CMPIFC5                   ((uint32_t)0x00020000)        /*!<Transfer complete flag clear of Channe 5 */
#define  DMA_INTFC_HLFIFC5                   ((uint32_t)0x00040000)        /*!<Half transfer flag clear of Channel 5 */
#define  DMA_INTFC_ERRIFC5                   ((uint32_t)0x00080000)        /*!<Transfer error flag clear of Channel 5 */

#define  DMA_INTFC_GIFC6                     ((uint32_t)0x00100000)        /*!<Global interrupt flag clear of Channel 6 */
#define  DMA_INTFC_CMPIFC6                   ((uint32_t)0x00200000)        /*!<Transfer complete flag clear of Channe 6 */
#define  DMA_INTFC_HLFIFC6                   ((uint32_t)0x00400000)        /*!<Half transfer flag clear of Channel 6  */
#define  DMA_INTFC_ERRIFC6                   ((uint32_t)0x00800000)        /*!<Transfer error flag clear of Channel 6 */

#define  DMA_INTFC_GIFC7                     ((uint32_t)0x01000000)        /*!<Global interrupt flag clear of Channel 7 */
#define  DMA_INTFC_CMPIFC7                   ((uint32_t)0x02000000)        /*!<Transfer complete flag clear of Channe 7 */
#define  DMA_INTFC_HLFIFC7                   ((uint32_t)0x04000000)        /*!<Half transfer flag clear of Channel 7  */
#define  DMA_INTFC_ERRIFC7                   ((uint32_t)0x08000000)        /*!<Transfer error flag clear of Channel 7 */

/**
  * @brief DMA_CHxCTR register Bit definition
  */
#define  DMA_CHxCTR_CEN                      ((uint32_t)0x00000001)        /*!<Channel enable */
#define  DMA_CHxCTR_CMPIE                    ((uint32_t)0x00000002)        /*!<Enable bit for Transfer complete interrupt */
#define  DMA_CHxCTR_HLFIE                    ((uint32_t)0x00000004)        /*!<Enable bit for Half transfer interrupt */
#define  DMA_CHxCTR_ERRIE                    ((uint32_t)0x00000008)        /*!<Enable bit for Transfer error interrupt */
#define  DMA_CHxCTR_DIR                      ((uint32_t)0x00000010)        /*!<Direction of transfer */
#define  DMA_CHxCTR_CIRM                     ((uint32_t)0x00000020)        /*!<Circular mode */
#define  DMA_CHxCTR_PAGM                     ((uint32_t)0x00000040)        /*!<Peripheral address generation mode */
#define  DMA_CHxCTR_MAGM                     ((uint32_t)0x00000080)        /*!<Memory address generation mode */

#define  DMA_CHxCTR_PWDH                     ((uint32_t)0x00000300)        /*!<Peripheral width PWDH[1:0] bits */
#define  DMA_CHxCTR_PWDH_0                   ((uint32_t)0x00000100)        /*!<Bit 0 */
#define  DMA_CHxCTR_PWDH_1                   ((uint32_t)0x00000200)        /*!<Bit 1 */
                                                                            
#define  DMA_CHxCTR_MWDH                     ((uint32_t)0x00000C00)        /*!<Memory width MWDH[1:0] bits */
#define  DMA_CHxCTR_MWDH_0                   ((uint32_t)0x00000400)        /*!<Bit 0 */
#define  DMA_CHxCTR_MWDH_1                   ((uint32_t)0x00000800)        /*!<Bit 1 */

#define  DMA_CHxCTR_PRIL                     ((uint32_t)0x00003000)        /*!<Priority level of Channel PRIL[1:0]bits */
#define  DMA_CHxCTR_PRIL_0                   ((uint32_t)0x00001000)        /*!<Bit 0 */
#define  DMA_CHxCTR_PRIL_1                   ((uint32_t)0x00002000)        /*!<Bit 1 */

#define  DMA_CHxCTR_M2MM                     ((uint32_t)0x00004000)        /*!<Memory to memory mode */

/**
  * @brief DMA_CHxNUM register Bit definition
  */
#define  DMA_CHxNUM_NUM                      ((uint32_t)0x0000FFFF)        /*!<Number of data to Transfer */

/**
  * @brief DMA_CHxPA register Bit definition
  */
#define  DMA_CHxPA_PADR                      ((uint32_t)0xFFFFFFFF)        /*!<Peripheral Address */

/**
  * @brief DMA_CHxMA register Bit definition
  */
#define  DMA_CHxMA_MADR                      ((uint32_t)0xFFFFFFFF)        /*!<Memory Address */

/******************************************************************************/
/*                                                                            */
/*                        Analog to Digital Converter                         */
/*                                                                            */
/******************************************************************************/

/**
  * @brief ADC_STS register Bit definition
  */
#define  ADC_STS_STRTF              ((uint32_t)0x00000010)      /*!< Regular channel Start flag. */
#define  ADC_STS_JSTRTF             ((uint32_t)0x00000008)      /*!< Injected channel Start flag. */
#define  ADC_STS_EOJCH              ((uint32_t)0x00000004)      /*!< End of injected channel conversion. */
#define  ADC_STS_EOCH               ((uint32_t)0x00000002)      /*!< End of channel conversion flag. */
#define  ADC_STS_WDEVT              ((uint32_t)0x00000001)      /*!< Analog watchdog event flag. */

/**
  * @brief ADC_CTR1 register Bit definition
  */
#define  ADC_CTR1_WDGEN             ((uint32_t)0x00800000)      /*!< Analog watchdog enable. */
#define  ADC_CTR1_AWDJCHEN          ((uint32_t)0x00400000)      /*!< Analog watchdog enable on injected channels. */
#define  ADC_CTR1_DMS               ((uint32_t)0x000F0000)      /*!< Dual mode selection. */
#define  ADC_CTR1_DISCCHCNT         ((uint32_t)0x0000E000)      /*!< Discontinuous mode channel count. */
#define  ADC_CTR1_JDISCEN           ((uint32_t)0x00001000)      /*!< Discontinuous mode enable on injected channels. */
#define  ADC_CTR1_DISCEN            ((uint32_t)0x00000800)      /*!< Discontinuous mode on regular channels. */
#define  ADC_CTR1_JAUTOC            ((uint32_t)0x00000400)      /*!< Automatic Injected Group conversion. */
#define  ADC_CTR1_AWDSGLEN          ((uint32_t)0x00000200)      /*!< Enable the watchdog on a single channel in scan mode. */
#define  ADC_CTR1_SCANMD            ((uint32_t)0x00000100)      /*!< Scan mode. */
#define  ADC_CTR1_JEOCIE            ((uint32_t)0x00000080)      /*!< Interrupt enable for injected channels. */
#define  ADC_CTR1_WDEVTIE           ((uint32_t)0x00000040)      /*!< Analog watchdog event interrupt enable. */
#define  ADC_CTR1_EOCIE             ((uint32_t)0x00000020)      /*!< Interrupt enable for EOC. */
#define  ADC_CTR1_WDGCHAN           ((uint32_t)0x0000001F)      /*!< Analog watchdog channel select bits. */

/**
  * @brief ADC_CTR register Bit definition
  */
#define  ADC_CTR_TSVRFEN            ((uint32_t)0x00800000)      /*!< Temperature sensor and VREFINT enable. */
#define  ADC_CTR_START              ((uint32_t)0x00400000)      /*!< Start conversion of regular channels. */
#define  ADC_CTR_JSTART             ((uint32_t)0x00200000)      /*!< Start conversion of injected channels. */
#define  ADC_CTR_EXTTRG             ((uint32_t)0x00100000)      /*!< External trigger conversion mode for regular channels. */
#define  ADC_CTR_EVTS               ((uint32_t)0x000E0000)      /*!< External event select for regular group. */
#define  ADC_CTR_JEXTTRG            ((uint32_t)0x00008000)      /*!< External trigger conversion mode for injected channels. */
#define  ADC_CTR_JEVTS              ((uint32_t)0x00007000)      /*!< External event select for injected group. */
#define  ADC_CTR_DALGN              ((uint32_t)0x00000800)      /*!< Data alignment. */
#define  ADC_CTR_DMAEN              ((uint32_t)0x00000100)      /*!< Direct memory access mode enable. */
#define  ADC_CTR_RSTCALB            ((uint32_t)0x00000008)      /*!< Reset calibration. */
#define  ADC_CTR_CALB               ((uint32_t)0x00000004)      /*!< A/D Calibration. */
#define  ADC_CTR_GCONT              ((uint32_t)0x00000002)      /*!< Continuous conversion. */
#define  ADC_CTR_ADCON              ((uint32_t)0x00000001)      /*!< ADC power on command. */

/**
  * @brief ADC_SMPR1 register Bit definition
  */
#define  ADC_SMPR1_SMPLT17          ((uint32_t)0x00E00000)      /*!< Channel 17 Sample time selection. */
#define  ADC_SMPR1_SMPLT16          ((uint32_t)0x001C0000)      /*!< Channel 16 Sample time selection. */
#define  ADC_SMPR1_SMPLT15          ((uint32_t)0x00038000)      /*!< Channel 15 Sample time selection. */
#define  ADC_SMPR1_SMPLT14          ((uint32_t)0x00007000)      /*!< Channel 14 Sample time selection. */
#define  ADC_SMPR1_SMPLT13          ((uint32_t)0x00000E00)      /*!< Channel 13 Sample time selection. */
#define  ADC_SMPR1_SMPLT12          ((uint32_t)0x000001C0)      /*!< Channel 12 Sample time selection. */
#define  ADC_SMPR1_SMPLT11          ((uint32_t)0x00000038)      /*!< Channel 11 Sample time selection. */
#define  ADC_SMPR1_SMPLT10          ((uint32_t)0x00000007)      /*!< Channel 10 Sample time selection. */

/**
  * @brief ADC_SMPR2 register Bit definition
  */
#define  ADC_SMPR2_SMPLT9           ((uint32_t)0x38000000)      /*!< Channel 9 Sample time selection. */
#define  ADC_SMPR2_SMPLT8           ((uint32_t)0x07000000)      /*!< Channel 8 Sample time selection. */
#define  ADC_SMPR2_SMPLT7           ((uint32_t)0x00E00000)      /*!< Channel 7 Sample time selection. */
#define  ADC_SMPR2_SMPLT6           ((uint32_t)0x001C0000)      /*!< Channel 6 Sample time selection. */
#define  ADC_SMPR2_SMPLT5           ((uint32_t)0x00038000)      /*!< Channel 5 Sample time selection. */
#define  ADC_SMPR2_SMPLT4           ((uint32_t)0x00007000)      /*!< Channel 4 Sample time selection. */
#define  ADC_SMPR2_SMPLT3           ((uint32_t)0x00000E00)      /*!< Channel 3 Sample time selection. */
#define  ADC_SMPR2_SMPLT2           ((uint32_t)0x000001C0)      /*!< Channel 2 Sample time selection. */
#define  ADC_SMPR2_SMPLT1           ((uint32_t)0x00000038)      /*!< Channel 1 Sample time selection. */
#define  ADC_SMPR2_SMPLT0           ((uint32_t)0x00000007)      /*!< Channel 0 Sample time selection. */

/**
  * @brief ADC_SQR1 register Bit definition
  */
#define  ADC_SQR1_SEQLEN            ((uint32_t)0x00F00000)      /*!< Regular channel sequence length. */
#define  ADC_SQR1_CONVSQ16          ((uint32_t)0x000F8000)      /*!< 16th conversion in regular sequence. */
#define  ADC_SQR1_CONVSQ15          ((uint32_t)0x00007C00)      /*!< 15th conversion in regular sequence. */
#define  ADC_SQR1_CONVSQ14          ((uint32_t)0x000003E0)      /*!< 14th conversion in regular sequence. */
#define  ADC_SQR1_CONVSQ13          ((uint32_t)0x0000001F)      /*!< 13th conversion in regular sequence. */

/**
  * @brief ADC_SQR2 register Bit definition
  */
#define  ADC_SQR2_CONVSQ12          ((uint32_t)0x3E000000)      /*!< 12th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ11          ((uint32_t)0x01F00000)      /*!< 11th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ10          ((uint32_t)0x000F8000)      /*!< 10th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ9           ((uint32_t)0x00007C00)      /*!< 9th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ8           ((uint32_t)0x000003E0)      /*!< 8th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ7           ((uint32_t)0x0000001F)      /*!< 7th conversion in regular sequence. */

/**
  * @brief ADC_SQR3 register Bit definition
  */
#define  ADC_SQR2_CONVSQ6           ((uint32_t)0x3E000000)      /*!< 6th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ5           ((uint32_t)0x01F00000)      /*!< 5th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ4           ((uint32_t)0x000F8000)      /*!< 4th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ3           ((uint32_t)0x00007C00)      /*!< 3th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ2           ((uint32_t)0x000003E0)      /*!< 2th conversion in regular sequence. */
#define  ADC_SQR2_CONVSQ1           ((uint32_t)0x0000001F)      /*!< 1th conversion in regular sequence. */

/**
  * @brief ADC_JSQR register Bit definition
  */
#define  ADC_JSQR_JSEQLEN           ((uint32_t)0x00300000)      /*!< Injected channel sequence length. */
#define  ADC_JSQR_JCONVSQ4          ((uint32_t)0x000F8000)      /*!< 4th conversion in injected sequence. */
#define  ADC_JSQR_JCONVSQ3          ((uint32_t)0x00007C00)      /*!< 3th conversion in injected sequence. */
#define  ADC_JSQR_JCONVSQ2          ((uint32_t)0x000003E0)      /*!< 2th conversion in injected sequence. */
#define  ADC_JSQR_JCONVSQ1          ((uint32_t)0x0000001F)      /*!< 1th conversion in injected sequence. */

/**
  * @brief ADC_OUTDAT register Bit definition
  */
#define  ADC_OUTDAT_ADCC2OUTDAT     ((uint32_t)0xFFFF0000)      /*!< ADC2 conversion output data. */
#define  ADC_OUTDAT_OUTDAT          ((uint32_t)0x0000FFFF)      /*!< Conversion output data. */


/******************************************************************************/
/*                                                                            */
/*                                    TIM                                     */
/*                                                                            */
/******************************************************************************/

/**
  * @brief TIM_CTR1 register Bit definition
  */
#define  TIM_CTR1_CKDIV             ((uint16_t)0x0300)      /*!< Clock division. */
#define  TIM_CTR1_UVALSEN           ((uint16_t)0x0080)      /*!< UVAL shadow enable. */
#define  TIM_CTR1_CPS               ((uint16_t)0x0060)      /*!< Count pattern selection. */
#define  TIM_CTR1_DIR               ((uint16_t)0x0010)      /*!< Direction. */
#define  TIM_CTR1_SPEN              ((uint16_t)0x0008)      /*!< Single pulse enable. */
#define  TIM_CTR1_URSEL             ((uint16_t)0x0004)      /*!< Update request selection. */
#define  TIM_CTR1_UPD               ((uint16_t)0x0002)      /*!< Update disable. */
#define  TIM_CTR1_CEN               ((uint16_t)0x0001)      /*!< Counter enable. */

/**
  * @brief TIM_CTR2 register Bit definition
  */
#define  TIM_CTR2_IVO4              ((uint16_t)0x4000)      /*!< Idle value output of channel 4. */
#define  TIM_CTR2_IVO3N             ((uint16_t)0x2000)      /*!< Idle value output of channel 3 complementary. */
#define  TIM_CTR2_IVO3              ((uint16_t)0x1000)      /*!< Idle value output of channel 3. */
#define  TIM_CTR2_IVO2N             ((uint16_t)0x0800)      /*!< Idle value output of channel 2 complementary . */
#define  TIM_CTR2_IVO2              ((uint16_t)0x0400)      /*!< Idle value output of channel 2. */
#define  TIM_CTR2_IVO1N             ((uint16_t)0x0200)      /*!< Idle value output of channel 1 complementary . */
#define  TIM_CTR2_IVO1              ((uint16_t)0x0100)      /*!< Idle value output of channel 1. */
#define  TIM_CTR2_TI1XOR            ((uint16_t)0x0080)      /*!< XOR input for TI1 . */
#define  TIM_CTR2_MMTOC             ((uint16_t)0x0070)      /*!< Master mode trigger out control. */
#define  TIM_CTR2_CHDMARS           ((uint16_t)0x0008)      /*!< DMA request source of channel. */
#define  TIM_CTR2_CHPUS             ((uint16_t)0x0004)      /*!< Preload update source of channel. */
#define  TIM_CTR2_CHPSEN            ((uint16_t)0x0001)      /*!< Preload shadow enable of channel . */

/**
  * @brief TIM_SMCFG register Bit definition
  */
#define  TIM_SMCFG_ETRINV           ((uint16_t)0x8000)      /*!< ETR invert. */
#define  TIM_SMCFG_ECMODE2          ((uint16_t)0x4000)      /*!< External clock mode 2 enable. */
#define  TIM_SMCFG_ETPDIV           ((uint16_t)0x3000)      /*!< External trigger pre-divide. */
#define  TIM_SMCFG_ETFLT            ((uint16_t)0x0F00)      /*!< External trigger filter. */
#define  TIM_SMCFG_MSM              ((uint16_t)0x0080)      /*!< Master/slave mode. */
#define  TIM_SMCFG_TRIGS            ((uint16_t)0x0070)      /*!< Trigger source selection. */
#define  TIM_SMCFG_SMCFG            ((uint16_t)0x0007)      /*!< Slave mode configuration. */

/**
  * @brief TIM_DIEN register Bit definition
  */
#define  TIM_DIEN_TDREN             ((uint16_t)0x4000)      /*!< Enable bit for trigger DMA request. */
#define  TIM_DIEN_COMDEN            ((uint16_t)0x2000)      /*!< Enable bit for COM event DMA request. */
#define  TIM_DIEN_CH4DEN            ((uint16_t)0x1000)      /*!< Enable bit for channel 4 DMA request. */
#define  TIM_DIEN_CH3DEN            ((uint16_t)0x0800)      /*!< Enable bit for channel 3 DMA request. */
#define  TIM_DIEN_CH2DEN            ((uint16_t)0x0400)      /*!< Enable bit for channel 2 DMA request. */
#define  TIM_DIEN_CH1DEN            ((uint16_t)0x0200)      /*!< Enable bit for channel 1 DMA request. */
#define  TIM_DIEN_UPDEN             ((uint16_t)0x0100)      /*!< Enable bit for update DMA request. */
#define  TIM_DIEN_BRKINTEN          ((uint16_t)0x0080)      /*!< Enable bit for break interrupt. */
#define  TIM_DIEN_TINTEN            ((uint16_t)0x0040)      /*!< Enable bit for trigger interrupt. */
#define  TIM_DIEN_COMINTEN          ((uint16_t)0x0020)      /*!< Enable bit for COM event interrupt. */
#define  TIM_DIEN_CH4INTEN          ((uint16_t)0x0010)      /*!< Enable bit for channel 4 interrupt. */
#define  TIM_DIEN_CH3INTEN          ((uint16_t)0x0008)      /*!< Enable bit for channel 3 interrupt. */
#define  TIM_DIEN_CH2INTEN          ((uint16_t)0x0004)      /*!< Enable bit for channel 2 interrupt. */
#define  TIM_DIEN_CH1INTEN          ((uint16_t)0x0002)      /*!< Enable bit for channel 1 interrupt. */
#define  TIM_DIEN_UPINTEN           ((uint16_t)0x0001)      /*!< Enable bit for update interrupt. */

/**
  * @brief TIM_STS register Bit definition
  */
#define  TIM_STS_CH4ICOF            ((uint16_t)0x1000)      /*!< Channel 4 input capture overflow. */
#define  TIM_STS_CH3ICOF            ((uint16_t)0x0800)      /*!< Channel 3 input capture overflow. */
#define  TIM_STS_CH2ICOF            ((uint16_t)0x0400)      /*!< Channel 2 input capture overflow. */
#define  TIM_STS_CH1ICOF            ((uint16_t)0x0200)      /*!< Channel 1 input capture overflow. */
#define  TIM_STS_BRKIF              ((uint16_t)0x0080)      /*!< Interrupt flag of break. */
#define  TIM_STS_TRIGIF             ((uint16_t)0x0040)      /*!< Interrupt flag of trigger. */
#define  TIM_STS_CHCOMIF            ((uint16_t)0x0020)      /*!< channel communication event interrupt flag. */
#define  TIM_STS_CH4CCIF            ((uint16_t)0x0010)      /*!< Channel 4 capture compare interrupt flag. */
#define  TIM_STS_CH3CCIF            ((uint16_t)0x0008)      /*!< Channel 3 capture compare interrupt flag. */
#define  TIM_STS_CH2CCIF            ((uint16_t)0x0004)      /*!< Channel 2 capture compare interrupt flag. */
#define  TIM_STS_CH1CCIF            ((uint16_t)0x0002)      /*!< Channel 1 capture compare interrupt flag. */
#define  TIM_STS_UPIF               ((uint16_t)0x0001)      /*!< Interrupt flag of update. */

/**
  * @brief TIM_SWEGR register Bit definition
  */
#define  TIM_SWEGR_BRKEG            ((uint16_t)0x0080)      /*!< Break event generation. */
#define  TIM_SWEGR_TRIGEG           ((uint16_t)0x0040)      /*!< Trigger event generation. */
#define  TIM_SWEGR_COMEG            ((uint16_t)0x0020)      /*!< COM event generation. */
#define  TIM_SWEGR_CH4CCG           ((uint16_t)0x0010)      /*!< Channel 4 capture compare event generation. */
#define  TIM_SWEGR_CH3CCG           ((uint16_t)0x0008)      /*!< Channel 3 capture compare event generation. */
#define  TIM_SWEGR_CH2CCG           ((uint16_t)0x0004)      /*!< Channel 2 capture compare event generation. */
#define  TIM_SWEGR_CH1CCG           ((uint16_t)0x0002)      /*!< Channel 1 capture compare event generation. */
#define  TIM_SWEGR_UEG              ((uint16_t)0x0001)      /*!< Update event generation. */

/**
  * @brief TIM_CH12CFGR register Bit definition
  */
#define  TIM_CH12CFGR_CH2OCCEN      ((uint16_t)0x8000)      /*!< Channel 2 output compare clear enable. */
#define  TIM_CH12CFGR_CH2OCMSEL     ((uint16_t)0x7000)      /*!< Channel 2 output compare mode selection. */
#define  TIM_CH12CFGR_CH2OCVPEN     ((uint16_t)0x0800)      /*!< Channel 2 output compare value preload enable. */
#define  TIM_CH12CFGR_CH2OCFEN      ((uint16_t)0x0400)      /*!< Channel 2 output compare fast enable. */
#define  TIM_CH12CFGR_CH2FS         ((uint16_t)0x0300)      /*!< Channel 2 function selection. */
#define  TIM_CH12CFGR_CH1OCCEN      ((uint16_t)0x0080)      /*!< Channel 1 output compare clear enable. */
#define  TIM_CH12CFGR_CH1OCMSEL     ((uint16_t)0x0070)      /*!< Channel 1 output compare mode selection. */
#define  TIM_CH12CFGR_CH1OCVPEN     ((uint16_t)0x0008)      /*!< Channel 1 output compare value preload enable. */
#define  TIM_CH12CFGR_CH1OCFEN      ((uint16_t)0x0004)      /*!< Channel 1 output compare fast enable. */
#define  TIM_CH12CFGR_CH1FS         ((uint16_t)0x0003)      /*!< Channel 1 function selection. */

#define  TIM_CH12CFGR_CH2ICFLT      ((uint16_t)0xF000)      /*!< Channel 2 input capture filter. */
#define  TIM_CH12CFGR_CH2ICPDIV     ((uint16_t)0x0C00)      /*!< Channel 2 input capture pre-divide. */
#define  TIM_CH12CFGR_CH2FS         ((uint16_t)0x0300)      /*!< Channel 2 function selection. */
#define  TIM_CH12CFGR_CH1ICFLT      ((uint16_t)0x00F0)      /*!< Channel 1 input capture filter. */
#define  TIM_CH12CFGR_CH1ICPDIV     ((uint16_t)0x000C)      /*!< Channel 1 input capture pre-divide. */
#define  TIM_CH12CFGR_CH1FS         ((uint16_t)0x0003)      /*!< Channel 1 function selection. */

/**
  * @brief TIM_CH34CFGR register Bit definition
  */
#define  TIM_CH34CFGR_CH4OCCEN      ((uint16_t)0x8000)      /*!< Channel 4 output compare clear enable. */
#define  TIM_CH34CFGR_CH4OCMSEL     ((uint16_t)0x7000)      /*!< Channel 4 output compare mode selection. */
#define  TIM_CH34CFGR_CH4OCVPEN     ((uint16_t)0x0800)      /*!< Channel 4 output compare value preload enable. */
#define  TIM_CH34CFGR_CH4OCFEN      ((uint16_t)0x0400)      /*!< Channel 4 output compare fast enable. */
#define  TIM_CH34CFGR_CH4FS         ((uint16_t)0x0300)      /*!< Channel 4 function selection. */
#define  TIM_CH34CFGR_CH3OCCEN      ((uint16_t)0x0080)      /*!< Channel 3 output compare clear enable. */
#define  TIM_CH34CFGR_CH3OCMSEL     ((uint16_t)0x0070)      /*!< Channel 3 output compare mode selection. */
#define  TIM_CH34CFGR_CH3OCVPEN     ((uint16_t)0x0008)      /*!< Channel 3 output compare value preload enable. */
#define  TIM_CH34CFGR_CH3OCFEN      ((uint16_t)0x0004)      /*!< Channel 3 output compare fast enable. */
#define  TIM_CH34CFGR_CH3FS         ((uint16_t)0x0003)      /*!< Channel 3 function selection. */

#define  TIM_CH34CFGR_CH4ICFLT      ((uint16_t)0xF000)      /*!< Channel 4 input capture filter. */
#define  TIM_CH34CFGR_CH4ICPDIV     ((uint16_t)0x0C00)      /*!< Channel 4 input capture pre-divide. */
#define  TIM_CH34CFGR_CH4FS         ((uint16_t)0x0300)      /*!< Channel 4 function selection. */
#define  TIM_CH34CFGR_CH3ICFLT      ((uint16_t)0x00F0)      /*!< Channel 3 input capture filter. */
#define  TIM_CH34CFGR_CH3ICPDIV     ((uint16_t)0x000C)      /*!< Channel 3 input capture pre-divide. */
#define  TIM_CH34CFGR_CH3FS         ((uint16_t)0x0003)      /*!< Channel 3 function selection. */

/**
  * @brief TIM_CCCTR register Bit definition
  */
#define  TIM_CCCTR_CH4CCP           ((uint16_t)0x2000)      /*!< Channel 4 capture compare polarity. */
#define  TIM_CCCTR_CH4CCEN          ((uint16_t)0x1000)      /*!< Channel 4 capture compare enable. */
#define  TIM_CCCTR_CH3NCCP          ((uint16_t)0x0800)      /*!< Channel 3 complementary capture compare  polarity. */
#define  TIM_CCCTR_CH3NCCEN         ((uint16_t)0x0400)      /*!< Channel 3 complementary capture compare  enable. */
#define  TIM_CCCTR_CH3CCP           ((uint16_t)0x0200)      /*!< Channel 3 capture compare polarity. */
#define  TIM_CCCTR_CH3CCEN          ((uint16_t)0x0100)      /*!< Channel 3 capture compare enable. */
#define  TIM_CCCTR_CH2NCCP          ((uint16_t)0x0080)      /*!< Channel 2 complementary capture compare  polarity. */
#define  TIM_CCCTR_CH2NCCEN         ((uint16_t)0x0040)      /*!< Channel 2 complementary capture compare  enable. */
#define  TIM_CCCTR_CH2CCP           ((uint16_t)0x0020)      /*!< Channel 2 capture compare polarity. */
#define  TIM_CCCTR_CH2CCEN          ((uint16_t)0x0010)      /*!< Channel 2 capture compare enable. */
#define  TIM_CCCTR_CH1NCCP          ((uint16_t)0x0008)      /*!< Channel 1 complementary capture compare  polarity. */
#define  TIM_CCCTR_CH1NCCEN         ((uint16_t)0x0004)      /*!< Channel 1 complementary capture compare  enable. */
#define  TIM_CCCTR_CH1CCP           ((uint16_t)0x0002)      /*!< Channel 1 capture compare polarity. */
#define  TIM_CCCTR_CH1CCEN          ((uint16_t)0x0001)      /*!< Channel 1 capture compare enable. */

/**
  * @brief TIM_CHOPR register Bit definition
  */
#define  TIM_CHOPR_CHOPEN           ((uint16_t)0x8000)      /*!< Channel output pad enable. */
#define  TIM_CHOPR_CHOPAEN          ((uint16_t)0x4000)      /*!< Channel output pad auto enable. */
#define  TIM_CHOPR_BRKPOL           ((uint16_t)0x2000)      /*!< Break polarity. */
#define  TIM_CHOPR_BRKEN            ((uint16_t)0x1000)      /*!< Break enable. */
#define  TIM_CHOPR_RUNOS            ((uint16_t)0x0800)      /*!< Run mode off-state control. */
#define  TIM_CHOPR_IDLEOS           ((uint16_t)0x0400)      /*!< Idle mode off-state control. */
#define  TIM_CHOPR_LCKLV            ((uint16_t)0x0300)      /*!< Lock level control. */
#define  TIM_CHOPR_DTCFG            ((uint16_t)0x00FF)      /*!< Dead-time configuration. */

/**
  * @brief TIM_DMAACR register Bit definition
  */
#define  TIM_DMAACR_DMATL           ((uint16_t)0x1F00)
#define  TIM_DMAACR_DMASA           ((uint16_t)0x001F)

/******************************************************************************/
/*                                                                            */
/*                             Real-Time Clock                                */
/*                                                                            */
/******************************************************************************/

/**
  * @brief RTC_ICTR register Bit definition
  */
#define  RTC_ICTR_OVRIE             ((uint32_t)0x00000004)      /*!< Overflow interrupt enable. */
#define  RTC_ICTR_ALRIE             ((uint32_t)0x00000002)      /*!< Alarm interrupt enable. */
#define  RTC_ICTR_SCIE              ((uint32_t)0x00000001)      /*!< Second interrupt enable. */

/**
  * @brief RTC_STS register Bit definition
  */
#define  RTC_STS_RTCOCF             ((uint32_t)0x00000020)      /*!< RTC operation complete flag. */
#define  RTC_STS_CMF                ((uint32_t)0x00000010)      /*!< Configuration mode flag. */
#define  RTC_STS_RSF                ((uint32_t)0x00000008)      /*!< Register synchronized flag. */
#define  RTC_STS_OVRF               ((uint32_t)0x00000004)      /*!< Overflow flag. */
#define  RTC_STS_ALRF               ((uint32_t)0x00000002)      /*!< Alarm flag. */
#define  RTC_STS_SCF                ((uint32_t)0x00000001)      /*!< Second flag. */

/**
  * @brief RTC_PDIVLH register Bit definition
  */
#define  RTC_PDIVLH_PDIVL           ((uint32_t)0x0000000F)      /*!< RTC pre-divider reload value high. */

/**
  * @brief RTC_PDIVLL register Bit definition
  */
#define  RTC_PDIVLL_PDIVL           ((uint32_t)0x0000FFFF)      /*!< RTC pre-divider reload value low. */

/**
  * @brief RTC_PDIVH register Bit definition
  */
#define  RTC_PDIVH_PDIV             ((uint32_t)0x0000000F)      /*!< RTC pre-divider value high. */

/**
  * @brief RTC_PDIVL register Bit definition
  */
#define  RTC_PDIVL_PDIV             ((uint32_t)0x0000FFFF)      /*!< RTC pre-divider value low. */

/**
  * @brief RTC_CNTRH register Bit definition
  */
#define  RTC_CNTRH_CNT              ((uint32_t)0x0000FFFF)      /*!< RTC counter value high. */

/**
  * @brief RTC_CNTRL register Bit definition
  */
#define  RTC_CNTRL_CNT              ((uint32_t)0x0000FFFF)      /*!< RTC counter value low. */

/**
  * @brief RTC_ALRH register Bit definition
  */
#define  RTC_ALRH_ALR               ((uint32_t)0x0000FFFF)      /*!< RTC alarm value high. */

/**
  * @brief RTC_ALRL register Bit definition
  */
#define  RTC_ALRL_ALR               ((uint32_t)0x0000FFFF)      /*!< RTC alarm value low. */

/******************************************************************************/
/*                                                                            */
/*                      FWDT(Free Watchdog Timer)                             */
/*                                                                            */
/******************************************************************************/

/**
  * @brief FWDT_CCODE register Bit definition
  */
#define  FWDT_CCODE_CCODE                      ((uint16_t)0xFFFF)     /*!< Control code (write only, read 0000h) */

/**
  * @brief FWDT_PDIV register Bit definition
  */
#define  FWDT_PDIV_PDIV                        ((uint8_t)0x07)        /*!< PDIV[2:0] (Pre-divider) */
#define  FWDT_PDIV_PDIV_0                      ((uint8_t)0x01)        /*!< Bit 0 */
#define  FWDT_PDIV_PDIV_1                      ((uint8_t)0x02)        /*!< Bit 1 */
#define  FWDT_PDIV_PDIV_2                      ((uint8_t)0x04)        /*!< Bit 2 */

/**
  * @brief FWDT_UVAL register Bit definition
  */
#define  FWDT_UVAL_RL                          ((uint16_t)0x0FFF)     /*!< Watchdog counter update value */

/**
  * @brief FWDT_STS register Bit definition
  */
#define  FWDT_STS_DRF                          ((uint8_t)0x01)        /*!< FWDT divider refresh flag */
#define  FWDT_STS_UVRF                         ((uint8_t)0x02)        /*!< FWDT update value refresh flag */

/******************************************************************************/
/*                                                                            */
/*                       WWDT(Window Watchdog Timer)                          */
/*                                                                            */
/******************************************************************************/

/**
  * @brief WWDT_CTR1 register Bit definition
  */
#define  WWDT_CTR1_CCODE                    ((uint8_t)0x7F)          /*!< CCODE[6:0] bits (7-Bit counter (MSB to LSB)) */
#define  WWDT_CTR1_CCODE0                   ((uint8_t)0x01)          /*!< Bit 0 */
#define  WWDT_CTR1_CCODE1                   ((uint8_t)0x02)          /*!< Bit 1 */
#define  WWDT_CTR1_CCODE2                   ((uint8_t)0x04)          /*!< Bit 2 */
#define  WWDT_CTR1_CCODE3                   ((uint8_t)0x08)          /*!< Bit 3 */
#define  WWDT_CTR1_CCODE4                   ((uint8_t)0x10)          /*!< Bit 4 */
#define  WWDT_CTR1_CCODE5                   ((uint8_t)0x20)          /*!< Bit 5 */
#define  WWDT_CTR1_CCODE6                   ((uint8_t)0x40)          /*!< Bit 6 */
#define  WWDT_CTR1_WWDTEN                   ((uint8_t)0x80)          /*!< Window watchdog timer enable */

/**
  * @brief WWDT_CTR2 register Bit definition
  */
#define  WWDT_CTR2_WVAL                     ((uint16_t)0x007F)       /*!< WVAL[6:0] bits (7-bit window value) */
#define  WWDT_CTR2_WVAL0                    ((uint16_t)0x0001)       /*!< Bit 0 */
#define  WWDT_CTR2_WVAL1                    ((uint16_t)0x0002)       /*!< Bit 1 */
#define  WWDT_CTR2_WVAL2                    ((uint16_t)0x0004)       /*!< Bit 2 */
#define  WWDT_CTR2_WVAL3                    ((uint16_t)0x0008)       /*!< Bit 3 */
#define  WWDT_CTR2_WVAL4                    ((uint16_t)0x0010)       /*!< Bit 4 */
#define  WWDT_CTR2_WVAL5                    ((uint16_t)0x0020)       /*!< Bit 5 */
#define  WWDT_CTR2_WVAL6                    ((uint16_t)0x0040)       /*!< Bit 6 */

#define  WWDT_CTR2_PDIV                     ((uint16_t)0x0180)       /*!< PDIV[1:0] bits (Pre-divider) */
#define  WWDT_CTR2_PDIV0                    ((uint16_t)0x0080)       /*!< Bit 0 */
#define  WWDT_CTR2_PDIV1                    ((uint16_t)0x0100)       /*!< Bit 1 */
#define  WWDT_CTR2_RMDIE                    ((uint16_t)0x0200)       /*!< Reminder interrupt enable */

/**
  * @brief WWDT_STS register Bit definition
  */
#define  WWDT_STS_RMDIF                        ((uint8_t)0x01)               /*!< Reminder interrupt flag */

/******************************************************************************/
/*                                                                            */
/*                         Controller Area Network                            */
/*                                                                            */
/******************************************************************************/

/**
  * @brief CAN_MCTR register Bit definition
  */
#define  CAN_MCTR_INRQ                        ((uint32_t)0x0001)            /*!< Initialization request */
#define  CAN_MCTR_SLEEP                       ((uint32_t)0x0002)            /*!< Sleep mode */
#define  CAN_MCTR_TXFP                        ((uint32_t)0x0004)            /*!< Transmit FIFO Priority */
#define  CAN_MCTR_RFOWRM                      ((uint32_t)0x0008)            /*!< Receive FIFO Locked Mode */
#define  CAN_MCTR_NAUTORT                     ((uint32_t)0x0010)            /*!< No automatic retransmission */
#define  CAN_MCTR_AUTOATV                     ((uint32_t)0x0020)            /*!< Automatic activation mode */
#define  CAN_MCTR_AUTOFFM                     ((uint32_t)0x0040)            /*!< Automatic bus-off mode */
#define  CAN_MCTR_TTCEN                       ((uint32_t)0x0080)            /*!< Time triggered communication enable */
#define  CAN_MCTR_SOFTRST                     ((uint32_t)0x8000)            /*!< CAN software reset */
#define  CAN_MCTR_DBGW                        ((uint32_t)0x10000)           /*!< Debug wake */

/**
  * @brief CAN_MSTS register Bit definition
  */
#define  CAN_MSTS_INITF                       ((uint32_t)0x0001)            /*!< Initialization flag */
#define  CAN_MSTS_SLEEPF                      ((uint32_t)0x0002)            /*!< Sleep flag */
#define  CAN_MSTS_ERI                         ((uint32_t)0x0004)            /*!< Error interrupt */
#define  CAN_MSTS_WKUI                        ((uint32_t)0x0008)            /*!< Wakeup interrupt */
#define  CAN_MSTS_SEI                         ((uint32_t)0x0010)            /*!< Sleep ensure interrupt */
#define  CAN_MSTS_TXM                         ((uint32_t)0x0100)            /*!< Transmit mode */
#define  CAN_MSTS_RXM                         ((uint32_t)0x0200)            /*!< Receive mode */
#define  CAN_MSTS_LSAMP                       ((uint32_t)0x0400)            /*!< Last sample point on Rx pin */
#define  CAN_MSTS_RPS                         ((uint32_t)0x0800)            /*!< Rx Pin status */

/**
  * @brief CAN_TXSTS register Bit definition
  */
#define  CAN_TXSTS_MRQCP0                    ((uint32_t)0x00000001)        /*!< Mailbox0 Request completed */
#define  CAN_TXSTS_MTXSU0                    ((uint32_t)0x00000002)        /*!< Mailbox0 transmit successful */
#define  CAN_TXSTS_MAL0                      ((uint32_t)0x00000004)        /*!< Mailbox0 arbitration lost */
#define  CAN_TXSTS_MTXERR0                   ((uint32_t)0x00000008)        /*!< mailbox0 Transmission error */
#define  CAN_TXSTS_MATXRQ0                   ((uint32_t)0x00000080)        /*!< mailbox1 Abort transmission request */
#define  CAN_TXSTS_MRQCP1                    ((uint32_t)0x00000100)        /*!< Mailbox1 Request completed */
#define  CAN_TXSTS_MTXSU1                    ((uint32_t)0x00000200)        /*!< Mailbox1 transmit successful */
#define  CAN_TXSTS_MAL1                      ((uint32_t)0x00000400)        /*!< Mailbox1 arbitration lost */
#define  CAN_TXSTS_MTXERR1                   ((uint32_t)0x00000800)        /*!< mailbox1 Transmission error */
#define  CAN_TXSTS_MATXRQ1                   ((uint32_t)0x00008000)        /*!< mailbox1 Abort transmission request */
#define  CAN_TXSTS_MRQCP2                    ((uint32_t)0x00010000)        /*!< Mailbox2 Request completed */
#define  CAN_TXSTS_MTXSU2                    ((uint32_t)0x00020000)        /*!< Mailbox2 transmit successful */
#define  CAN_TXSTS_MAL2                      ((uint32_t)0x00040000)        /*!< Mailbox2 arbitration lost */
#define  CAN_TXSTS_MTXERR2                   ((uint32_t)0x00080000)        /*!< mailbox2 Transmission error */
#define  CAN_TXSTS_MATXRQ2                   ((uint32_t)0x00800000)        /*!< mailbox2 Abort transmission request */
#define  CAN_TXSTS_MNUM                      ((uint32_t)0x03000000)        /*!< Mailbox number */

#define  CAN_TXSTS_TME                       ((uint32_t)0x1C000000)        /*!< TME[2:0] bits */
#define  CAN_TXSTS_TME0                      ((uint32_t)0x04000000)        /*!< transmit mailbox 2 empty flag */
#define  CAN_TXSTS_TME1                      ((uint32_t)0x08000000)        /*!< transmit mailbox 1 empty flag */
#define  CAN_TXSTS_TME2                      ((uint32_t)0x10000000)        /*!< transmit mailbox 2 empty flag */

#define  CAN_TXSTS_LOW                       ((uint32_t)0xE0000000)        /*!< LOW[2:0] bits */
#define  CAN_TXSTS_LOW0                      ((uint32_t)0x20000000)        /*!< Lowest priority flag for mailbox 0 */
#define  CAN_TXSTS_LOW1                      ((uint32_t)0x40000000)        /*!< Lowest priority flag for mailbox 1 */
#define  CAN_TXSTS_LOW2                      ((uint32_t)0x80000000)        /*!< Lowest priority flag for mailbox 2 */

/**
  * @brief CAN_RXFF0 register Bit definition
  */
#define  CAN_RXFF0_FIFOMP0                   ((uint8_t)0x03)               /*!< FIFO0 message pengding */
#define  CAN_RXFF0_FIFOF0                    ((uint8_t)0x08)               /*!< FIFO 0 full */
#define  CAN_RXFF0_FIFOVR0                   ((uint8_t)0x10)               /*!< FIFO 0 overrun */
#define  CAN_RXFF0_RFOM0                     ((uint8_t)0x20)               /*!< Release FIFO0 output mailbox */

/**
  * @brief CAN_RXFF1 register Bit definition
  */
#define  CAN_RXFF1_FIFOMP1                   ((uint8_t)0x03)               /*!< FIFO0 message pengding */
#define  CAN_RXFF1_FIFOF1                    ((uint8_t)0x08)               /*!< FIFO 1 Full */
#define  CAN_RXFF1_FIFOVR1                   ((uint8_t)0x10)               /*!< FIFO 1 Overrun */
#define  CAN_RXFF1_RFOM1                     ((uint8_t)0x20)               /*!< Release FIFO 1 output mailbox */

/**
  * @brief CAN_INTR register Bit definition
  */
#define  CAN_INTR_TXEIE                      ((uint32_t)0x00000001)        /*!< Transmit empty interrupt enable of mailbox */
#define  CAN_INTR_FFMPIE0                    ((uint32_t)0x00000002)        /*!< FIFO0 message pending interrupt enable */
#define  CAN_INTR_FFUIE0                     ((uint32_t)0x00000004)        /*!< FIFO0 full interrupt enable */
#define  CAN_INTR_FFOVRIE0                   ((uint32_t)0x00000008)        /*!< FIFO0 overrun interrupt enable */
#define  CAN_INTR_FFMPIE1                    ((uint32_t)0x00000010)        /*!< FIFO1 message pending interrupt enable */
#define  CAN_INTR_FFUIE1                     ((uint32_t)0x00000020)        /*!< FIFO1 full interrupt enable */
#define  CAN_INTR_FFOVRIE1                   ((uint32_t)0x00000040)        /*!< FIFO1 overrun interrupt enable */
#define  CAN_INTR_EWIE                       ((uint32_t)0x00000100)        /*!< Error warning interrupt enable */
#define  CAN_INTR_EPSSIE                     ((uint32_t)0x00000200)        /*!< Error passive interrupt enable */
#define  CAN_INTR_BOFIE                      ((uint32_t)0x00000400)        /*!< Bus-off interrupt enable */
#define  CAN_INTR_LENIE                      ((uint32_t)0x00000800)        /*!< Last error number interrupt enable */
#define  CAN_INTR_ERRIE                      ((uint32_t)0x00008000)        /*!< Error interrupt enable */
#define  CAN_INTR_WKUPIE                     ((uint32_t)0x00010000)        /*!< Wakeup interrupt enable */
#define  CAN_INTR_SLEEPIE                    ((uint32_t)0x00020000)        /*!< Sleep interrupt enable */

/**
  * @brief CAN_ERRSTS register Bit definition
  */
#define  CAN_ERRSTS_ERRWF                    ((uint32_t)0x00000001)        /*!< Error warning flag */
#define  CAN_ERRSTS_ERRPF                    ((uint32_t)0x00000002)        /*!< Error passive flag */
#define  CAN_ERRSTS_BOFF                     ((uint32_t)0x00000004)        /*!< Bus-off flag */

#define  CAN_ERRSTS_LEC                      ((uint32_t)0x00000070)        /*!< LEC[2:0] bits (Last error code) */
#define  CAN_ERRSTS_LEC_0                    ((uint32_t)0x00000010)        /*!< Bit 0 */
#define  CAN_ERRSTS_LEC_1                    ((uint32_t)0x00000020)        /*!< Bit 1 */
#define  CAN_ERRSTS_LEC_2                    ((uint32_t)0x00000040)        /*!< Bit 2 */

#define  CAN_ERRSTS_TEC                      ((uint32_t)0x00FF0000)        /*!< Least significant byte of the 9-bit transmit error counter */
#define  CAN_ERRSTS_RFC                      ((uint32_t)0xFF000000)        /*!< Receive failed counter */

/**
  * @brief CAN_BTM register Bit definition
  */
#define  CAN_BTR_BRP                         ((uint32_t)0x000003FF)        /*!< Baud Rate Prescaler */
#define  CAN_BTR_TS1                         ((uint32_t)0x000F0000)        /*!< Time Segment 1 */
#define  CAN_BTR_TS2                         ((uint32_t)0x00700000)        /*!< Time Segment 2 */
#define  CAN_BTR_SJW                         ((uint32_t)0x03000000)        /*!< Resynchronization Jump Width */
#define  CAN_BTR_LBKM                        ((uint32_t)0x40000000)        /*!< Loop Back Mode (Debug) */
#define  CAN_BTR_SILM                        ((uint32_t)0x80000000)        /*!< Silent Mode */

/**
  * @brief CAN_TXMI0 register Bit definition
  */
#define  CAN_TXMI0_TRQ                       ((uint32_t)0x00000001)        /*!< Transmit mailbox request */
#define  CAN_TXMI0_RTRQ                      ((uint32_t)0x00000002)        /*!< Remote transmission request */
#define  CAN_TXMI0_SESEL                     ((uint32_t)0x00000004)        /*!< Standard or extended identifier select */
#define  CAN_TXMI0_EXTID                     ((uint32_t)0x001FFFF8)        /*!< Extended identifier */
#define  CAN_TXMI0_STDID                     ((uint32_t)0xFFE00000)        /*!< Standard identifier or extended identifier */

/**
  * @brief CAN_TXLT0 register Bit definition
  */
#define  CAN_TXLT0_TDL                       ((uint32_t)0x0000000F)        /*!< Transmit data length */
#define  CAN_TXLT0_TTS                       ((uint32_t)0x00000100)        /*!< Transmit Time stamp */
#define  CAN_TXLT0_TIMES                     ((uint32_t)0xFFFF0000)        /*!< Time stamp */

/**
  * @brief CAN_TXDL0 register Bit definition
  */
#define  CAN_TXDL0_DATA0                     ((uint32_t)0x000000FF)        /*!< Data byte 0 */
#define  CAN_TXDL0_DATA1                     ((uint32_t)0x0000FF00)        /*!< Data byte 1 */
#define  CAN_TXDL0_DATA2                     ((uint32_t)0x00FF0000)        /*!< Data byte 2 */
#define  CAN_TXDL0_DATA3                     ((uint32_t)0xFF000000)        /*!< Data byte 3 */

/**
  * @brief CAN_TXDH0 register Bit definition
  */
#define  CAN_TXDL0_DATA4                     ((uint32_t)0x000000FF)        /*!< Data byte 4 */
#define  CAN_TXDL0_DATA5                     ((uint32_t)0x0000FF00)        /*!< Data byte 5 */
#define  CAN_TXDL0_DATA6                     ((uint32_t)0x00FF0000)        /*!< Data byte 6 */
#define  CAN_TXDL0_DATA7                     ((uint32_t)0xFF000000)        /*!< Data byte 7 */

/**
  * @brief CAN_TXMI1 register Bit definition
  */
#define  CAN_TXMI1_TRQ                       ((uint32_t)0x00000001)        /*!< Transmit mailbox request */
#define  CAN_TXMI1_RTRQ                      ((uint32_t)0x00000002)        /*!< Remote transmission request */
#define  CAN_TXMI1_SESEL                     ((uint32_t)0x00000004)        /*!< Standard or extended identifier select */
#define  CAN_TXMI1_EXTID                     ((uint32_t)0x001FFFF8)        /*!< Extended identifier */
#define  CAN_TXMI1_STDID                     ((uint32_t)0xFFE00000)        /*!< Standard identifier or extended identifier */

/**
  * @brief CAN_TXLT1 register Bit definition
  */
#define  CAN_TXLT1_TDL                       ((uint32_t)0x0000000F)        /*!< Transmit data length */
#define  CAN_TXLT1_TTS                       ((uint32_t)0x00000100)        /*!< Transmit Time stamp */
#define  CAN_TXLT1_TIMES                     ((uint32_t)0xFFFF0000)        /*!< Time stamp */

/**
  * @brief CAN_TXDL1 register Bit definition
  */
#define  CAN_TXDL1_DATA0                     ((uint32_t)0x000000FF)        /*!< Data byte 0 */
#define  CAN_TXDL1_DATA1                     ((uint32_t)0x0000FF00)        /*!< Data byte 1 */
#define  CAN_TXDL1_DATA2                     ((uint32_t)0x00FF0000)        /*!< Data byte 2 */
#define  CAN_TXDL1_DATA3                     ((uint32_t)0xFF000000)        /*!< Data byte 3 */

/**
  * @brief CAN_TXDH1 register Bit definition
  */
#define  CAN_TXDH1_DATA4                     ((uint32_t)0x000000FF)        /*!< Data byte 4 */
#define  CAN_TXDH1_DATA5                     ((uint32_t)0x0000FF00)        /*!< Data byte 5 */
#define  CAN_TXDH1_DATA6                     ((uint32_t)0x00FF0000)        /*!< Data byte 6 */
#define  CAN_TXDH1_DATA7                     ((uint32_t)0xFF000000)        /*!< Data byte 7 */

/**
  * @brief CAN_TXMI2 register Bit definition
  */
#define  CAN_TXMI2_TRQ                       ((uint32_t)0x00000001)        /*!< Transmit mailbox request */
#define  CAN_TXMI2_RTRQ                      ((uint32_t)0x00000002)        /*!< Remote transmission request */
#define  CAN_TXMI2_SESEL                     ((uint32_t)0x00000004)        /*!< Standard or extended identifier select */
#define  CAN_TXMI2_EXTID                     ((uint32_t)0x001FFFF8)        /*!< Extended identifier */
#define  CAN_TXMI2_STDID                     ((uint32_t)0xFFE00000)        /*!< Standard identifier or extended identifier */

/**
  * @brief CAN_TXLT2 register Bit definition
  */
#define  CAN_TXLT2_TDL                       ((uint32_t)0x0000000F)        /*!< Transmit data length */
#define  CAN_TXLT2_TTS                       ((uint32_t)0x00000100)        /*!< Transmit Time stamp */
#define  CAN_TXLT2_TIMES                     ((uint32_t)0xFFFF0000)        /*!< Time stamp */

/**
  * @brief CAN_TXDL2 register Bit definition
  */
#define  CAN_TXDL2_DATA0                     ((uint32_t)0x000000FF)        /*!< Data byte 0 */
#define  CAN_TXDL2_DATA1                     ((uint32_t)0x0000FF00)        /*!< Data byte 1 */
#define  CAN_TXDL2_DATA2                     ((uint32_t)0x00FF0000)        /*!< Data byte 2 */
#define  CAN_TXDL2_DATA3                     ((uint32_t)0xFF000000)        /*!< Data byte 3 */

/**
  * @brief CAN_TXDH2 register Bit definition
  */
#define  CAN_TXDH2_DATA4                     ((uint32_t)0x000000FF)        /*!< Data byte 4 */
#define  CAN_TXDH2_DATA5                     ((uint32_t)0x0000FF00)        /*!< Data byte 5 */
#define  CAN_TXDH2_DATA6                     ((uint32_t)0x00FF0000)        /*!< Data byte 6 */
#define  CAN_TXDH2_DATA7                     ((uint32_t)0xFF000000)        /*!< Data byte 7 */

/**
  * @brief CAN_RFFID0 register Bit definition
  */
#define  CAN_RFFID0_RTRQ                     ((uint32_t)0x00000002)        /*!< Remote transmission request */
#define  CAN_RFFID0_SESEL                    ((uint32_t)0x00000004)        /*!< Standard or extended identifier select */
#define  CAN_RFFID0_EXTID                    ((uint32_t)0x001FFFF8)        /*!< Extended identifier */
#define  CAN_RFFID0_STDID                    ((uint32_t)0xFFE00000)        /*!< Standard identifier or extended identifier */

/**
  * @brief CAN_RXLT0 register Bit definition
  */
#define  CAN_RXLT0_RDL                       ((uint32_t)0x0000000F)        /*!< receive data length */
#define  CAN_RXLT0_ICCTR                     ((uint32_t)0x0000FF00)        /*!< Information choose control */
#define  CAN_RXLT0_TIMES                     ((uint32_t)0xFFFF0000)        /*!< Time stamp */

/**
  * @brief CAN_RXDL0 register Bit definition
  */
#define  CAN_RXDL0_DATA0                     ((uint32_t)0x000000FF)        /*!< Data byte 0 */
#define  CAN_RXDL0_DATA1                     ((uint32_t)0x0000FF00)        /*!< Data byte 1 */
#define  CAN_RXDL0_DATA2                     ((uint32_t)0x00FF0000)        /*!< Data byte 2 */
#define  CAN_RXDL0_DATA3                     ((uint32_t)0xFF000000)        /*!< Data byte 3 */

/**
  * @brief CAN_RXDH0 register Bit definition
  */
#define  CAN_RXDH0_DATA4                     ((uint32_t)0x000000FF)        /*!< Data byte 4 */
#define  CAN_RXDH0_DATA5                     ((uint32_t)0x0000FF00)        /*!< Data byte 5 */
#define  CAN_RXDH0_DATA6                     ((uint32_t)0x00FF0000)        /*!< Data byte 6 */
#define  CAN_RXDH0_DATA7                     ((uint32_t)0xFF000000)        /*!< Data byte 7 */

/**
  * @brief CAN_RFFID1 register Bit definition
  */
#define  CAN_RFFID1_RTRQ                     ((uint32_t)0x00000002)        /*!< Remote transmission request */
#define  CAN_RFFID1_SESEL                    ((uint32_t)0x00000004)        /*!< Standard or extended identifier select */
#define  CAN_RFFID1_EXTID                    ((uint32_t)0x001FFFF8)        /*!< Extended identifier */
#define  CAN_RFFID1_STDID                    ((uint32_t)0xFFE00000)        /*!< Standard identifier or extended identifier */

/**
  * @brief CAN_RXLT1 register Bit definition
  */
#define  CAN_RXLT1_RDL                       ((uint32_t)0x0000000F)        /*!< receive data length */
#define  CAN_RXLT1_ICCTR                     ((uint32_t)0x0000FF00)        /*!< Information choose control */
#define  CAN_RXLT1_TIMES                     ((uint32_t)0xFFFF0000)        /*!< Time stamp */

/**
  * @brief CAN_RXDL1 register Bit definition
  */
#define  CAN_RXDL1_DATA0                     ((uint32_t)0x000000FF)        /*!< Data byte 0 */
#define  CAN_RXDL1_DATA1                     ((uint32_t)0x0000FF00)        /*!< Data byte 1 */
#define  CAN_RXDL1_DATA2                     ((uint32_t)0x00FF0000)        /*!< Data byte 2 */
#define  CAN_RXDL1_DATA3                     ((uint32_t)0xFF000000)        /*!< Data byte 3 */

/**
  * @brief CAN_RXDH1 register Bit definition
  */
#define  CAN_RXDH1_DATA4                     ((uint32_t)0x000000FF)        /*!< Data byte 4 */
#define  CAN_RXDH1_DATA5                     ((uint32_t)0x0000FF00)        /*!< Data byte 5 */
#define  CAN_RXDH1_DATA6                     ((uint32_t)0x00FF0000)        /*!< Data byte 6 */
#define  CAN_RXDH1_DATA7                     ((uint32_t)0xFF000000)        /*!< Data byte 7 */

/**
  * @brief CAN_FLTM register Bit definition
  */
#define  CAN_FLTM_FWKM                       ((uint8_t)0x01)               /*!< Filter init mode */

/**
  * @brief CAN_FMOD register Bit definition
  */
#define  CAN_FMOD_FM                         ((uint16_t)0x3FFF)            /*!< Filter Mode */

/**
  * @brief CAN_FBW register Bit definition
  */
#define  CAN_FBW_FBC                         ((uint16_t)0x3FFF)            /*!< CAN filter bit width configuration */

/**
  * @brief CAN_FLTFA register Bit definition
  */
#define  CAN_FLTFA_FFA                       ((uint16_t)0x3FFF)            /*!< Filter FIFO assignment for filter x */

/**
  * @brief CAN_FAS register Bit definition
  */
#define  CAN_FAS_FAS                       ((uint16_t)0x3FFF)            /*!< Filter activation settings */



/******************************************************************************/
/*                                                                            */
/*                        Serial Peripheral Interface                         */
/*                                                                            */
/******************************************************************************/

/*******************  Bit definition for SPI_CTR1 register  ********************/
#define  SPI_CTR1_CPHA                       ((uint16_t)0x0001)            /*!< Clock Phase */
#define  SPI_CTR1_CPOL                       ((uint16_t)0x0002)            /*!< Clock Polarity */
#define  SPI_CTR1_SPIM                       ((uint16_t)0x0004)            /*!< SPI mode */
#define  SPI_CTR1_CRSEL                      ((uint16_t)0x0038)            /*!< Communication rate selection */
#define  SPI_CTR1_CRSEL_0                    ((uint16_t)0x0008)            /*!< Bit 0 */
#define  SPI_CTR1_CRSEL_1                    ((uint16_t)0x0010)            /*!< Bit 1 */
#define  SPI_CTR1_CRSEL_2                    ((uint16_t)0x0020)            /*!< Bit 2 */
#define  SPI_CTR1_SPIEN                      ((uint16_t)0x0040)            /*!< SPI enable */
#define  SPI_CTR1_LSBF                       ((uint16_t)0x0080)            /*!< LSB first */
#define  SPI_CTR1_NVSWNSSM                   ((uint16_t)0x0100)            /*!< NSS value in software NSS mode */
#define  SPI_CTR1_SWNSSM                     ((uint16_t)0x0200)            /*!< software NSS mode */
#define  SPI_CTR1_ROM                        ((uint16_t)0x0400)            /*!< Receive only mode */
#define  SPI_CTR1_DFLS                       ((uint16_t)0x0800)            /*!< Data frame length select */
#define  SPI_CTR1_NXTCRC                     ((uint16_t)0x1000)            /*!< Next CRC transfer */
#define  SPI_CTR1_CRCEN                      ((uint16_t)0x2000)            /*!< CRC calculation enable */
#define  SPI_CTR1_SBOEN                      ((uint16_t)0x4000)            /*!< Single-wire bidirectional output enable */
#define  SPI_CTR1_SBMODE                     ((uint16_t)0x8000)            /*!< Single-wire bidirectional mode */

/*******************  Bit definition for SPI_CTR2 register  ********************/
#define  SPI_CTR2_DMARXEN                    ((uint16_t)0x0001)            /*!< DMA enable for receive */
#define  SPI_CTR2_DMATXEN                    ((uint16_t)0x0002)            /*!< DMA enable for transmit */
#define  SPI_CTR2_NSSOEN                     ((uint16_t)0x0004)            /*!< NSS output enable */
#define  SPI_CTR2_ERRINTEN                   ((uint16_t)0x0020)            /*!< Enable bit for error interrupt */
#define  SPI_CTR2_RXNEINTEN                  ((uint16_t)0x0040)            /*!< Enable bit for receive Buffer Not Empty Interrupt */
#define  SPI_CTR2_TXEINTEN                   ((uint16_t)0x0080)            /*!< Enable bit for transmit Buffer Empty Interrupt */

/********************  Bit definition for SPI_STS register  ********************/
#define  SPI_STS_RXNE                        ((uint16_t)0x0001)            /*!< Receive buffer not empty */
#define  SPI_STS_TXE                         ((uint16_t)0x0002)            /*!< Transmit buffer empty */
#define  SPI_STS_CRCERR                      ((uint16_t)0x0010)            /*!< CRC error */
#define  SPI_STS_MMERR                       ((uint16_t)0x0020)            /*!< Multi-master error */
#define  SPI_STS_RXOFERR                     ((uint16_t)0x0040)            /*!< Receiver data overflow error */
#define  SPI_STS_BUSY                        ((uint16_t)0x0080)            /*!< Busy */

/********************  Bit definition for SPI_DATA register  ********************/
#define  SPI_DATA_DATA                       ((uint16_t)0xFFFF)            /*!< Transfer data */

/*******************  Bit definition for SPI_CRCPOLYR register  ******************/
#define  SPI_CRCPOLYR_CRCPR                  ((uint16_t)0xFFFF)            /*!< CRC polynomial register */

/******************  Bit definition for SPI_RCRC register  ******************/
#define  SPI_RCRC_RCR                        ((uint16_t)0xFFFF)            /*!< RX CRC register */

/******************  Bit definition for SPI_TCRC register  ******************/
#define  SPI_TCRC_TCR                        ((uint16_t)0xFFFF)            /*!< TX CRC register */


/******************************************************************************/
/*                                                                            */
/*                      Inter-integrated Circuit Interface                    */
/*                                                                            */
/******************************************************************************/

/*******************  Bit definition for I2C_CR1 register  ********************/
/**
 * @brief I2C CTR1 register Bit definition
*/
#define  I2C_CTR1_CLEAR_MASK                ((uint16_t)0xFBF5)     /*!< I2C clear mask */
#define  I2C_CTR1_I2C_SWREST                ((uint16_t)0x8000)     /*!< I2C software reset */
#define  I2C_CTR1_SMBUS_ALERT               ((uint16_t)0x2000)     /*!< I2C SMBus alert */
#define  I2C_CTR1_TRANS_PEC_SET             ((uint16_t)0x1000)     /*!< I2C transimit packet error checking */
#define  I2C_CTR1_I2C_APPOS_SET             ((uint16_t)0x0800)     /*!< I2C acknowledge/PEC Position (for data reception) */
#define  I2C_CTR1_I2C_ACKEN                 ((uint16_t)0x0400)     /*!< I2C acknowledge enable */
#define  I2C_CTR1_I2C_STOPGEN               ((uint16_t)0x0200)     /*!< I2C generate stop condition */
#define  I2C_CTR1_I2C_STARTGEN              ((uint16_t)0x0100)     /*!< I2C generate start condition */
#define  I2C_CTR1_DISSTRETCH_DIS            ((uint16_t)0x0080)     /*!< I2C clock stretching disable (Slave mode) */
#define  I2C_CTR1_I2C_GCEN                  ((uint16_t)0x0040)     /*!< I2C general call enable */
#define  I2C_CTR1_PEC_EN                    ((uint16_t)0x0020)     /*!< I2C PEC enable */
#define  I2C_CTR1_ARP_EN                    ((uint16_t)0x0010)     /*!< I2C ARP enable */
#define  I2C_CTR1_I2CEN                     ((uint16_t)0x0001)     /*!< I2C enable */

/**
 * @brief I2C CTR2 register Bit definition
*/
#define  I2C_CTR2_DMA_LAST_EN               ((uint16_t)0x1000)     /*!< I2C the next DMA transfer will be the last one enable */
#define  I2C_CTR2_DMAEN                     ((uint16_t)0x0800)     /*!< I2C DMA request enable */
#define  I2C_CTR2_BUFITEN                   ((uint16_t)0x0400)     /*!< I2C buffer interrupt enable */
#define  I2C_CTR2_EVTITEN                   ((uint16_t)0x0200)     /*!< I2C event interrupt enable */
#define  I2C_CTR2_ERRITEN                   ((uint16_t)0x0100)     /*!< I2C error interrupt enable */
#define  I2C_CTR2_CLKFREQ                   ((uint16_t)0x003F)     /*!< I2C peripheral clock frequency */

/**
  * @brief I2C SADR2 register Bit definition
  */
#define  I2C_SADR2_ADDR2                    ((uint16_t)0x00FE)     /*!< I2C slave Address 2 */
#define  I2C_SADR2_ADDR2_EN                 ((uint16_t)0x0001)     /*!< I2C address 2 enable */

/******************************************************************************/
/*                                                                            */
/*         Universal Synchronous Asynchronous Receiver Transmitter            */
/*                                                                            */
/******************************************************************************/

/**
  * @brief USART_STS register Bit definition
  */
#define  USART_STS_PERRF                    ((uint32_t)0x00000001)            /*!< Parity Error */
#define  USART_STS_FERRF                    ((uint32_t)0x00000002)            /*!< Framing Error */
#define  USART_STS_NF                       ((uint32_t)0x00000004)            /*!< Noise Error Flag */
#define  USART_STS_OVRERRF                  ((uint32_t)0x00000008)            /*!< OverRun Error */
#define  USART_STS_IDLEF                    ((uint32_t)0x00000010)            /*!< IDLE line detected */
#define  USART_STS_RXNE                     ((uint32_t)0x00000020)            /*!< Read Data Register Not Empty */
#define  USART_STS_TCF                      ((uint32_t)0x00000040)            /*!< Transmission Complete */
#define  USART_STS_TXE                      ((uint32_t)0x00000080)            /*!< Transmit Data Register Empty */
#define  USART_STS_LINBKF                   ((uint32_t)0x00000100)            /*!< LIN Break Detection Flag */
#define  USART_STS_CTSF                     ((uint32_t)0x00000200)            /*!< CTS Flag */

/**
  * @brief USART_DATA register Bit definition
  */
#define  USART_DATA_DATA                    ((uint32_t)0x000001FF)            /*!< Data value */

/**
  * @brief USART_BRT register Bit definition
  */
#define  USART_BRT_FRADIV                   ((uint32_t)0x0000000F)            /*!< Fraction of USARTDIV */
#define  USART_BRT_INTDIV                   ((uint32_t)0x0000FFF0)            /*!< Mantissa of USARTDIV */

/**
  * @brief USART_CTR1 register Bit definition
  */
#define  USART_CTR1_BRKFEN                  ((uint32_t)0x00000001)            /*!< Send Break */
#define  USART_CTR1_WKEN                    ((uint32_t)0x00000002)            /*!< Receiver wakeup */
#define  USART_CTR1_REN                     ((uint32_t)0x00000004)            /*!< Receiver Enable */
#define  USART_CTR1_TEN                     ((uint32_t)0x00000008)            /*!< Transmitter Enable */
#define  USART_CTR1_IDLEIE                  ((uint32_t)0x00000010)            /*!< IDLE Interrupt Enable */
#define  USART_CTR1_RXNEIE                  ((uint32_t)0x00000020)            /*!< RXNE Interrupt Enable */
#define  USART_CTR1_TCIE                    ((uint32_t)0x00000040)            /*!< Transmission Complete Interrupt Enable */
#define  USART_CTR1_TXEIE                   ((uint32_t)0x00000080)            /*!< Transmit Buffer Empty Interrupt Enable */
#define  USART_CTR1_PERRIE                  ((uint32_t)0x00000100)            /*!< Parity Error Interrupt Enable */
#define  USART_CTR1_ODDS                    ((uint32_t)0x00000200)            /*!< Parity Selection */
#define  USART_CTR1_PEN                     ((uint32_t)0x00000400)            /*!< Parity Control Enable */
#define  USART_CTR1_WKUPS                   ((uint32_t)0x00000800)            /*!< Wakeup method */
#define  USART_CTR1_FM                      ((uint32_t)0x00001000)            /*!< Frame length */
#define  USART_CTR1_UEN                     ((uint32_t)0x00002000)            /*!< USART Enable */

/**
  * @brief USART_CTR2 register Bit definition
  */
#define  USART_CTR2_UADR                    ((uint32_t)0x0000000F)            /*!< Address of the USART node */
#define  USART_CTR2_LINBRK11                ((uint32_t)0x00000020)            /*!< LIN Break Detection Length */
#define  USART_CTR2_LBDIE                   ((uint32_t)0x00000040)            /*!< LIN Break Detection Interrupt Enable */
#define  USART_CTR2_CKLEN                   ((uint32_t)0x00000100)            /*!< CK pulse length */
#define  USART_CTR2_CPHA                    ((uint32_t)0x00000200)            /*!< Clock Phase */
#define  USART_CTR2_CPOL                    ((uint32_t)0x00000400)            /*!< Clock Polarity */
#define  USART_CTR2_CKEN                    ((uint32_t)0x00000800)            /*!< CK Enable */
#define  USART_CTR2_STOP                    ((uint32_t)0x00003000)            /*!< STOP[1:0] bits (STOP bits) */
#define  USART_CTR2_LINEN                   ((uint32_t)0x00004000)            /*!< LIN mode enable */


/**
  * @brief USART_CTR3 register Bit definition
  */
#define  USART_CTR3_ERRIE                   ((uint32_t)0x00000001)            /*!< Error Interrupt Enable */
#define  USART_CTR3_IRDAEN                  ((uint32_t)0x00000002)            /*!< IrDA mode Enable */
#define  USART_CTR3_IRMS                    ((uint32_t)0x00000004)            /*!< IrDA Low-Power */
#define  USART_CTR3_HDEN                    ((uint32_t)0x00000008)            /*!< Half-Duplex Selection */
#define  USART_CTR3_SCNACK                  ((uint32_t)0x00000010)            /*!< Smartcard NACK enable */
#define  USART_CTR3_SCEN                    ((uint32_t)0x00000020)            /*!< Smartcard mode enable */
#define  USART_CTR3_RXDMA                   ((uint32_t)0x00000040)            /*!< DMA Enable Receiver */
#define  USART_CTR3_TXDMA                   ((uint32_t)0x00000080)            /*!< DMA Enable Transmitter */
#define  USART_CTR3_RTSEN                   ((uint32_t)0x00000100)            /*!< RTS Enable */
#define  USART_CTR3_CTSEN                   ((uint32_t)0x00000200)            /*!< CTS Enable */
#define  USART_CTR3_CTSIE                   ((uint32_t)0x00000400)            /*!< CTS Interrupt Enable */

/**
  * @brief USART_GTPDIV register Bit definition
  */
#define  USART_GTPDIV_PDIV                  ((uint32_t)0x000000FF)            /*!< PDIV[7:0] bits (Prescaler value) */
#define  USART_GTPDIV_GUDT                  ((uint32_t)0x0000FF00)            /*!< Guard time value */

/*--------------------------------------------------------------------------*/
/*                           Debug module                                   */
/*--------------------------------------------------------------------------*/

/**
  * @brief DBG_ID register Bit definition
  */
#define  DBG_ID_DEV_ID                ((uint32_t)0x00000FFF)       /*!< Device dentifier */

#define  DBG_ID_REV_ID                ((uint32_t)0xFFFF0000)       /*!< REV_ID[5:0] bits (Revision Identifier) */
#define  DBG_ID_REV_ID_0              ((uint32_t)0x00010000)       /*!< Bit 0 */
#define  DBG_ID_REV_ID_1              ((uint32_t)0x00020000)       /*!< Bit 1 */
#define  DBG_ID_REV_ID_2              ((uint32_t)0x00040000)       /*!< Bit 2 */
#define  DBG_ID_REV_ID_3              ((uint32_t)0x00080000)       /*!< Bit 3 */
#define  DBG_ID_REV_ID_4              ((uint32_t)0x00100000)       /*!< Bit 4 */
#define  DBG_ID_REV_ID_5              ((uint32_t)0x00200000)       /*!< Bit 5 */
#define  DBG_ID_REV_ID_6              ((uint32_t)0x00400000)       /*!< Bit 6 */
#define  DBG_ID_REV_ID_7              ((uint32_t)0x00800000)       /*!< Bit 7 */
#define  DBG_ID_REV_ID_8              ((uint32_t)0x01000000)       /*!< Bit 8 */
#define  DBG_ID_REV_ID_9              ((uint32_t)0x02000000)       /*!< Bit 9 */
#define  DBG_ID_REV_ID_10             ((uint32_t)0x04000000)       /*!< Bit 10 */
#define  DBG_ID_REV_ID_11             ((uint32_t)0x08000000)       /*!< Bit 11 */
#define  DBG_ID_REV_ID_12             ((uint32_t)0x10000000)       /*!< Bit 12 */
#define  DBG_ID_REV_ID_13             ((uint32_t)0x20000000)       /*!< Bit 13 */
#define  DBG_ID_REV_ID_14             ((uint32_t)0x40000000)       /*!< Bit 14 */
#define  DBG_ID_REV_ID_15             ((uint32_t)0x80000000)       /*!< Bit 15 */

/**
  * @brief DBG_LPWCFG register Bit definition
  */
#define  DBG_LPWCFG_SLEEP_DBGEN             ((uint32_t)0x00000001)        /*!< Sleep mode debug support  */
#define  DBG_LPWCFG_STOP_DBGEN              ((uint32_t)0x00000002)        /*!< Stop mode debug support */
#define  DBG_LPWCFG_STDBY_DBGEN             ((uint32_t)0x00000004)        /*!< Standby mode debug support */
#define  DBG_LPWCFG_TRACE_IOEN              ((uint32_t)0x00000020)        /*!< Trace IO enable */

#define  DBG_LPWCFG_TRACE_MD                ((uint32_t)0x000000C0)        /*!< Trace pin assignment control */
#define  DBG_LPWCFG_TRACE_MODE_0            ((uint32_t)0x00000040)        /*!< Bit 0 */
#define  DBG_LPWCFG_TRACE_MODE_1            ((uint32_t)0x00000080)        /*!< Bit 1 */

#define  DBG_LPWCFG_FWDT_DBG_PAUSE          ((uint32_t)0x00000100)        /*!< Free watchdog paused when system is in debug mode */
#define  DBG_LPWCFG_WWDT_DBG_PAUSE          ((uint32_t)0x00000200)        /*!< window watchdog paused when system is in debug mode */
#define  DBG_LPWCFG_TIM1_DBG_PAUSE          ((uint32_t)0x00000400)        /*!< TIM1 counter paused when system is in debug mode */
#define  DBG_LPWCFG_TIM2_DBG_PAUSE          ((uint32_t)0x00000800)        /*!< TIM2 counter paused when system is in debug mode */
#define  DBG_LPWCFG_TIM3_DBG_PAUSE          ((uint32_t)0x00001000)        /*!< TIM3 counter paused when system is in debug mode */
#define  DBG_LPWCFG_TIM4_DBG_PAUSE          ((uint32_t)0x00002000)        /*!< TIM4 counter paused when system is in debug mode */
#define  DBG_LPWCFG_CAN1_DBG_PAUSE          ((uint32_t)0x00004000)        /*!< CAN1 paused when system is in debug mode */
#define  DBG_LPWCFG_I2C1_SMBUS_DBG_TMOUT    ((uint32_t)0x00008000)        /*!< I2C1 SMBUS timeout counter  paused when system is in debug mod */
#define  DBG_LPWCFG_I2C2_SMBUS_DBG_TMOUT    ((uint32_t)0x00010000)        /*!< I2C2 SMBUS timeout counter  paused when system is in debug mod */

/*---------------------------------------------------------------------------*/
/*   FMC(FLASH Memory Control) and Option Byes Registers                     */
/*---------------------------------------------------------------------------*/

/**
* @brief FMC_WCR register Bit definition
*/
#define FMC_WCR_WCNT                          ((uint32_t)0x00000007)        /*!< NT bit (FMC wait conuter) */
#define FMC_WCR_WCNT_0                        ((uint32_t)0x00000000)        /*!< FLASH wait 0 cycle */  
#define FMC_WCR_WCNT_1                        ((uint32_t)0x00000001)        /*!< FLASH wati 1 cycle */  
#define FMC_WCR_WCNT_2                        ((uint32_t)0x00000002)        /*!< FLASH wati 2 cycle */  

#define  FMC_WCR_HCAEN                        ((uint32_t)0x00000008)        /*!< Flash half cycle access enable */
#define  FMC_WCR_WE                           ((uint32_t)0x00000010)        /*!< MC wait enable */
#define  FMC_WCR_WS                           ((uint32_t)0x00000020)        /*!< MC wait status(read only) */


/**
* @brief FMC_CCODE register Bit definition
*/
#define  FMC_CCODE_CCODE                      ((uint32_t)0xFFFFFFFF)        /*!< MC Control code */

/**
* @brief FMC_OBCCODE register Bit definition
*/
#define  FMC_OBCCODE_OBCCODE                  ((uint32_t)0xFFFFFFFF)        /*!< FMC Option Byte Control code */

/**
* @brief FMC CCODE Bit definition
*/
#define FMC_CCODE_VALUE1                      ((uint32_t)0x45670123)        /*!< Flash program erase key1 */
#define FMC_CCODE_VALUE2                      ((uint32_t)0xCDEF89AB)        /*!< Flash program erase key2 */
#define FMC_OBCCODE_VALUE1                    ((uint32_t)0x45670123)        /*!< Flash option key1 */
#define FMC_OBCCODE_VALUE2                    ((uint32_t)0xCDEF89AB)        /*!< Flash option key2 */

/**
* @brief FMC_STS register Bit definition
*/
#define  FMC_STS_BUSY                         ((uint32_t)0x00000001)        /*!< Busy */
#define  FMC_STS_PGERR                        ((uint32_t)0x00000004)        /*!< Programming Error */
#define  FMC_STS_WPERR                        ((uint32_t)0x00000010)        /*!< Write Protection Error */
#define  FMC_STS_ENDF                         ((uint32_t)0x00000020)        /*!< End of operation */

/**
* @brief FMC_CTR register Bit definition
*/
#define  FMC_CTR_PG                           ((uint32_t)0x00000001)        /*!< Main flash program */
#define  FMC_CTR_PERS                         ((uint32_t)0x00000002)        /*!< Flash page erase */
#define  FMC_CTR_CHIPERS                      ((uint32_t)0x00000004)        /*!< Mass flash CHIP ERASE */
#define  FMC_CTR_OBPG                         ((uint32_t)0x00000010)        /*!< Option byte program command */
#define  FMC_CTR_OBERS                        ((uint32_t)0x00000020)        /*!< Option bytes erase command */
#define  FMC_CTR_START                        ((uint32_t)0x00000040)        /*!< Start of erase */
#define  FMC_CTR_LOCK                         ((uint32_t)0x00000080)        /*!< Lock */
#define  FMC_CTR_OBWEN                        ((uint32_t)0x00000200)        /*!< Option byte erase/program enable */
#define  FMC_CTR_ERRIE                        ((uint32_t)0x00000400)        /*!< Error interrupt enable */
#define  FMC_CTR_ENDIE                        ((uint32_t)0x00001000)        /*!< End of operation interrupt enable */

/**
  * @brief FMC_ERSADR register Bit definition
  */
#define  FMC_ERSADR_ERSADR                    ((uint32_t)0xFFFFFFFF)        /*!< lash erase address */

/**
* @brief FMC_OBSTS register Bit definition
*/
#define  FMC_OBSTS_OBERR                      ((uint32_t)0x00000001)        /*!< OBERR */
#define  FMC_OBSTS_RPROT                      ((uint32_t)0x00000002)        /*!< Read protection Level bit */
#define  FMC_OBSTS_USER                       ((uint16_t)0x03FC)            /*!<  User Option Bytes */
#define  FMC_OBSTS_FWDT_AO                    ((uint16_t)0x0004)            /*!< FWDT_AO */
#define  FMC_OBSTS_nRST_STOP                  ((uint16_t)0x0008)            /*!< nRST_STOP */
#define  FMC_OBSTS_nRST_STDBY                 ((uint16_t)0x0010)            /*!< nRST_STDBY */

/**
* @brief FMC_WPSTS register Bit definition
*/
#define  FMC_WPSTS_WPROT                      ((uint32_t)0xFFFFFFFF)        /*!< Write Protect */


/**
* @brief OB_RPROT register Bit definition
*/
#define  OB_RPROT_RPROT                       ((uint32_t)0x000000FF)        /*!< Read protection option byte */
#define  OB_RPROT_nRPROT                      ((uint32_t)0x0000FF00)        /*!< Read protection complemented option byte */

/**
* @brief OB_USER register Bit definition
*/
#define  OB_USER_USER                         ((uint32_t)0x00FF0000)        /*!< User option byte */
#define  OB_USER_nUSER                        ((uint32_t)0xFF000000)        /*!< User complemented option byte */

/**
* @brief OB_DATA0 register Bit definition
*/
#define  OB_DATA0_DATA0                   ((uint32_t)0x000000FF)        /*!< Data0 option byte */
#define  OB_DATA0_nDATA0                  ((uint32_t)0x0000FF00)        /*!< Data0 complemented option byte */

/**
* @brief OB_DATA1 register Bit definition
*/
#define  OB_DATA0_DATA1                   ((uint32_t)0x00FF0000)        /*!< Data1 option byte */
#define  OB_DATA0_nDATA1                  ((uint32_t)0xFF000000)        /*!< Data1 complemented option byte */

/**
* @brief OB_WPROT0 register Bit definition
*/
#define  OB_WPROT0_WPROT0                     ((uint32_t)0x000000FF)        /*!< Flash memory write protection option bytes */
#define  OB_WPROT0_nWPROT0                    ((uint32_t)0x0000FF00)        /*!< Flash memory write protection complemented option bytes */

/**
* @brief OB_WPROT1 register Bit definition
*/
#define  OB_WPROT1_WPROT1                     ((uint32_t)0x00FF0000)        /*!< Flash memory write protection option bytes */
#define  OB_WPROT1_nWPROT1                    ((uint32_t)0xFF000000)        /*!< Flash memory write protection complemented option bytes */

/**
* @brief OB_WPROT2 register Bit definition
*/
#define  OB_WPROT2_WPROT2                     ((uint32_t)0x000000FF)        /*!< Flash memory write protection option bytes */
#define  OB_WPROT2_nWPROT2                    ((uint32_t)0x0000FF00)        /*!< Flash memory write protection complemented option bytes */

/**
* @brief OB_WPROT3 register Bit definition
*/
#define  OB_WPROT3_WPROT3                     ((uint32_t)0x00FF0000)        /*!< Flash memory write protection option bytes */
#define  OB_WPROT3_nWPROT3                    ((uint32_t)0xFF000000)        /*!< Flash memory write protection complemented option bytes */


/**
  * @}
  */

 /**
  * @}
  */ 


/** @addtogroup Exported_macro
  * @{
  */

#define SET_BIT(REG, BIT)     ((REG) |= (BIT))

#define CLEAR_BIT(REG, BIT)   ((REG) &= ~(BIT))

#define READ_BIT(REG, BIT)    ((REG) & (BIT))

#define CLEAR_REG(REG)        ((REG) = (0x0))

#define WRITE_REG(REG, VAL)   ((REG) = (VAL))

#define READ_REG(REG)         ((REG))


/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /* end of group __CS32F10X_H__ */


/*** (C) COPYRIGHT Software Development . ***/

